@@ -1,24 +1,23 @@
-/*
- * adpcm.c  codex functions for MS_ADPCM data
+/* adpcm.c  codex functions for MS_ADPCM data
  *          (hopefully) provides interoperability with
  *          Microsoft's ADPCM format, but, as usual,
  *          see LACK-OF-WARRANTY information below.
  *
  *      Copyright (C) 1999 Stanley J. Brooks <stabro@megsinet.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License as published by the Free Software Foundation; either
- *   version 2 of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or (at
+ * your option) any later version.
  *
- *   This library is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *   Lesser General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.
  *
- *   You should have received a copy of the GNU Lesser General Public
- *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -32,15 +31,15 @@
  *
  */
 
-#include <sys/types.h>
-#include <math.h>
-#include <stdio.h>
 #include "sox_i.h"
 #include "adpcm.h"
 
-typedef struct MsState {
+#include <sys/types.h>
+#include <stdio.h>
+
+typedef struct {
         sox_sample_t  step;      /* step size */
-        short iCoef[2];
+        short lsx_ms_adpcm_i_coef[2];
 } MsState_t;
 
 #define lsbshortldi(x,p) { (x)=((short)((int)(p)[0] + ((int)(p)[1]<<8))); (p) += 2; }
@@ -58,11 +57,11 @@ sox_sample_t stepAdjustTable[] = {
         768, 614, 512, 409, 307, 230, 230, 230
 };
 
-/* TODO : The first 7 iCoef sets are always hardcoded and must
+/* TODO : The first 7 lsx_ms_adpcm_i_coef sets are always hardcoded and must
    appear in the actual WAVE file.  They should be read in
    in case a sound program added extras to the list. */
 
-const short iCoef[7][2] = {
+const short lsx_ms_adpcm_i_coef[7][2] = {
                         { 256,   0},
                         { 512,-256},
                         {   0,   0},
@@ -89,8 +88,8 @@ static inline sox_sample_t AdpcmDecode(sox_sample_t c, MsState_t *state,
 
         /** make linear prediction for next sample **/
         vlin =
-                        ((sample1 * state->iCoef[0]) +
-                         (sample2 * state->iCoef[1])) >> 8;
+                        ((sample1 * state->lsx_ms_adpcm_i_coef[0]) +
+                         (sample2 * state->lsx_ms_adpcm_i_coef[1])) >> 8;
         /** then add the code*step adjustment **/
         c -= (c & 0x08) << 1;
         sample = (c * step) + vlin;
@@ -101,11 +100,11 @@ static inline sox_sample_t AdpcmDecode(sox_sample_t c, MsState_t *state,
         return (sample);
 }
 
-/* AdpcmBlockExpandI() outputs interleaved samples into one output buffer */
-const char *AdpcmBlockExpandI(
+/* lsx_ms_adpcm_block_expand_i() outputs interleaved samples into one output buffer */
+const char *lsx_ms_adpcm_block_expand_i(
         unsigned chans,          /* total channels             */
         int nCoef,
-        const short *iCoef,
+        const short *lsx_ms_adpcm_i_coef,
         const unsigned char *ibuff,/* input buffer[blockAlign]   */
         SAMPL *obuff,       /* output samples, n*chans    */
         int n               /* samples to decode PER channel */
@@ -124,8 +123,8 @@ const char *AdpcmBlockExpandI(
                         errmsg = "MSADPCM bpred >= nCoef, arbitrarily using 0\n";
                         bpred = 0;
                 }
-                state[ch].iCoef[0] = iCoef[(int)bpred*2+0];
-                state[ch].iCoef[1] = iCoef[(int)bpred*2+1];
+                state[ch].lsx_ms_adpcm_i_coef[0] = lsx_ms_adpcm_i_coef[(int)bpred*2+0];
+                state[ch].lsx_ms_adpcm_i_coef[1] = lsx_ms_adpcm_i_coef[(int)bpred*2+1];
 
         }
 
@@ -169,7 +168,7 @@ static int AdpcmMashS(
         unsigned ch,              /* channel number to encode, REQUIRE 0 <= ch < chans  */
         unsigned chans,           /* total channels */
         SAMPL v[2],          /* values to use as starting 2 */
-        const short iCoef[2],/* lin predictor coeffs */
+        const short lsx_ms_adpcm_i_coef[2],/* lin predictor coeffs */
         const SAMPL *ibuff,  /* ibuff[] is interleaved input samples */
         int n,               /* samples to encode PER channel */
         int *iostep,         /* input/output step, REQUIRE 16 <= *st <= 0x7fff */
@@ -209,7 +208,7 @@ static int AdpcmMashS(
                 int vlin,d,dp,c;
 
           /* make linear prediction for next sample */
-                vlin = (v0 * iCoef[0] + v1 * iCoef[1]) >> 8;
+                vlin = (v0 * lsx_ms_adpcm_i_coef[0] + v1 * lsx_ms_adpcm_i_coef[1]) >> 8;
                 d = *ip - vlin;  /* difference between linear prediction and current sample */
                 dp = d + (step<<3) + (step>>1);
                 c = 0;
@@ -232,7 +231,7 @@ static int AdpcmMashS(
                 if (op) {   /* if we want output, put it in proper place */
                         op[ox>>3] |= (ox&4)? c:(c<<4);
                         ox += 4*chans;
-                        sox_debug_more("%.1x",c);
+                        lsx_debug_more("%.1x",c);
 
                 }
 
@@ -241,9 +240,9 @@ static int AdpcmMashS(
                 if (step < 16) step = 16;
 
         }
-        if (op) sox_debug_more("\n");
+        if (op) lsx_debug_more("\n");
         d2 /= n; /* be sure it's non-negative */
-        sox_debug_more("ch%d: st %d->%d, d %.1f\n", ch, *iostep, step, sqrt(d2));
+        lsx_debug_more("ch%d: st %d->%d, d %.1f\n", ch, *iostep, step, sqrt(d2));
         *iostep = step;
         return (int) sqrt(d2);
 }
@@ -274,13 +273,13 @@ static inline void AdpcmMashChannel(
         for (k=0; k<7; k++) {
                 int d0,d1;
                 ss = s0 = *st;
-                d0=AdpcmMashS(ch, chans, v, iCoef[k], ip, n, &ss, NULL); /* with step s0 */
+                d0=AdpcmMashS(ch, chans, v, lsx_ms_adpcm_i_coef[k], ip, n, &ss, NULL); /* with step s0 */
 
                 s1 = s0;
-                AdpcmMashS(ch, chans, v, iCoef[k], ip, n0, &s1, NULL);
-                sox_debug_more(" s32 %d\n",s1);
+                AdpcmMashS(ch, chans, v, lsx_ms_adpcm_i_coef[k], ip, n0, &s1, NULL);
+                lsx_debug_more(" s32 %d\n",s1);
                 ss = s1 = (3*s0+s1)/4;
-                d1=AdpcmMashS(ch, chans, v, iCoef[k], ip, n, &ss, NULL); /* with step s1 */
+                d1=AdpcmMashS(ch, chans, v, lsx_ms_adpcm_i_coef[k], ip, n, &ss, NULL); /* with step s1 */
                 if (!k || d0<dmin || d1<dmin) {
                         kmin = k;
                         if (d0<=d1) {
@@ -293,12 +292,12 @@ static inline void AdpcmMashChannel(
                 }
         }
         *st = smin;
-        sox_debug_more("kmin %d, smin %5d, ",kmin,smin);
-        d=AdpcmMashS(ch, chans, v, iCoef[kmin], ip, n, st, obuff);
+        lsx_debug_more("kmin %d, smin %5d, ",kmin,smin);
+        d=AdpcmMashS(ch, chans, v, lsx_ms_adpcm_i_coef[kmin], ip, n, st, obuff);
         obuff[ch] = kmin;
 }
 
-void AdpcmBlockMashI(
+void lsx_ms_adpcm_block_mash_i(
         unsigned chans,          /* total channels */
         const SAMPL *ip,    /* ip[n*chans] is interleaved input samples */
         int n,              /* samples to encode PER channel */
@@ -310,7 +309,7 @@ void AdpcmBlockMashI(
         unsigned ch;
         unsigned char *p;
 
-        sox_debug("AdpcmMashI(chans %d, ip %p, n %d, st %p, obuff %p, bA %d)\n",
+        lsx_debug_more("AdpcmMashI(chans %d, ip %p, n %d, st %p, obuff %p, bA %d)\n",
             chans, (void *)ip, n, (void *)st, obuff, blockAlign);
 
         for (p=obuff+7*chans; p<obuff+blockAlign; p++) *p=0;
@@ -320,20 +319,20 @@ void AdpcmBlockMashI(
 }
 
 /*
- * AdpcmSamplesIn(dataLen, chans, blockAlign, samplesPerBlock)
+ * lsx_ms_adpcm_samples_in(dataLen, chans, blockAlign, samplesPerBlock)
  *  returns the number of samples/channel which would be
  *  in the dataLen, given the other parameters ...
  *  if input samplesPerBlock is 0, then returns the max
  *  samplesPerBlock which would go into a block of size blockAlign
  *  Yes, it is confusing usage.
  */
-sox_size_t AdpcmSamplesIn(
-        sox_size_t dataLen,
-        sox_size_t chans,
-        sox_size_t blockAlign,
-        sox_size_t samplesPerBlock
+size_t lsx_ms_adpcm_samples_in(
+        size_t dataLen,
+        size_t chans,
+        size_t blockAlign,
+        size_t samplesPerBlock
 ){
-        sox_size_t m, n;
+        size_t m, n;
 
         if (samplesPerBlock) {
                 n = (dataLen / blockAlign) * samplesPerBlock;
@@ -351,15 +350,15 @@ sox_size_t AdpcmSamplesIn(
         return n;
 }
 
-sox_size_t AdpcmBytesPerBlock(
-        sox_size_t chans,
-        sox_size_t samplesPerBlock
+size_t lsx_ms_adpcm_bytes_per_block(
+        size_t chans,
+        size_t samplesPerBlock
 )
 {
-        sox_size_t n;
+        size_t n;
         n = 7*chans;  /* header */
         if (samplesPerBlock > 2)
-                n += (((sox_size_t)samplesPerBlock-2)*chans + 1)/2;
+                n += (((size_t)samplesPerBlock-2)*chans + 1)/2;
         return n;
 }
 
