@@ -1,9 +1,8 @@
-/*
- * File format: AMR-WB   (c) 2007 robs@users.sourceforge.net
- * 
+/* File format: AMR-WB   (c) 2007 robs@users.sourceforge.net
+ *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 2.1 of the License, or (at
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -12,8 +11,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this library.  If not, write to the Free Software Foundation,
- * Fifth Floor, 51 Franklin Street, Boston, MA 02111-1301, USA.
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /* In order to use this format with SoX, first build & install:
@@ -21,6 +20,8 @@
  * or install equivalent package(s) e.g. marillat.
  */
 
+#include "sox_i.h"
+
 #include "amrwb/typedef.h"
 #include "amrwb/enc_if.h"
 #include "amrwb/dec_if.h"
@@ -33,6 +34,5 @@ static char const magic[] = "#!AMR-WB\n";
 #define AMR_FRAME           L_FRAME16k
 #define AMR_MODE_MAX        8
 #define AMR_NAMES           "amr-wb", "awb"
-#define AMR_PRIV_TOO_BIG    amr_wb_PRIVSIZE_too_big
 #define AMR_RATE            16000
 #include "amr.h"
