@@ -1,4 +1,4 @@
-/* libSoX Biquad filter effects   (c) 2006-7 robs@users.sourceforge.net
+/* libSoX Biquad filter effects   (c) 2006-8 robs@users.sourceforge.net
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
@@ -66,7 +66,7 @@ typedef biquad_t priv_t;
 
 
 static int hilo1_getopts(sox_effect_t * effp, int n, char **argv) {
-  return sox_biquad_getopts(effp, n, argv, 1, 1, 0, 1, 2, "",
+  return lsx_biquad_getopts(effp, n, argv, 1, 1, 0, 1, 2, "",
       *effp->handler.name == 'l'? filter_LPF_1 : filter_HPF_1);
 }
 
@@ -78,7 +78,7 @@ static int hilo2_getopts(sox_effect_t * effp, int n, char **argv) {
   if (n != 0 && strcmp(argv[0], "-2") == 0)
     ++argv, --n;
   p->width = sqrt(0.5); /* Default to Butterworth */
-  return sox_biquad_getopts(effp, n, argv, 1, 2, 0, 1, 2, "qohk",
+  return lsx_biquad_getopts(effp, n, argv, 1, 2, 0, 1, 2, "qohk",
       *effp->handler.name == 'l'? filter_LPF : filter_HPF);
 }
 
@@ -87,12 +87,12 @@ static int bandpass_getopts(sox_effect_t * effp, int n, char **argv) {
   filter_t type = filter_BPF;
   if (n != 0 && strcmp(argv[0], "-c") == 0)
     ++argv, --n, type = filter_BPF_CSG;
-  return sox_biquad_getopts(effp, n, argv, 2, 2, 0, 1, 2, "hkqob", type);
+  return lsx_biquad_getopts(effp, n, argv, 2, 2, 0, 1, 2, "hkqob", type);
 }
 
 
 static int bandrej_getopts(sox_effect_t * effp, int n, char **argv) {
-  return sox_biquad_getopts(effp, n, argv, 2, 2, 0, 1, 2, "hkqob", filter_notch);
+  return lsx_biquad_getopts(effp, n, argv, 2, 2, 0, 1, 2, "hkqob", filter_notch);
 }
 
 
@@ -104,7 +104,7 @@ static int allpass_getopts(sox_effect_t * effp, int n, char **argv) {
   else if (n != 0 && strcmp(argv[0], "-2") == 0)
     ++argv, --n, type = filter_AP2;
   m = 1 + (type == filter_APF);
-  return sox_biquad_getopts(effp, n, argv, m, m, 0, 1, 2, "hkqo", type);
+  return lsx_biquad_getopts(effp, n, argv, m, m, 0, 1, 2, "hkqo", type);
 }
 
 
@@ -112,13 +112,13 @@ static int tone_getopts(sox_effect_t * effp, int n, char **argv) {
   priv_t * p = (priv_t *)effp->priv;
   p->width = 0.5;
   p->fc = *effp->handler.name == 'b'? 100 : 3000;
-  return sox_biquad_getopts(effp, n, argv, 1, 3, 1, 2, 0, "shkqo",
+  return lsx_biquad_getopts(effp, n, argv, 1, 3, 1, 2, 0, "shkqo",
       *effp->handler.name == 'b'?  filter_lowShelf: filter_highShelf);
 }
 
 
 static int equalizer_getopts(sox_effect_t * effp, int n, char **argv) {
-  return sox_biquad_getopts(effp, n, argv, 3, 3, 0, 1, 2, "qohk", filter_peakingEQ);
+  return lsx_biquad_getopts(effp, n, argv, 3, 3, 0, 1, 2, "qohk", filter_peakingEQ);
 }
 
 
@@ -126,7 +126,7 @@ static int band_getopts(sox_effect_t * effp, int n, char **argv) {
   filter_t type = filter_BPF_SPK;
   if (n != 0 && strcmp(argv[0], "-n") == 0)
     ++argv, --n, type = filter_BPF_SPK_N;
-  return sox_biquad_getopts(effp, n, argv, 1, 2, 0, 1, 2, "hkqo", type);
+  return lsx_biquad_getopts(effp, n, argv, 1, 2, 0, 1, 2, "hkqo", type);
 }
 
 
@@ -135,10 +135,30 @@ static int deemph_getopts(sox_effect_t * effp, int n, char **argv) {
   p->fc    = 5283;
   p->width = 0.4845;
   p->gain  = -9.477;
-  return sox_biquad_getopts(effp, n, argv, 0, 0, 0, 1, 2, "s", filter_deemph);
+  return lsx_biquad_getopts(effp, n, argv, 0, 0, 0, 1, 2, "s", filter_deemph);
 }
 
 
+static int riaa_getopts(sox_effect_t * effp, int argc, char **argv) {
+  priv_t * p = (priv_t *)effp->priv;
+  p->filter_type = filter_riaa;
+  (void)argv;
+  return argc? lsx_usage(effp) : SOX_SUCCESS;
+}
+
+
+static void make_poly_from_roots(
+    double const * roots, size_t num_roots, double * poly)
+{
+  size_t i, j;
+  poly[0] = 1;
+  poly[1] = -roots[0];
+  memset(poly + 2, 0, (num_roots + 1 - 2) * sizeof(*poly));
+  for (i = 1; i < num_roots; ++i)
+    for (j = num_roots; j > 0; --j)
+      poly[j] -= poly[j - 1] * roots[i];
+}
+
 static int start(sox_effect_t * effp)
 {
   priv_t * p = (priv_t *)effp->priv;
@@ -147,7 +167,7 @@ static int start(sox_effect_t * effp)
   double alpha = 0;
 
   if (w0 > M_PI) {
-    sox_fail("frequency must be less than half the sample-rate (Nyquist rate)");
+    lsx_fail("frequency must be less than half the sample-rate (Nyquist rate)");
     return SOX_EOF;
   }
 
@@ -257,7 +277,7 @@ static int start(sox_effect_t * effp)
 
     case filter_deemph:  /* See deemph.plt for documentation */
       if (effp->in_signal.rate != 44100) {
-        sox_fail("Sample rate must be 44100 (audio-CD)");
+        lsx_fail("Sample rate must be 44100 (audio-CD)");
         return SOX_EOF;
       }
       /* Falls through... */
@@ -309,8 +329,42 @@ static int start(sox_effect_t * effp)
       p->a1 = -2 * cos(w0);
       p->a2 = 1 - sin(w0);
       break;
+
+    case filter_riaa: /* http://www.dsprelated.com/showmessage/73300/3.php */
+      if (effp->in_signal.rate == 44100) {
+        static const double zeros[] = {-0.2014898, 0.9233820};
+        static const double poles[] = {0.7083149, 0.9924091};
+        make_poly_from_roots(zeros, (size_t)2, &p->b0);
+        make_poly_from_roots(poles, (size_t)2, &p->a0);
+      }
+      else if (effp->in_signal.rate == 48000) {
+        static const double zeros[] = {-0.1766069, 0.9321590};
+        static const double poles[] = {0.7396325, 0.9931330};
+        make_poly_from_roots(zeros, (size_t)2, &p->b0);
+        make_poly_from_roots(poles, (size_t)2, &p->a0);
+      }
+      else if (effp->in_signal.rate == 88200) {
+        static const double zeros[] = {-0.1168735, 0.9648312};
+        static const double poles[] = {0.8590646, 0.9964002};
+        make_poly_from_roots(zeros, (size_t)2, &p->b0);
+        make_poly_from_roots(poles, (size_t)2, &p->a0);
+      }
+      else if (effp->in_signal.rate == 96000) {
+        static const double zeros[] = {-0.1141486, 0.9676817};
+        static const double poles[] = {0.8699137, 0.9966946};
+        make_poly_from_roots(zeros, (size_t)2, &p->b0);
+        make_poly_from_roots(poles, (size_t)2, &p->a0);
+      }
+      else {
+        lsx_fail("Sample rate must be 44.1k, 48k, 88.2k, or 96k");
+        return SOX_EOF;
+      }
+      {double g = dB_to_linear(19.9 - linear_to_dB(
+            (p->b0 + p->b1 + p->b2) / (p->a0 + p->a1 + p->a2)));
+      p->b0 *= g; p->b1 *= g; p->b2 *= g;}
+      break;
   }
-  return sox_biquad_start(effp);
+  return lsx_biquad_start(effp);
 }
 
 
@@ -318,7 +372,7 @@ static int start(sox_effect_t * effp)
 sox_effect_handler_t const * sox_##name##_effect_fn(void) { \
   static sox_effect_handler_t handler = { \
     #name, usage, flags, \
-    group##_getopts, start, sox_biquad_flow, 0, 0, 0, sizeof(biquad_t)\
+    group##_getopts, start, lsx_biquad_flow, 0, 0, 0, sizeof(biquad_t)\
   }; \
   return &handler; \
 }
@@ -333,3 +387,4 @@ BIQUAD_EFFECT(treble,    tone,     "gain [frequency(3000) [width[s|h|k|q|o]](0.5
 BIQUAD_EFFECT(equalizer, equalizer,"frequency width[q|o|h|k] gain", 0)
 BIQUAD_EFFECT(band,      band,     "[-n] center [width[h|k|q|o]]", 0)
 BIQUAD_EFFECT(deemph,    deemph,   NULL, 0)
+BIQUAD_EFFECT(riaa,      riaa,     NULL, 0)
