@@ -1,15 +1,14 @@
-/*
- * August 24, 1998
+/* August 24, 1998
  * Copyright (C) 1998 Juergen Mueller And Sundry Contributors
  * This source code is freely redistributable and may be used for
- * any purpose.  This copyright notice must be maintained. 
- * Juergen Mueller And Sundry Contributors are not responsible for 
+ * any purpose.  This copyright notice must be maintained.
+ * Juergen Mueller And Sundry Contributors are not responsible for
  * the consequences of using this software.
  */
 
 /*
  *      Chorus effect.
- * 
+ *
  * Flow diagram scheme for n delays ( 1 <= n <= MAX_CHORUS ):
  *
  *        * gain-in                                           ___
@@ -28,7 +27,7 @@
  *            +---->| delay n |----------------------------->|   |
  *                  |_________|                              |   |
  *                     /|\                                   |___|
- *                      |                                      |  
+ *                      |                                      |
  *              +-----------------+   +--------------+         | * gain-out
  *              | Delay control n |<--| mod. speed n |         |
  *              +-----------------+   +--------------+         +----->obuff
@@ -36,7 +35,7 @@
  *
  * The delay i is controled by a sine or triangle modulation i ( 1 <= i <= n).
  *
- * Usage: 
+ * Usage:
  *   chorus gain-in gain-out delay-1 decay-1 speed-1 depth-1 -s1|t1 [
  *       delay-2 decay-2 speed-2 depth-2 -s2|-t2 ... ]
  *
@@ -51,7 +50,7 @@
  *
  * Note:
  *   when decay is close to 1.0, the samples can begin clipping and the output
- *   can saturate! 
+ *   can saturate!
  *
  * Hint:
  *   1 / out-gain < gain-in ( 1 + decay-1 + ... + decay-n )
@@ -62,19 +61,19 @@
  * libSoX chorus effect file.
  */
 
-#include <stdlib.h> /* Harmless, and prototypes atof() etc. --dgc */
-#include <math.h>
-#include <string.h>
 #include "sox_i.h"
 
+#include <stdlib.h> /* Harmless, and prototypes atof() etc. --dgc */
+#include <string.h>
+
 #define MOD_SINE        0
 #define MOD_TRIANGLE    1
 #define MAX_CHORUS      7
 
-typedef struct chorusstuff {
+typedef struct {
         int     num_chorus;
         int     modulation[MAX_CHORUS];
-        int     counter;                        
+        int     counter;
         long    phase[MAX_CHORUS];
         float   *chorusbuf;
         float   in_gain, out_gain;
@@ -85,28 +84,28 @@ typedef struct chorusstuff {
         int     depth_samples[MAX_CHORUS], samples[MAX_CHORUS];
         int maxsamples;
         unsigned int fade_out;
-} *chorus_t;
+} priv_t;
 
 /*
  * Process options
  */
-static int sox_chorus_getopts(sox_effect_t * effp, int n, char **argv) 
+static int sox_chorus_getopts(sox_effect_t * effp, int n, char **argv)
 {
-        chorus_t chorus = (chorus_t) effp->priv;
+        priv_t * chorus = (priv_t *) effp->priv;
         int i;
 
         chorus->num_chorus = 0;
         i = 0;
 
         if ( ( n < 7 ) || (( n - 2 ) % 5 ) )
-          return sox_usage(effp);
+          return lsx_usage(effp);
 
         sscanf(argv[i++], "%f", &chorus->in_gain);
         sscanf(argv[i++], "%f", &chorus->out_gain);
         while ( i < n ) {
                 if ( chorus->num_chorus > MAX_CHORUS )
                 {
-                        sox_fail("chorus: to many delays, use less than %i delays", MAX_CHORUS);
+                        lsx_fail("chorus: to many delays, use less than %i delays", MAX_CHORUS);
                         return (SOX_EOF);
                 }
                 sscanf(argv[i++], "%f", &chorus->delay[chorus->num_chorus]);
@@ -118,7 +117,7 @@ static int sox_chorus_getopts(sox_effect_t * effp, int n, char **argv)
                 else if ( ! strcmp(argv[i], "-t"))
                         chorus->modulation[chorus->num_chorus] = MOD_TRIANGLE;
                 else
-                  return sox_usage(effp);
+                  return lsx_usage(effp);
                 i++;
                 chorus->num_chorus++;
         }
@@ -130,7 +129,7 @@ static int sox_chorus_getopts(sox_effect_t * effp, int n, char **argv)
  */
 static int sox_chorus_start(sox_effect_t * effp)
 {
-        chorus_t chorus = (chorus_t) effp->priv;
+        priv_t * chorus = (priv_t *) effp->priv;
         int i;
         float sum_in_volume;
 
@@ -138,78 +137,78 @@ static int sox_chorus_start(sox_effect_t * effp)
 
         if ( chorus->in_gain < 0.0 )
         {
-                sox_fail("chorus: gain-in must be positive!");
+                lsx_fail("chorus: gain-in must be positive!");
                 return (SOX_EOF);
         }
         if ( chorus->in_gain > 1.0 )
         {
-                sox_fail("chorus: gain-in must be less than 1.0!");
+                lsx_fail("chorus: gain-in must be less than 1.0!");
                 return (SOX_EOF);
         }
         if ( chorus->out_gain < 0.0 )
         {
-                sox_fail("chorus: gain-out must be positive!");
+                lsx_fail("chorus: gain-out must be positive!");
                 return (SOX_EOF);
         }
         for ( i = 0; i < chorus->num_chorus; i++ ) {
-                chorus->samples[i] = (int) ( ( chorus->delay[i] + 
-                        chorus->depth[i] ) * effp->ininfo.rate / 1000.0);
-                chorus->depth_samples[i] = (int) (chorus->depth[i] * 
-                        effp->ininfo.rate / 1000.0);
+                chorus->samples[i] = (int) ( ( chorus->delay[i] +
+                        chorus->depth[i] ) * effp->in_signal.rate / 1000.0);
+                chorus->depth_samples[i] = (int) (chorus->depth[i] *
+                        effp->in_signal.rate / 1000.0);
 
                 if ( chorus->delay[i] < 20.0 )
                 {
-                        sox_fail("chorus: delay must be more than 20.0 msec!");
+                        lsx_fail("chorus: delay must be more than 20.0 msec!");
                         return (SOX_EOF);
                 }
                 if ( chorus->delay[i] > 100.0 )
                 {
-                        sox_fail("chorus: delay must be less than 100.0 msec!");
+                        lsx_fail("chorus: delay must be less than 100.0 msec!");
                         return (SOX_EOF);
                 }
                 if ( chorus->speed[i] < 0.1 )
                 {
-                        sox_fail("chorus: speed must be more than 0.1 Hz!");
+                        lsx_fail("chorus: speed must be more than 0.1 Hz!");
                         return (SOX_EOF);
                 }
                 if ( chorus->speed[i] > 5.0 )
                 {
-                        sox_fail("chorus: speed must be less than 5.0 Hz!");
+                        lsx_fail("chorus: speed must be less than 5.0 Hz!");
                         return (SOX_EOF);
                 }
                 if ( chorus->depth[i] < 0.0 )
                 {
-                        sox_fail("chorus: delay must be more positive!");
+                        lsx_fail("chorus: delay must be more positive!");
                         return (SOX_EOF);
                 }
                 if ( chorus->depth[i] > 10.0 )
                 {
-                    sox_fail("chorus: delay must be less than 10.0 msec!");
+                    lsx_fail("chorus: delay must be less than 10.0 msec!");
                     return (SOX_EOF);
                 }
                 if ( chorus->decay[i] < 0.0 )
                 {
-                        sox_fail("chorus: decay must be positive!" );
+                        lsx_fail("chorus: decay must be positive!" );
                         return (SOX_EOF);
                 }
                 if ( chorus->decay[i] > 1.0 )
                 {
-                        sox_fail("chorus: decay must be less that 1.0!" );
+                        lsx_fail("chorus: decay must be less that 1.0!" );
                         return (SOX_EOF);
                 }
-                chorus->length[i] = effp->ininfo.rate / chorus->speed[i];
-                chorus->lookup_tab[i] = (int *) xmalloc(sizeof (int) * chorus->length[i]);
+                chorus->length[i] = effp->in_signal.rate / chorus->speed[i];
+                chorus->lookup_tab[i] = lsx_malloc(sizeof (int) * chorus->length[i]);
 
                 if (chorus->modulation[i] == MOD_SINE)
-                  sox_generate_wave_table(SOX_WAVE_SINE, SOX_INT, chorus->lookup_tab[i],
-                                         (unsigned)chorus->length[i], 0., (double)chorus->depth_samples[i], 0.);
+                  lsx_generate_wave_table(SOX_WAVE_SINE, SOX_INT, chorus->lookup_tab[i],
+                                         (size_t)chorus->length[i], 0., (double)chorus->depth_samples[i], 0.);
                 else
-                  sox_generate_wave_table(SOX_WAVE_TRIANGLE, SOX_INT, chorus->lookup_tab[i], 
-                                         (unsigned)chorus->length[i],
+                  lsx_generate_wave_table(SOX_WAVE_TRIANGLE, SOX_INT, chorus->lookup_tab[i],
+                                         (size_t)chorus->length[i],
                                          (double)(chorus->samples[i] - 1 - 2 * chorus->depth_samples[i]),
                                          (double)(chorus->samples[i] - 1), 3 * M_PI_2);
                 chorus->phase[i] = 0;
-                
+
                 if ( chorus->samples[i] > chorus->maxsamples )
                   chorus->maxsamples = chorus->samples[i];
         }
@@ -219,10 +218,10 @@ static int sox_chorus_start(sox_effect_t * effp)
         for ( i = 0; i < chorus->num_chorus; i++ )
                 sum_in_volume += chorus->decay[i];
         if ( chorus->in_gain * ( sum_in_volume ) > 1.0 / chorus->out_gain )
-        sox_warn("chorus: warning >>> gain-out can cause saturation or clipping of output <<<");
+        lsx_warn("chorus: warning >>> gain-out can cause saturation or clipping of output <<<");
 
 
-        chorus->chorusbuf = (float *) xmalloc(sizeof (float) * chorus->maxsamples);
+        chorus->chorusbuf = lsx_malloc(sizeof (float) * chorus->maxsamples);
         for ( i = 0; i < chorus->maxsamples; i++ )
                 chorus->chorusbuf[i] = 0.0;
 
@@ -235,14 +234,14 @@ static int sox_chorus_start(sox_effect_t * effp)
  * Processed signed long samples from ibuf to obuf.
  * Return number of samples processed.
  */
-static int sox_chorus_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf, 
-                   sox_size_t *isamp, sox_size_t *osamp)
+static int sox_chorus_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf,
+                   size_t *isamp, size_t *osamp)
 {
-        chorus_t chorus = (chorus_t) effp->priv;
+        priv_t * chorus = (priv_t *) effp->priv;
         int i;
         float d_in, d_out;
         sox_sample_t out;
-        sox_size_t len = min(*isamp, *osamp);
+        size_t len = min(*isamp, *osamp);
         *isamp = *osamp = len;
 
         while (len--) {
@@ -251,8 +250,8 @@ static int sox_chorus_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sa
                 /* Compute output first */
                 d_out = d_in * chorus->in_gain;
                 for ( i = 0; i < chorus->num_chorus; i++ )
-                        d_out += chorus->chorusbuf[(chorus->maxsamples + 
-                        chorus->counter - chorus->lookup_tab[i][chorus->phase[i]]) % 
+                        d_out += chorus->chorusbuf[(chorus->maxsamples +
+                        chorus->counter - chorus->lookup_tab[i][chorus->phase[i]]) %
                         chorus->maxsamples] * chorus->decay[i];
                 /* Adjust the output volume and size to 24 bit */
                 d_out = d_out * chorus->out_gain;
@@ -260,10 +259,10 @@ static int sox_chorus_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sa
                 *obuf++ = out * 256;
                 /* Mix decay of delay and input */
                 chorus->chorusbuf[chorus->counter] = d_in;
-                chorus->counter = 
+                chorus->counter =
                         ( chorus->counter + 1 ) % chorus->maxsamples;
                 for ( i = 0; i < chorus->num_chorus; i++ )
-                        chorus->phase[i]  = 
+                        chorus->phase[i]  =
                                 ( chorus->phase[i] + 1 ) % chorus->length[i];
         }
         /* processed all samples */
@@ -271,14 +270,14 @@ static int sox_chorus_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sa
 }
 
 /*
- * Drain out reverb lines. 
+ * Drain out reverb lines.
  */
-static int sox_chorus_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *osamp)
+static int sox_chorus_drain(sox_effect_t * effp, sox_sample_t *obuf, size_t *osamp)
 {
-        chorus_t chorus = (chorus_t) effp->priv;
-        sox_size_t done;
+        priv_t * chorus = (priv_t *) effp->priv;
+        size_t done;
         int i;
-        
+
         float d_in, d_out;
         sox_sample_t out;
 
@@ -288,8 +287,8 @@ static int sox_chorus_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t
                 d_out = 0;
                 /* Compute output first */
                 for ( i = 0; i < chorus->num_chorus; i++ )
-                        d_out += chorus->chorusbuf[(chorus->maxsamples + 
-                chorus->counter - chorus->lookup_tab[i][chorus->phase[i]]) % 
+                        d_out += chorus->chorusbuf[(chorus->maxsamples +
+                chorus->counter - chorus->lookup_tab[i][chorus->phase[i]]) %
                 chorus->maxsamples] * chorus->decay[i];
                 /* Adjust the output volume and size to 24 bit */
                 d_out = d_out * chorus->out_gain;
@@ -297,10 +296,10 @@ static int sox_chorus_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t
                 *obuf++ = out * 256;
                 /* Mix decay of delay and input */
                 chorus->chorusbuf[chorus->counter] = d_in;
-                chorus->counter = 
+                chorus->counter =
                         ( chorus->counter + 1 ) % chorus->maxsamples;
                 for ( i = 0; i < chorus->num_chorus; i++ )
-                        chorus->phase[i]  = 
+                        chorus->phase[i]  =
                                 ( chorus->phase[i] + 1 ) % chorus->length[i];
                 done++;
                 chorus->fade_out--;
@@ -318,14 +317,14 @@ static int sox_chorus_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t
  */
 static int sox_chorus_stop(sox_effect_t * effp)
 {
-        chorus_t chorus = (chorus_t) effp->priv;
+        priv_t * chorus = (priv_t *) effp->priv;
         int i;
 
-        free((char *) chorus->chorusbuf);
-        chorus->chorusbuf = (float *) -1;   /* guaranteed core dump */
+        free(chorus->chorusbuf);
+        chorus->chorusbuf = NULL;
         for ( i = 0; i < chorus->num_chorus; i++ ) {
-                free((char *) chorus->lookup_tab[i]);
-                chorus->lookup_tab[i] = (int *) -1;   /* guaranteed core dump */
+                free(chorus->lookup_tab[i]);
+                chorus->lookup_tab[i] = NULL;
         }
         return (SOX_SUCCESS);
 }
@@ -339,7 +338,7 @@ static sox_effect_handler_t sox_chorus_effect = {
   sox_chorus_flow,
   sox_chorus_drain,
   sox_chorus_stop,
-  NULL
+  NULL, sizeof(priv_t)
 };
 
 const sox_effect_handler_t *sox_chorus_effect_fn(void)
