@@ -1,91 +1,47 @@
-/*
- *      CVSD (Continuously Variable Slope Delta modulation)
+/*      libSoX CVSD (Continuously Variable Slope Delta modulation)
  *      conversion routines
  *
  *      The CVSD format is described in the MIL Std 188 113, which is
  *      available from http://bbs.itsi.disa.mil:5580/T3564
  *
- *      Copyright (C) 1996  
+ *      Copyright (C) 1996
  *      Thomas Sailer (sailer@ife.ee.ethz.ch) (HB9JNX/AE4WA)
  *      Swiss Federal Institute of Technology, Electronics Lab
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License as published by the Free Software Foundation; either
- *   version 2 of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or (at
+ * your option) any later version.
  *
- *   This library is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *   Lesser General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.
  *
- *   You should have received a copy of the GNU Lesser General Public
- *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * Change History:
  *
  * June 1, 1998 - Chris Bagwell (cbagwell@sprynet.com)
  *   Fixed compile warnings reported by Kjetil Torgrim Homme
  *   <kjetilho@ifi.uio.no>
- *
- *
  */
 
-/* ---------------------------------------------------------------------- */
-
 #include "sox_i.h"
 #include "cvsd.h"
+#include "cvsdfilt.h"
 
-#include <math.h>
 #include <string.h>
 #include <time.h>
-#include <stdio.h>
-#include <errno.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>     /* For SEEK_* defines if not found in stdio */
-#endif
-
-#include "cvsdfilt.h"
 
 /* ---------------------------------------------------------------------- */
 /*
  * private data structures
  */
 
-struct cvsd_common_state {
-        unsigned overload;
-        float mla_int;
-        float mla_tc0;
-        float mla_tc1;
-        unsigned phase;
-        unsigned phase_inc;
-        float v_min, v_max;
-};
-
-struct cvsd_decode_state {
-        float output_filter[DEC_FILTERLEN];
-};
-
-struct cvsd_encode_state {
-        float recon_int;
-        float input_filter[ENC_FILTERLEN];
-};
-
-struct cvsdpriv {
-        struct cvsd_common_state com;
-        union {
-                struct cvsd_decode_state dec;
-                struct cvsd_encode_state enc;
-        } c;
-        struct {
-                unsigned char shreg;
-                unsigned mask;
-                unsigned cnt;
-        } bit;
-        unsigned bytes_written;
-        unsigned cvsd_rate;
-};
+typedef cvsd_priv_t priv_t;
 
 static int debug_count = 0;
 
@@ -113,13 +69,12 @@ static float float_conv(float *fp1, float *fp2,int n)
 
 static void cvsdstartcommon(sox_format_t * ft)
 {
-        struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
-        
+        priv_t *p = (priv_t *) ft->priv;
+
         p->cvsd_rate = (ft->signal.rate <= 24000) ? 16000 : 32000;
         ft->signal.rate = 8000;
         ft->signal.channels = 1;
-        ft->signal.size = SOX_SIZE_16BIT; /* make output format default to words */
-        ft->signal.encoding = SOX_ENCODING_SIGN2;
+        lsx_rawstart(ft, sox_true, sox_false, sox_true, SOX_ENCODING_CVSD, 1);
         /*
          * initialize the decoder
          */
@@ -145,15 +100,15 @@ static void cvsdstartcommon(sox_format_t * ft)
         p->bytes_written = 0;
         p->com.v_min = 1;
         p->com.v_max = -1;
-        sox_report("cvsd: bit rate %dbit/s, bits from %s", p->cvsd_rate,
-               ft->signal.reverse_bits ? "msb to lsb" : "lsb to msb");
+        lsx_report("cvsd: bit rate %dbit/s, bits from %s", p->cvsd_rate,
+               ft->encoding.reverse_bits ? "msb to lsb" : "lsb to msb");
 }
 
 /* ---------------------------------------------------------------------- */
 
-int sox_cvsdstartread(sox_format_t * ft) 
+int lsx_cvsdstartread(sox_format_t * ft)
 {
-        struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
+        priv_t *p = (priv_t *) ft->priv;
         float *fp1;
         int i;
 
@@ -167,9 +122,9 @@ int sox_cvsdstartread(sox_format_t * ft)
          * this is now done in the filter parameter generation utility
          */
         /*
-         * zero the filter 
+         * zero the filter
          */
-        for(fp1 = p->c.dec.output_filter, i = DEC_FILTERLEN; i > 0; i--)
+        for(fp1 = p->c.dec.output_filter, i = CVSD_DEC_FILTERLEN; i > 0; i--)
                 *fp1++ = 0;
 
         return (SOX_SUCCESS);
@@ -177,9 +132,9 @@ int sox_cvsdstartread(sox_format_t * ft)
 
 /* ---------------------------------------------------------------------- */
 
-int sox_cvsdstartwrite(sox_format_t * ft) 
+int lsx_cvsdstartwrite(sox_format_t * ft)
 {
-        struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
+        priv_t *p = (priv_t *) ft->priv;
         float *fp1;
         int i;
 
@@ -188,9 +143,9 @@ int sox_cvsdstartwrite(sox_format_t * ft)
         p->com.mla_tc1 = 0.1 * (1 - p->com.mla_tc0);
         p->com.phase = 4;
         /*
-         * zero the filter 
+         * zero the filter
          */
-        for(fp1 = p->c.enc.input_filter, i = ENC_FILTERLEN; i > 0; i--)
+        for(fp1 = p->c.enc.input_filter, i = CVSD_ENC_FILTERLEN; i > 0; i--)
                 *fp1++ = 0;
         p->c.enc.recon_int = 0;
 
@@ -199,27 +154,27 @@ int sox_cvsdstartwrite(sox_format_t * ft)
 
 /* ---------------------------------------------------------------------- */
 
-int sox_cvsdstopwrite(sox_format_t * ft)
+int lsx_cvsdstopwrite(sox_format_t * ft)
 {
-        struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
+        priv_t *p = (priv_t *) ft->priv;
 
         if (p->bit.cnt) {
-                sox_writeb(ft, p->bit.shreg);
+                lsx_writeb(ft, p->bit.shreg);
                 p->bytes_written++;
         }
-        sox_debug("cvsd: min slope %f, max slope %f", 
-               p->com.v_min, p->com.v_max);     
+        lsx_debug("cvsd: min slope %f, max slope %f",
+               p->com.v_min, p->com.v_max);
 
         return (SOX_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
 
-int sox_cvsdstopread(sox_format_t * ft)
+int lsx_cvsdstopread(sox_format_t * ft)
 {
-        struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
+        priv_t *p = (priv_t *) ft->priv;
 
-        sox_debug("cvsd: min value %f, max value %f", 
+        lsx_debug("cvsd: min value %f, max value %f",
                p->com.v_min, p->com.v_max);
 
         return(SOX_SUCCESS);
@@ -227,15 +182,15 @@ int sox_cvsdstopread(sox_format_t * ft)
 
 /* ---------------------------------------------------------------------- */
 
-sox_size_t sox_cvsdread(sox_format_t * ft, sox_sample_t *buf, sox_size_t nsamp) 
+size_t lsx_cvsdread(sox_format_t * ft, sox_sample_t *buf, size_t nsamp)
 {
-        struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
-        sox_size_t done = 0;
+        priv_t *p = (priv_t *) ft->priv;
+        size_t done = 0;
         float oval;
-        
+
         while (done < nsamp) {
                 if (!p->bit.cnt) {
-                        if (sox_readb(ft, &(p->bit.shreg)) == SOX_EOF)
+                        if (lsx_read_b_buf(ft, &(p->bit.shreg), (size_t) 1) != 1)
                                 return done;
                         p->bit.cnt = 8;
                         p->bit.mask = 1;
@@ -244,7 +199,7 @@ sox_size_t sox_cvsdread(sox_format_t * ft, sox_sample_t *buf, sox_size_t nsamp)
                  * handle one bit
                  */
                 p->bit.cnt--;
-                p->com.overload = ((p->com.overload << 1) | 
+                p->com.overload = ((p->com.overload << 1) |
                                    (!!(p->bit.shreg & p->bit.mask))) & 7;
                 p->bit.mask <<= 1;
                 p->com.mla_int *= p->com.mla_tc0;
@@ -261,12 +216,12 @@ sox_size_t sox_cvsdread(sox_format_t * ft, sox_sample_t *buf, sox_size_t nsamp)
                  */
                 p->com.phase += p->com.phase_inc;
                 if (p->com.phase >= 4) {
-                        oval = float_conv(p->c.dec.output_filter, 
-                                          (p->cvsd_rate < 24000) ? 
-                                          dec_filter_16 : dec_filter_32, 
-                                          DEC_FILTERLEN);
-                        sox_debug_more("input %d %f\n", debug_count, p->com.mla_int);
-                        sox_debug_more("recon %d %f\n", debug_count, oval);
+                        oval = float_conv(p->c.dec.output_filter,
+                                          (p->cvsd_rate < 24000) ?
+                                          dec_filter_16 : dec_filter_32,
+                                          CVSD_DEC_FILTERLEN);
+                        lsx_debug_more("input %d %f\n", debug_count, p->com.mla_int);
+                        lsx_debug_more("recon %d %f\n", debug_count, oval);
                         debug_count++;
 
                         if (oval > p->com.v_max)
@@ -283,10 +238,10 @@ sox_size_t sox_cvsdread(sox_format_t * ft, sox_sample_t *buf, sox_size_t nsamp)
 
 /* ---------------------------------------------------------------------- */
 
-sox_size_t sox_cvsdwrite(sox_format_t * ft, const sox_sample_t *buf, sox_size_t nsamp) 
+size_t lsx_cvsdwrite(sox_format_t * ft, const sox_sample_t *buf, size_t nsamp)
 {
-        struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
-        sox_size_t done = 0;
+        priv_t *p = (priv_t *) ft->priv;
+        size_t done = 0;
         float inval;
 
         for(;;) {
@@ -298,17 +253,17 @@ sox_size_t sox_cvsdwrite(sox_format_t * ft, const sox_sample_t *buf, sox_size_t
                                 return done;
                         memmove(p->c.enc.input_filter+1, p->c.enc.input_filter,
                                 sizeof(p->c.enc.input_filter)-sizeof(float));
-                        p->c.enc.input_filter[0] = (*buf++) / 
+                        p->c.enc.input_filter[0] = (*buf++) /
                                 ((float)SOX_SAMPLE_MAX);
                         done++;
                 }
                 p->com.phase &= 3;
                 /* insert input filter here! */
-                inval = float_conv(p->c.enc.input_filter, 
-                                   (p->cvsd_rate < 24000) ? 
-                                   (enc_filter_16[(p->com.phase >= 2)]) : 
-                                   (enc_filter_32[p->com.phase]), 
-                                   ENC_FILTERLEN);
+                inval = float_conv(p->c.enc.input_filter,
+                                   (p->cvsd_rate < 24000) ?
+                                   (enc_filter_16[(p->com.phase >= 2)]) :
+                                   (enc_filter_32[p->com.phase]),
+                                   CVSD_ENC_FILTERLEN);
                 /*
                  * encode one bit
                  */
@@ -327,15 +282,15 @@ sox_size_t sox_cvsdwrite(sox_format_t * ft, const sox_sample_t *buf, sox_size_t
                 } else
                         p->c.enc.recon_int -= p->com.mla_int;
                 if ((++(p->bit.cnt)) >= 8) {
-                        sox_writeb(ft, p->bit.shreg);
+                        lsx_writeb(ft, p->bit.shreg);
                         p->bytes_written++;
                         p->bit.shreg = p->bit.cnt = 0;
                         p->bit.mask = 1;
                 } else
                         p->bit.mask <<= 1;
                 p->com.phase += p->com.phase_inc;
-                sox_debug_more("input %d %f\n", debug_count, inval);
-                sox_debug_more("recon %d %f\n", debug_count, p->c.enc.recon_int);
+                lsx_debug_more("input %d %f\n", debug_count, inval);
+                lsx_debug_more("recon %d %f\n", debug_count, p->c.enc.recon_int);
                 debug_count++;
         }
 }
@@ -344,6 +299,38 @@ sox_size_t sox_cvsdwrite(sox_format_t * ft, const sox_sample_t *buf, sox_size_t
 /*
  * DVMS file header
  */
+
+/* FIXME: eliminate these 4 functions */
+
+static uint32_t get32_le(unsigned char **p)
+{
+  uint32_t val = (((*p)[3]) << 24) | (((*p)[2]) << 16) |
+          (((*p)[1]) << 8) | (**p);
+  (*p) += 4;
+  return val;
+}
+
+static uint16_t get16_le(unsigned char **p)
+{
+  unsigned val = (((*p)[1]) << 8) | (**p);
+  (*p) += 2;
+  return val;
+}
+
+static void put32_le(unsigned char **p, uint32_t val)
+{
+  *(*p)++ = val & 0xff;
+  *(*p)++ = (val >> 8) & 0xff;
+  *(*p)++ = (val >> 16) & 0xff;
+  *(*p)++ = (val >> 24) & 0xff;
+}
+
+static void put16_le(unsigned char **p, unsigned val)
+{
+  *(*p)++ = val & 0xff;
+  *(*p)++ = (val >> 8) & 0xff;
+}
+
 struct dvms_header {
         char          Filename[14];
         unsigned      Id;
@@ -351,7 +338,7 @@ struct dvms_header {
         time_t        Unixtime;
         unsigned      Usender;
         unsigned      Ureceiver;
-        sox_size_t     Length;
+        size_t     Length;
         unsigned      Srate;
         unsigned      Days;
         unsigned      Custom1;
@@ -372,7 +359,7 @@ static int dvms_read_header(sox_format_t * ft, struct dvms_header *hdr)
         int i;
         unsigned sum;
 
-        if (sox_readbuf(ft, hdrbuf, sizeof(hdrbuf)) != sizeof(hdrbuf))
+        if (lsx_readbuf(ft, hdrbuf, sizeof(hdrbuf)) != sizeof(hdrbuf))
         {
                 return (SOX_EOF);
         }
@@ -396,9 +383,9 @@ static int dvms_read_header(sox_format_t * ft, struct dvms_header *hdr)
         memcpy(hdr->extend, pch, sizeof(hdr->extend));
         pch += sizeof(hdr->extend);
         hdr->Crc = get16_le(&pch);
-        if (sum != hdr->Crc) 
+        if (sum != hdr->Crc)
         {
-                sox_report("DVMS header checksum error, read %u, calculated %u",
+                lsx_report("DVMS header checksum error, read %u, calculated %u",
                      hdr->Crc, sum);
                 return (SOX_EOF);
         }
@@ -425,7 +412,7 @@ static int dvms_write_header(sox_format_t * ft, struct dvms_header *hdr)
         put32_le(&pch, (unsigned)hdr->Unixtime);
         put16_le(&pch, hdr->Usender);
         put16_le(&pch, hdr->Ureceiver);
-        put32_le(&pch, hdr->Length);
+        put32_le(&pch, (unsigned) hdr->Length);
         put16_le(&pch, hdr->Srate);
         put16_le(&pch, hdr->Days);
         put16_le(&pch, hdr->Custom1);
@@ -438,14 +425,14 @@ static int dvms_write_header(sox_format_t * ft, struct dvms_header *hdr)
                 sum += *pchs++;
         hdr->Crc = sum;
         put16_le(&pch, hdr->Crc);
-        if (sox_seeki(ft, 0, SEEK_SET) < 0)
+        if (lsx_seeki(ft, (off_t)0, SEEK_SET) < 0)
         {
-                sox_report("seek failed\n: %s",strerror(errno));
+                lsx_report("seek failed\n: %s",strerror(errno));
                 return (SOX_EOF);
         }
-        if (sox_writebuf(ft, hdrbuf, sizeof(hdrbuf)) != sizeof(hdrbuf))
+        if (lsx_writebuf(ft, hdrbuf, sizeof(hdrbuf)) != sizeof(hdrbuf))
         {
-                sox_report("%s",strerror(errno));
+                lsx_report("%s",strerror(errno));
                 return (SOX_EOF);
         }
         return (SOX_SUCCESS);
@@ -455,8 +442,9 @@ static int dvms_write_header(sox_format_t * ft, struct dvms_header *hdr)
 
 static void make_dvms_hdr(sox_format_t * ft, struct dvms_header *hdr)
 {
-        struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
+        priv_t *p = (priv_t *) ft->priv;
         size_t len;
+        char * comment = lsx_cat_comments(ft->oob.comments);
 
         memset(hdr->Filename, 0, sizeof(hdr->Filename));
         len = strlen(ft->filename);
@@ -464,50 +452,51 @@ static void make_dvms_hdr(sox_format_t * ft, struct dvms_header *hdr)
                 len = sizeof(hdr->Filename)-1;
         memcpy(hdr->Filename, ft->filename, len);
         hdr->Id = hdr->State = 0;
-        hdr->Unixtime = time(NULL);
+        hdr->Unixtime = sox_globals.repeatable? 0 : time(NULL);
         hdr->Usender = hdr->Ureceiver = 0;
         hdr->Length = p->bytes_written;
         hdr->Srate = p->cvsd_rate/100;
         hdr->Days = hdr->Custom1 = hdr->Custom2 = 0;
         memset(hdr->Info, 0, sizeof(hdr->Info));
-        len = strlen(ft->comment);
+        len = strlen(comment);
         if (len >= sizeof(hdr->Info))
                 len = sizeof(hdr->Info)-1;
-        memcpy(hdr->Info, ft->comment, len);
+        memcpy(hdr->Info, comment, len);
         memset(hdr->extend, 0, sizeof(hdr->extend));
+        free(comment);
 }
 
 /* ---------------------------------------------------------------------- */
 
-int sox_dvmsstartread(sox_format_t * ft) 
+int lsx_dvmsstartread(sox_format_t * ft)
 {
         struct dvms_header hdr;
         int rc;
 
         rc = dvms_read_header(ft, &hdr);
         if (rc){
-            sox_fail_errno(ft,SOX_EHDR,"unable to read DVMS header");
+            lsx_fail_errno(ft,SOX_EHDR,"unable to read DVMS header");
             return rc;
         }
 
-        sox_debug("DVMS header of source file \"%s\":", ft->filename);
-        sox_debug("  filename  \"%.14s\"", hdr.Filename);
-        sox_debug("  id        0x%x", hdr.Id);
-        sox_debug("  state     0x%x", hdr.State);
-        sox_debug("  time      %s", ctime(&hdr.Unixtime)); /* ctime generates lf */
-        sox_debug("  usender   %u", hdr.Usender);
-        sox_debug("  ureceiver %u", hdr.Ureceiver);
-        sox_debug("  length    %u", hdr.Length);
-        sox_debug("  srate     %u", hdr.Srate);
-        sox_debug("  days      %u", hdr.Days);
-        sox_debug("  custom1   %u", hdr.Custom1);
-        sox_debug("  custom2   %u", hdr.Custom2);
-        sox_debug("  info      \"%.16s\"", hdr.Info);
+        lsx_debug("DVMS header of source file \"%s\":", ft->filename);
+        lsx_debug("  filename  \"%.14s\"", hdr.Filename);
+        lsx_debug("  id        0x%x", hdr.Id);
+        lsx_debug("  state     0x%x", hdr.State);
+        lsx_debug("  time      %s", ctime(&hdr.Unixtime)); /* ctime generates lf */
+        lsx_debug("  usender   %u", hdr.Usender);
+        lsx_debug("  ureceiver %u", hdr.Ureceiver);
+        lsx_debug("  length    %lu", (unsigned long)hdr.Length);
+        lsx_debug("  srate     %u", hdr.Srate);
+        lsx_debug("  days      %u", hdr.Days);
+        lsx_debug("  custom1   %u", hdr.Custom1);
+        lsx_debug("  custom2   %u", hdr.Custom2);
+        lsx_debug("  info      \"%.16s\"", hdr.Info);
         ft->signal.rate = (hdr.Srate < 240) ? 16000 : 32000;
-        sox_debug("DVMS rate %dbit/s using %gbit/s deviation %g%%", 
-               hdr.Srate*100, ft->signal.rate, 
+        lsx_debug("DVMS rate %dbit/s using %gbit/s deviation %g%%",
+               hdr.Srate*100, ft->signal.rate,
                ((ft->signal.rate - hdr.Srate*100) * 100) / ft->signal.rate);
-        rc = sox_cvsdstartread(ft);
+        rc = lsx_cvsdstartread(ft);
         if (rc)
             return rc;
 
@@ -516,51 +505,51 @@ int sox_dvmsstartread(sox_format_t * ft)
 
 /* ---------------------------------------------------------------------- */
 
-int sox_dvmsstartwrite(sox_format_t * ft) 
+int lsx_dvmsstartwrite(sox_format_t * ft)
 {
         struct dvms_header hdr;
         int rc;
-        
-        rc = sox_cvsdstartwrite(ft);
+
+        rc = lsx_cvsdstartwrite(ft);
         if (rc)
             return rc;
 
         make_dvms_hdr(ft, &hdr);
         rc = dvms_write_header(ft, &hdr);
         if (rc){
-                sox_fail_errno(ft,rc,"cannot write DVMS header");
+                lsx_fail_errno(ft,rc,"cannot write DVMS header");
             return rc;
         }
 
         if (!ft->seekable)
-               sox_warn("Length in output .DVMS header will wrong since can't seek to fix it");
+               lsx_warn("Length in output .DVMS header will wrong since can't seek to fix it");
 
         return(SOX_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
 
-int sox_dvmsstopwrite(sox_format_t * ft)
+int lsx_dvmsstopwrite(sox_format_t * ft)
 {
         struct dvms_header hdr;
         int rc;
-        
-        sox_cvsdstopwrite(ft);
+
+        lsx_cvsdstopwrite(ft);
         if (!ft->seekable)
         {
-            sox_warn("File not seekable");
+            lsx_warn("File not seekable");
             return (SOX_EOF);
         }
-        if (sox_seeki(ft, 0, 0) != 0)
+        if (lsx_seeki(ft, (off_t)0, 0) != 0)
         {
-                sox_fail_errno(ft,errno,"Can't rewind output file to rewrite DVMS header.");
+                lsx_fail_errno(ft,errno,"Can't rewind output file to rewrite DVMS header.");
                 return(SOX_EOF);
         }
         make_dvms_hdr(ft, &hdr);
         rc = dvms_write_header(ft, &hdr);
         if(rc){
-            sox_fail_errno(ft,rc,"cannot write DVMS header");
+            lsx_fail_errno(ft,rc,"cannot write DVMS header");
             return rc;
-        }       
+        }
         return rc;
 }
