@@ -1,15 +1,12 @@
-/*
- * August 24, 1998
+/* libSoX Echo effect             August 24, 1998
+ *
  * Copyright (C) 1998 Juergen Mueller And Sundry Contributors
  * This source code is freely redistributable and may be used for
- * any purpose.  This copyright notice must be maintained. 
- * Juergen Mueller And Sundry Contributors are not responsible for 
+ * any purpose.  This copyright notice must be maintained.
+ * Juergen Mueller And Sundry Contributors are not responsible for
  * the consequences of using this software.
- */
-
-/*
- * Echos effect for dsp.
- * 
+ *
+ *
  * Flow diagram scheme for n delays ( 1 <= n <= MAX_ECHOS ):
  *
  *                                                    * gain-in  ___
@@ -20,12 +17,11 @@
  *         |               |             +--------------------->|   |
  *         |               |             |            * decay n |   |
  *         |    _________  |  _________  |     _________   +--->|___|
- *         |   |         | | |         | |    |         |  |      | 
+ *         |   |         | | |         | |    |         |  |      |
  *         +-->| delay 1 |-+-| delay 2 |-+...-| delay n |--+      | * gain-out
  *             |_________|   |_________|      |_________|         |
  *                                                                +----->obuff
- *
- * Usage: 
+ * Usage:
  *   echos gain-in gain-out delay-1 decay-1 [delay-2 decay-2 ... delay-n decay-n]
  *
  * Where:
@@ -35,50 +31,46 @@
  *
  * Note:
  *   when decay is close to 1.0, the samples can begin clipping and the output
- *   can saturate! 
+ *   can saturate!
  *
  * Hint:
  *   1 / out-gain > gain-in ( 1 + decay-1 + ... + decay-n )
  *
-*/
-
-/*
- * libSoX reverb effect file.
  */
 
-#include <stdlib.h> /* Harmless, and prototypes atof() etc. --dgc */
-#include <math.h>
 #include "sox_i.h"
 
-#define DELAY_BUFSIZ ( 50 * SOX_MAXRATE )
+#include <stdlib.h> /* Harmless, and prototypes atof() etc. --dgc */
+
+#define DELAY_BUFSIZ ( 50 * 50U * 1024 )
 #define MAX_ECHOS 7     /* 24 bit x ( 1 + MAX_ECHOS ) = */
                         /* 24 bit x 8 = 32 bit !!!      */
 
 /* Private data for SKEL file */
-typedef struct echosstuff {
-        int     counter[MAX_ECHOS];                     
+typedef struct {
+        int     counter[MAX_ECHOS];
         int     num_delays;
         double  *delay_buf;
         float   in_gain, out_gain;
         float   delay[MAX_ECHOS], decay[MAX_ECHOS];
-        sox_ssize_t samples[MAX_ECHOS], pointer[MAX_ECHOS];
-        sox_size_t sumsamples;
-} *echos_t;
+        ptrdiff_t samples[MAX_ECHOS], pointer[MAX_ECHOS];
+        size_t sumsamples;
+} priv_t;
 
 /* Private data for SKEL file */
 
 /*
  * Process options
  */
-static int sox_echos_getopts(sox_effect_t * effp, int n, char **argv) 
+static int sox_echos_getopts(sox_effect_t * effp, int n, char **argv)
 {
-        echos_t echos = (echos_t) effp->priv;
+        priv_t * echos = (priv_t *) effp->priv;
         int i;
 
         echos->num_delays = 0;
 
         if ((n < 4) || (n % 2))
-          return sox_usage(effp);
+          return lsx_usage(effp);
 
         i = 0;
         sscanf(argv[i++], "%f", &echos->in_gain);
@@ -90,7 +82,7 @@ static int sox_echos_getopts(sox_effect_t * effp, int n, char **argv)
                 echos->num_delays++;
                 if ( echos->num_delays > MAX_ECHOS )
                 {
-                        sox_fail("echos: to many delays, use less than %i delays",
+                        lsx_fail("echos: to many delays, use less than %i delays",
                                 MAX_ECHOS);
                         return (SOX_EOF);
                 }
@@ -104,62 +96,62 @@ static int sox_echos_getopts(sox_effect_t * effp, int n, char **argv)
  */
 static int sox_echos_start(sox_effect_t * effp)
 {
-        echos_t echos = (echos_t) effp->priv;
+        priv_t * echos = (priv_t *) effp->priv;
         int i;
         float sum_in_volume;
         unsigned long j;
 
         if ( echos->in_gain < 0.0 )
         {
-                sox_fail("echos: gain-in must be positive!");
+                lsx_fail("echos: gain-in must be positive!");
                 return (SOX_EOF);
         }
         if ( echos->in_gain > 1.0 )
         {
-                sox_fail("echos: gain-in must be less than 1.0!");
+                lsx_fail("echos: gain-in must be less than 1.0!");
                 return (SOX_EOF);
         }
         if ( echos->out_gain < 0.0 )
         {
-                sox_fail("echos: gain-in must be positive!");
+                lsx_fail("echos: gain-in must be positive!");
                 return (SOX_EOF);
         }
         for ( i = 0; i < echos->num_delays; i++ ) {
-                echos->samples[i] = echos->delay[i] * effp->ininfo.rate / 1000.0;
+                echos->samples[i] = echos->delay[i] * effp->in_signal.rate / 1000.0;
                 if ( echos->samples[i] < 1 )
                 {
-                    sox_fail("echos: delay must be positive!");
+                    lsx_fail("echos: delay must be positive!");
                     return (SOX_EOF);
                 }
-                if ( echos->samples[i] > (sox_ssize_t)DELAY_BUFSIZ )
+                if ( echos->samples[i] > (ptrdiff_t)DELAY_BUFSIZ )
                 {
-                        sox_fail("echos: delay must be less than %g seconds!",
-                                DELAY_BUFSIZ / effp->ininfo.rate );
+                        lsx_fail("echos: delay must be less than %g seconds!",
+                                DELAY_BUFSIZ / effp->in_signal.rate );
                         return (SOX_EOF);
                 }
                 if ( echos->decay[i] < 0.0 )
                 {
-                    sox_fail("echos: decay must be positive!" );
+                    lsx_fail("echos: decay must be positive!" );
                     return (SOX_EOF);
                 }
                 if ( echos->decay[i] > 1.0 )
                 {
-                    sox_fail("echos: decay must be less than 1.0!" );
+                    lsx_fail("echos: decay must be less than 1.0!" );
                     return (SOX_EOF);
                 }
                 echos->counter[i] = 0;
                 echos->pointer[i] = echos->sumsamples;
                 echos->sumsamples += echos->samples[i];
         }
-        echos->delay_buf = (double *) xmalloc(sizeof (double) * echos->sumsamples);
+        echos->delay_buf = lsx_malloc(sizeof (double) * echos->sumsamples);
         for ( j = 0; j < echos->sumsamples; ++j )
                 echos->delay_buf[j] = 0.0;
         /* Be nice and check the hint with warning, if... */
         sum_in_volume = 1.0;
-        for ( i = 0; i < echos->num_delays; i++ ) 
+        for ( i = 0; i < echos->num_delays; i++ )
                 sum_in_volume += echos->decay[i];
         if ( sum_in_volume * echos->in_gain > 1.0 / echos->out_gain )
-                sox_warn("echos: warning >>> gain-out can cause saturation of output <<<");
+                lsx_warn("echos: warning >>> gain-out can cause saturation of output <<<");
         return (SOX_SUCCESS);
 }
 
@@ -167,14 +159,14 @@ static int sox_echos_start(sox_effect_t * effp)
  * Processed signed long samples from ibuf to obuf.
  * Return number of samples processed.
  */
-static int sox_echos_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf, 
-                sox_size_t *isamp, sox_size_t *osamp)
+static int sox_echos_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf,
+                size_t *isamp, size_t *osamp)
 {
-        echos_t echos = (echos_t) effp->priv;
+        priv_t * echos = (priv_t *) effp->priv;
         int j;
         double d_in, d_out;
         sox_sample_t out;
-        sox_size_t len = min(*isamp, *osamp);
+        size_t len = min(*isamp, *osamp);
         *isamp = *osamp = len;
 
         while (len--) {
@@ -194,12 +186,12 @@ static int sox_echos_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sam
                         if ( j == 0 )
                                 echos->delay_buf[echos->counter[j] + echos->pointer[j]] = d_in;
                         else
-                                echos->delay_buf[echos->counter[j] + echos->pointer[j]] = 
+                                echos->delay_buf[echos->counter[j] + echos->pointer[j]] =
                                    echos->delay_buf[echos->counter[j-1] + echos->pointer[j-1]] + d_in;
                 }
                 /* Adjust the counters */
                 for ( j = 0; j < echos->num_delays; j++ )
-                        echos->counter[j] = 
+                        echos->counter[j] =
                            ( echos->counter[j] + 1 ) % echos->samples[j];
         }
         /* processed all samples */
@@ -207,15 +199,15 @@ static int sox_echos_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sam
 }
 
 /*
- * Drain out reverb lines. 
+ * Drain out reverb lines.
  */
-static int sox_echos_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *osamp)
+static int sox_echos_drain(sox_effect_t * effp, sox_sample_t *obuf, size_t *osamp)
 {
-        echos_t echos = (echos_t) effp->priv;
+        priv_t * echos = (priv_t *) effp->priv;
         double d_in, d_out;
         sox_sample_t out;
         int j;
-        sox_size_t done;
+        size_t done;
 
         done = 0;
         /* drain out delay samples */
@@ -234,12 +226,12 @@ static int sox_echos_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *
                         if ( j == 0 )
                                 echos->delay_buf[echos->counter[j] + echos->pointer[j]] = d_in;
                         else
-                                echos->delay_buf[echos->counter[j] + echos->pointer[j]] = 
+                                echos->delay_buf[echos->counter[j] + echos->pointer[j]] =
                                    echos->delay_buf[echos->counter[j-1] + echos->pointer[j-1]];
                 }
                 /* Adjust the counters */
                 for ( j = 0; j < echos->num_delays; j++ )
-                        echos->counter[j] = 
+                        echos->counter[j] =
                            ( echos->counter[j] + 1 ) % echos->samples[j];
                 done++;
                 echos->sumsamples--;
@@ -257,10 +249,10 @@ static int sox_echos_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *
  */
 static int sox_echos_stop(sox_effect_t * effp)
 {
-        echos_t echos = (echos_t) effp->priv;
+        priv_t * echos = (priv_t *) effp->priv;
 
-        free((char *) echos->delay_buf);
-        echos->delay_buf = (double *) -1;   /* guaranteed core dump */
+        free(echos->delay_buf);
+        echos->delay_buf = NULL;
         return (SOX_SUCCESS);
 }
 
@@ -273,7 +265,7 @@ static sox_effect_handler_t sox_echos_effect = {
   sox_echos_flow,
   sox_echos_drain,
   sox_echos_stop,
-  NULL
+  NULL, sizeof(priv_t)
 };
 
 const sox_effect_handler_t *sox_echos_effect_fn(void)
