@@ -1,10 +1,8 @@
-/*
- * SoX Effects chain
- * (c) 2007 robs@users.sourceforge.net
+/* SoX Effects chain     (c) 2007 robs@users.sourceforge.net
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 2.1 of the License, or (at
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -13,11 +11,12 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this library.  If not, write to the Free Software Foundation,
- * Fifth Floor, 51 Franklin Street, Boston, MA 02111-1301, USA.
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "sox_i.h"
+#include "getopt.h"
 #include <assert.h>
 #include <string.h>
 #ifdef HAVE_STRINGS_H
@@ -25,25 +24,16 @@
 #endif
 
 
-#undef sox_fail
-#undef sox_report
-#define sox_fail sox_globals.subsystem=effp->handler.name,sox_fail
-#define sox_report sox_globals.subsystem=effp->handler.name,sox_report
+#undef lsx_fail
+#undef lsx_report
+#define lsx_fail sox_globals.subsystem=effp->handler.name,lsx_fail
+#define lsx_report sox_globals.subsystem=effp->handler.name,lsx_report
 
 #define DEBUG_EFFECTS_CHAIN 0
 
-
+/* FIXME: Not thread safe using globals */
 sox_effects_globals_t sox_effects_globals =
-    {sox_plot_off, 1, &sox_globals};
-
-int sox_usage(sox_effect_t * effp)
-{
-  if (effp->handler.usage)
-    sox_fail("usage: %s", effp->handler.usage);
-  else
-    sox_fail("this effect takes no parameters");
-  return SOX_EOF;
-}
+    {sox_plot_off, &sox_globals};
 
 /* Default effect handler functions for do-nothing situations: */
 
@@ -53,7 +43,7 @@ static int default_function(sox_effect_t * effp UNUSED)
 }
 
 /* Pass through samples verbatim */
-static int default_flow(sox_effect_t * effp UNUSED, const sox_sample_t *ibuf UNUSED, sox_sample_t *obuf UNUSED, sox_size_t *isamp, sox_size_t *osamp)
+static int default_flow(sox_effect_t * effp UNUSED, const sox_sample_t *ibuf UNUSED, sox_sample_t *obuf UNUSED, size_t *isamp, size_t *osamp)
 {
   *isamp = *osamp = min(*isamp, *osamp);
   memcpy(obuf, ibuf, *isamp * sizeof(*obuf));
@@ -61,7 +51,7 @@ static int default_flow(sox_effect_t * effp UNUSED, const sox_sample_t *ibuf UNU
 }
 
 /* Inform no more samples to drain */
-static int default_drain(sox_effect_t * effp UNUSED, sox_sample_t *obuf UNUSED, sox_size_t *osamp)
+static int default_drain(sox_effect_t * effp UNUSED, sox_sample_t *obuf UNUSED, size_t *osamp)
 {
   *osamp = 0;
   return SOX_EOF;
@@ -70,14 +60,14 @@ static int default_drain(sox_effect_t * effp UNUSED, sox_sample_t *obuf UNUSED,
 /* Check that no parameters have been given */
 static int default_getopts(sox_effect_t * effp, int argc, char **argv UNUSED)
 {
-  return argc? sox_usage(effp) : SOX_SUCCESS;
+  return argc? lsx_usage(effp) : SOX_SUCCESS;
 }
 
 /* Partially initialise the effect structure; signal info will come later */
-void sox_create_effect(sox_effect_t * effp, sox_effect_handler_t const * eh)
+sox_effect_t * sox_create_effect(sox_effect_handler_t const * eh)
 {
-  assert(eh);
-  memset(effp, 0, sizeof(*effp));
+  sox_effect_t * effp = lsx_calloc(1, sizeof(*effp));
+
   effp->global_info = &sox_effects_globals;
   effp->handler = *eh;
   if (!effp->handler.getopts) effp->handler.getopts = default_getopts;
@@ -86,24 +76,48 @@ void sox_create_effect(sox_effect_t * effp, sox_effect_handler_t const * eh)
   if (!effp->handler.drain  ) effp->handler.drain   = default_drain;
   if (!effp->handler.stop   ) effp->handler.stop    = default_function;
   if (!effp->handler.kill   ) effp->handler.kill    = default_function;
-}
 
+  effp->priv = lsx_calloc(1, effp->handler.priv_size);
 
+  return effp;
+} /* sox_create_effect */
+
+int sox_effect_options(sox_effect_t *effp, int argc, char * const argv[])
+{
+  int result, callers_optind = optind, callers_opterr = opterr;
+
+  if (effp->handler.flags & SOX_EFF_GETOPT)
+    --argv, ++argc; /* FIXME: push this processing back up the stack */
+  optind = 1, opterr = 0;
+  result = effp->handler.getopts(effp, argc, (char * *)argv);
+  optind = callers_optind, opterr = callers_opterr;
+  return result;
+} /* sox_effect_options */
 
 /* Effects chain: */
 
-sox_effects_chain_t * sox_create_effects_chain(void)
+sox_effects_chain_t * sox_create_effects_chain(
+    sox_encodinginfo_t const * in_enc, sox_encodinginfo_t const * out_enc)
 {
-  sox_effects_chain_t * result = xcalloc(1, sizeof(sox_effects_chain_t));
+  sox_effects_chain_t * result = lsx_calloc(1, sizeof(sox_effects_chain_t));
   result->global_info = sox_effects_globals;
+  result->in_enc = in_enc;
+  result->out_enc = out_enc;
   return result;
-}
+} /* sox_create_effects_chain */
+
+void sox_delete_effects_chain(sox_effects_chain_t *ecp)
+{
+    if (ecp && ecp->length)
+        sox_delete_effects(ecp);
+    free(ecp);
+} /* sox_delete_effects_chain */
 
 /* Effect can call in start() or flow() to set minimum input size to flow() */
-int sox_effect_set_imin(sox_effect_t * effp, sox_size_t imin)
+int lsx_effect_set_imin(sox_effect_t * effp, size_t imin)
 {
   if (imin > sox_globals.bufsiz / effp->flows) {
-    sox_fail("sox_bufsiz not big enough");
+    lsx_fail("sox_bufsiz not big enough");
     return SOX_EOF;
   }
 
@@ -117,73 +131,79 @@ int sox_effect_set_imin(sox_effect_t * effp, sox_size_t imin)
  * output rate and channels the effect does produce are written back to *in,
  * ready for the next effect in the chain.
  */
-int sox_add_effect(sox_effects_chain_t * chain, sox_effect_t * effp, sox_signalinfo_t * in, sox_signalinfo_t
-    const * out)
+int sox_add_effect(sox_effects_chain_t * chain, sox_effect_t * effp, sox_signalinfo_t * in, sox_signalinfo_t const * out)
 {
   int ret, (*start)(sox_effect_t * effp) = effp->handler.start;
   unsigned f;
   sox_effect_t eff0;  /* Copy of effect for flow 0 before calling start */
 
-  if (effp->handler.flags & SOX_EFF_NULL) {
-    sox_report("has no effect (is a proxy effect)");
-    return SOX_SUCCESS;
-  }
   effp->global_info = &chain->global_info;
-  effp->ininfo = *in;
-  effp->outinfo = *out;
+  effp->in_signal = *in;
+  effp->out_signal = *out;
+  effp->in_encoding = chain->in_enc;
+  effp->out_encoding = chain->out_enc;
   if (!(effp->handler.flags & SOX_EFF_CHAN))
-    effp->outinfo.channels = in->channels;
+    effp->out_signal.channels = in->channels;
   if (!(effp->handler.flags & SOX_EFF_RATE))
-    effp->outinfo.rate = in->rate;
+    effp->out_signal.rate = in->rate;
   if (!(effp->handler.flags & SOX_EFF_PREC))
-    effp->outinfo.size = in->size;
+    effp->out_signal.precision = in->precision;
 
   effp->flows =
-    (effp->handler.flags & SOX_EFF_MCHAN)? 1 : effp->ininfo.channels;
+    (effp->handler.flags & SOX_EFF_MCHAN)? 1 : effp->in_signal.channels;
   effp->clips = 0;
   effp->imin = 0;
   eff0 = *effp;
+  eff0.priv = lsx_memdup(eff0.priv, eff0.handler.priv_size);
   ret = start(effp);
   if (ret == SOX_EFF_NULL) {
-    sox_report("has no effect in this configuration");
+    lsx_report("has no effect in this configuration");
+    free(eff0.priv);
     return SOX_SUCCESS;
   }
-  if (ret != SOX_SUCCESS)
+  if (ret != SOX_SUCCESS) {
+    free(eff0.priv);
     return SOX_EOF;
+  }
 
-  *in = effp->outinfo;
+  *in = effp->out_signal;
 
   if (chain->length == SOX_MAX_EFFECTS) {
-    sox_fail("Too many effects!");
+    lsx_fail("Too many effects!");
+    free(eff0.priv);
     return SOX_EOF;
   }
   chain->effects[chain->length] =
-    xcalloc(effp->flows, sizeof(chain->effects[chain->length][0]));
+    lsx_calloc(effp->flows, sizeof(chain->effects[chain->length][0]));
   chain->effects[chain->length][0] = *effp;
 
   for (f = 1; f < effp->flows; ++f) {
     chain->effects[chain->length][f] = eff0;
     chain->effects[chain->length][f].flow = f;
-    if (start(&chain->effects[chain->length][f]) != SOX_SUCCESS)
+    chain->effects[chain->length][f].priv = lsx_memdup(eff0.priv, eff0.handler.priv_size);
+    if (start(&chain->effects[chain->length][f]) != SOX_SUCCESS) {
+      free(eff0.priv);
       return SOX_EOF;
+    }
   }
 
   ++chain->length;
+  free(eff0.priv);
   return SOX_SUCCESS;
 }
 
-static int flow_effect(sox_effects_chain_t * chain, unsigned n)
+static int flow_effect(sox_effects_chain_t * chain, size_t n)
 {
   sox_effect_t * effp1 = &chain->effects[n - 1][0];
   sox_effect_t * effp = &chain->effects[n][0];
   int effstatus = SOX_SUCCESS;
-  sox_size_t i, f;
+  size_t i, f;
   const sox_sample_t *ibuf;
-  sox_size_t idone = effp1->oend - effp1->obeg;
-  sox_size_t obeg = sox_globals.bufsiz - effp->oend;
+  size_t idone = effp1->oend - effp1->obeg;
+  size_t obeg = sox_globals.bufsiz - effp->oend;
 #if DEBUG_EFFECTS_CHAIN
-  sox_size_t pre_idone = idone;
-  sox_size_t pre_odone = obeg;
+  size_t pre_idone = idone;
+  size_t pre_odone = obeg;
 #endif
 
   if (effp->flows == 1)       /* Run effect on all channels at once */
@@ -191,7 +211,7 @@ static int flow_effect(sox_effects_chain_t * chain, unsigned n)
                                    &effp->obuf[effp->oend], &idone, &obeg);
   else {                 /* Run effect on each channel individually */
     sox_sample_t *obuf = &effp->obuf[effp->oend];
-    sox_size_t idone_last = 0, odone_last = 0; /* Initialised to prevent warning */
+    size_t idone_last = 0, odone_last = 0; /* Initialised to prevent warning */
 
     ibuf = &effp1->obuf[effp1->obeg];
     for (i = 0; i < idone; i += effp->flows)
@@ -199,12 +219,12 @@ static int flow_effect(sox_effects_chain_t * chain, unsigned n)
         chain->ibufc[f][i / effp->flows] = *ibuf++;
 
     for (f = 0; f < effp->flows; ++f) {
-      sox_size_t idonec = idone / effp->flows;
-      sox_size_t odonec = obeg / effp->flows;
+      size_t idonec = idone / effp->flows;
+      size_t odonec = obeg / effp->flows;
       int eff_status_c = effp->handler.flow(&chain->effects[n][f],
           chain->ibufc[f], chain->obufc[f], &idonec, &odonec);
       if (f && (idonec != idone_last || odonec != odone_last)) {
-        sox_fail("flowed asymmetrically!");
+        lsx_fail("flowed asymmetrically!");
         effstatus = SOX_EOF;
       }
       idone_last = idonec;
@@ -222,7 +242,7 @@ static int flow_effect(sox_effects_chain_t * chain, unsigned n)
     obeg = f * odone_last;
   }
 #if DEBUG_EFFECTS_CHAIN
-  sox_report("flow:  %5u%5u%5u%5u", pre_idone, pre_odone, idone, obeg);
+  lsx_report("flow:  %5u%5u%5u%5u", pre_idone, pre_odone, idone, obeg);
 #endif
   effp1->obeg += idone;
   if (effp1->obeg == effp1->oend)
@@ -239,27 +259,27 @@ static int flow_effect(sox_effects_chain_t * chain, unsigned n)
 }
 
 /* The same as flow_effect but with no input */
-static int drain_effect(sox_effects_chain_t * chain, unsigned n)
+static int drain_effect(sox_effects_chain_t * chain, size_t n)
 {
   sox_effect_t * effp = &chain->effects[n][0];
   int effstatus = SOX_SUCCESS;
-  sox_size_t i, f;
-  sox_size_t obeg = sox_globals.bufsiz - effp->oend;
+  size_t i, f;
+  size_t obeg = sox_globals.bufsiz - effp->oend;
 #if DEBUG_EFFECTS_CHAIN
-  sox_size_t pre_odone = obeg;
+  size_t pre_odone = obeg;
 #endif
 
   if (effp->flows == 1)   /* Run effect on all channels at once */
     effstatus = effp->handler.drain(effp, &effp->obuf[effp->oend], &obeg);
   else {                         /* Run effect on each channel individually */
     sox_sample_t *obuf = &effp->obuf[effp->oend];
-    sox_size_t odone_last = 0; /* Initialised to prevent warning */
+    size_t odone_last = 0; /* Initialised to prevent warning */
 
     for (f = 0; f < effp->flows; ++f) {
-      sox_size_t odonec = obeg / effp->flows;
+      size_t odonec = obeg / effp->flows;
       int eff_status_c = effp->handler.drain(&chain->effects[n][f], chain->obufc[f], &odonec);
       if (f && (odonec != odone_last)) {
-        sox_fail("drained asymmetrically!");
+        lsx_fail("drained asymmetrically!");
         effstatus = SOX_EOF;
       }
       odone_last = odonec;
@@ -274,7 +294,7 @@ static int drain_effect(sox_effects_chain_t * chain, unsigned n)
     obeg = f * odone_last;
   }
 #if DEBUG_EFFECTS_CHAIN
-  sox_report("drain: %5u%5u%5u%5u", 0, pre_odone, 0, obeg);
+  lsx_report("drain: %5u%5u%5u%5u", 0, pre_odone, 0, obeg);
 #endif
   if (!obeg)   /* This is the only thing that drain has and flow hasn't */
     effstatus = SOX_EOF;
@@ -288,21 +308,21 @@ static int drain_effect(sox_effects_chain_t * chain, unsigned n)
 int sox_flow_effects(sox_effects_chain_t * chain, int (* callback)(sox_bool all_done))
 {
   int flow_status = SOX_SUCCESS;
-  sox_size_t e, source_e = 0;               /* effect indices */
-  sox_size_t f, max_flows = 0;
+  size_t e, source_e = 0;               /* effect indices */
+  size_t f, max_flows = 0;
   sox_bool draining = sox_true;
 
   for (e = 0; e < chain->length; ++e) {
-    chain->effects[e][0].obuf = xmalloc(sox_globals.bufsiz * sizeof(chain->effects[e][0].obuf[0]));
+    chain->effects[e][0].obuf = lsx_malloc(sox_globals.bufsiz * sizeof(chain->effects[e][0].obuf[0]));
     chain->effects[e][0].obeg = chain->effects[e][0].oend = 0;
     max_flows = max(max_flows, chain->effects[e][0].flows);
   }
 
-  chain->ibufc = xcalloc(max_flows, sizeof(*chain->ibufc));
-  chain->obufc = xcalloc(max_flows, sizeof(*chain->obufc));
+  chain->ibufc = lsx_calloc(max_flows, sizeof(*chain->ibufc));
+  chain->obufc = lsx_calloc(max_flows, sizeof(*chain->obufc));
   for (f = 0; f < max_flows; ++f) {
-    chain->ibufc[f] = xcalloc(sox_globals.bufsiz / 2, sizeof(chain->ibufc[f][0]));
-    chain->obufc[f] = xcalloc(sox_globals.bufsiz / 2, sizeof(chain->obufc[f][0]));
+    chain->ibufc[f] = lsx_calloc(sox_globals.bufsiz / 2, sizeof(chain->ibufc[f][0]));
+    chain->obufc[f] = lsx_calloc(sox_globals.bufsiz / 2, sizeof(chain->obufc[f][0]));
   }
 
   e = chain->length - 1;
@@ -316,6 +336,8 @@ int sox_flow_effects(sox_effects_chain_t * chain, int (* callback)(sox_bool all_
       }
     } else if (have_imin && flow_effect(chain, e) == SOX_EOF) {
       flow_status = SOX_EOF;
+      if (e == chain->length - 1)
+        break;
       source_e = e;
       draining = sox_true;
     }
@@ -345,47 +367,90 @@ int sox_flow_effects(sox_effects_chain_t * chain, int (* callback)(sox_bool all_
   return flow_status;
 }
 
-sox_size_t sox_effects_clips(sox_effects_chain_t * chain)
+size_t sox_effects_clips(sox_effects_chain_t * chain)
 {
   unsigned i, f;
-  sox_size_t clips = 0;
+  size_t clips = 0;
   for (i = 1; i < chain->length - 1; ++i)
     for (f = 0; f < chain->effects[i][0].flows; ++f)
       clips += chain->effects[i][f].clips;
   return clips;
 }
 
-sox_size_t sox_stop_effect(sox_effects_chain_t * chain, sox_size_t e)
+size_t sox_stop_effect(sox_effect_t *effp)
 {
   unsigned f;
-  sox_effect_t * effp = &chain->effects[e][0];
-  sox_size_t clips = 0;
+  size_t clips = 0;
 
   for (f = 0; f < effp->flows; ++f) {
-    effp->handler.stop(&chain->effects[e][f]);
-    clips += chain->effects[e][f].clips;
+    effp[f].handler.stop(&effp[f]);
+    clips += effp[f].clips;
   }
   return clips;
 }
 
-/* Remove all effects from the chain */
+void sox_push_effect_last(sox_effects_chain_t *chain, sox_effect_t *effp)
+{
+  chain->effects[chain->length++] = effp;;
+} /* sox_push_effect_last */
+
+sox_effect_t *sox_pop_effect_last(sox_effects_chain_t *chain)
+{
+  if (chain->length > 0)
+  {
+    sox_effect_t *effp;
+    chain->length--;
+    effp = chain->effects[chain->length];
+    chain->effects[chain->length] = NULL;
+    return effp;
+  }
+  else
+    return NULL;
+} /* sox_pop_effect_last */
+
+/* Free resources related to effect.
+ * Note: This currently closes down the effect which might
+ * note be obvious from name.
+ */
+void sox_delete_effect(sox_effect_t *effp)
+{
+  size_t clips;
+  unsigned f;
+
+  if ((clips = sox_stop_effect(effp)) != 0)
+    lsx_warn("%s clipped %lu samples; decrease volume?",
+        effp->handler.name, (unsigned long)clips);
+  effp->handler.kill(effp); /* N.B. only one kill; not one per flow */
+  for (f = 0; f < effp->flows; ++f)
+    free(effp[f].priv);
+  free(effp);
+}
+
+void sox_delete_effect_last(sox_effects_chain_t *chain)
+{
+  if (chain->length > 0)
+  {
+    chain->length--;
+    sox_delete_effect(chain->effects[chain->length]);
+    chain->effects[chain->length] = NULL;
+  }
+} /* sox_delete_effect_last */
+
+/* Remove all effects from the chain.
+ * Note: This currently closes down the effect which might
+ * note be obvious from name.
+ */
 void sox_delete_effects(sox_effects_chain_t * chain)
 {
-  sox_size_t e, clips;
+  size_t e;
 
   for (e = 0; e < chain->length; ++e) {
-    sox_effect_t * effp = chain->effects[e];
-    if ((clips = sox_stop_effect(chain, e)) != 0)
-      sox_warn("%s clipped %u samples; decrease volume?",
-          chain->effects[e][0].handler.name, clips);
-    effp->handler.kill(effp);
-    free(chain->effects[e]);
+    sox_delete_effect(chain->effects[e]);
+    chain->effects[e] = NULL;
   }
   chain->length = 0;
 }
 
-
-
 /* Effects library: */
 
 sox_effect_fn_t sox_effect_fns[] = {
