@@ -1,5 +1,4 @@
-/*
- * libSoX ffmpeg formats.
+/* libSoX ffmpeg formats.
  *
  * Copyright 2007 Reuben Thomas <rrt@sc3d.org>
  *
@@ -26,20 +25,20 @@
  * THE SOFTWARE.
  *-----------------------------------------------------------------------
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or (at
+ * your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, write to the Free Software
- * Foundation, Fifth Floor, 51 Franklin Street, Boston, MA 02111-1301,
- * USA.  */
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 #include "sox_i.h"
 
@@ -47,11 +46,10 @@
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
-#include <ffmpeg/avformat.h>
+#include "ffmpeg.h"
 
 /* Private data for ffmpeg files */
-typedef struct ffmpeg
-{
+typedef struct {
   int audio_index;
   int audio_stream;
   AVStream *audio_st;
@@ -65,13 +63,10 @@ typedef struct ffmpeg
   AVPacket audio_pkt;
   uint8_t *audio_pkt_data;
   int audio_pkt_size;
-} *ffmpeg_t;
-
-assert_static(sizeof(struct ffmpeg) <= SOX_MAX_FILE_PRIVSIZE, 
-              /* else */ ffmpeg_PRIVSIZE_too_big);
+} priv_t;
 
 /* open a given stream. Return 0 if OK */
-static int stream_component_open(ffmpeg_t ffmpeg, int stream_index)
+static int stream_component_open(priv_t * ffmpeg, int stream_index)
 {
   AVFormatContext *ic = ffmpeg->ctxt;
   AVCodecContext *enc;
@@ -87,11 +82,16 @@ static int stream_component_open(ffmpeg_t ffmpeg, int stream_index)
 
   codec = avcodec_find_decoder(enc->codec_id);
   enc->workaround_bugs = 1;
+#if LIBAVCODEC_VERSION_INT < ((52<<16)+(0<<8)+0)
   enc->error_resilience = 1;
+#else
+  enc->error_recognition = 1;
+#endif
+
   if (!codec || avcodec_open(enc, codec) < 0)
     return -1;
   if (enc->codec_type != CODEC_TYPE_AUDIO) {
-    sox_fail("ffmpeg CODEC %x is not an audio CODEC", enc->codec_type);
+    lsx_fail("ffmpeg CODEC %x is not an audio CODEC", enc->codec_type);
     return -1;
   }
 
@@ -105,7 +105,7 @@ static int stream_component_open(ffmpeg_t ffmpeg, int stream_index)
   return 0;
 }
 
-static void stream_component_close(ffmpeg_t ffmpeg, int stream_index)
+static void stream_component_close(priv_t * ffmpeg, int stream_index)
 {
   AVFormatContext *ic = ffmpeg->ctxt;
   AVCodecContext *enc;
@@ -118,7 +118,7 @@ static void stream_component_close(ffmpeg_t ffmpeg, int stream_index)
 }
 
 /* Decode one audio frame and returns its uncompressed size */
-static int audio_decode_frame(ffmpeg_t ffmpeg, uint8_t *audio_buf, int buf_size)
+static int audio_decode_frame(priv_t * ffmpeg, uint8_t *audio_buf, int buf_size)
 {
   AVPacket *pkt = &ffmpeg->audio_pkt;
   int len1, data_size;
@@ -150,29 +150,29 @@ static int audio_decode_frame(ffmpeg_t ffmpeg, uint8_t *audio_buf, int buf_size)
 
 static int startread(sox_format_t * ft)
 {
-  ffmpeg_t ffmpeg = (ffmpeg_t)ft->priv;
+  priv_t * ffmpeg = (priv_t *)ft->priv;
   AVFormatParameters params;
   int ret;
-  unsigned i;
+  int i;
 
-  ffmpeg->audio_buf = xcalloc(1, AVCODEC_MAX_AUDIO_FRAME_SIZE);
+  ffmpeg->audio_buf = lsx_calloc(1, (size_t)AVCODEC_MAX_AUDIO_FRAME_SIZE);
 
   /* Signal audio stream not found */
   ffmpeg->audio_index = -1;
-  
+
   /* register all CODECs, demux and protocols */
   av_register_all();
 
   /* Open file and get format */
   memset(&params, 0, sizeof(params));
   if ((ret = av_open_input_file(&ffmpeg->ctxt, ft->filename, NULL, 0, &params)) < 0) {
-    sox_fail("ffmpeg cannot open file for reading: %s (code %d)", ft->filename, ret);
+    lsx_fail("ffmpeg cannot open file for reading: %s (code %d)", ft->filename, ret);
     return SOX_EOF;
   }
 
   /* Get CODEC parameters */
   if ((ret = av_find_stream_info(ffmpeg->ctxt)) < 0) {
-    sox_fail("ffmpeg could not find CODEC parameters for %s", ft->filename);
+    lsx_fail("ffmpeg could not find CODEC parameters for %s", ft->filename);
     return SOX_EOF;
   }
 
@@ -180,7 +180,7 @@ static int startread(sox_format_t * ft)
   av_read_play(ffmpeg->ctxt);
 
   /* Find audio stream (FIXME: allow different stream to be selected) */
-  for (i = 0; i < ffmpeg->ctxt->nb_streams; i++) {
+  for (i = 0; (unsigned)i < ffmpeg->ctxt->nb_streams; i++) {
     AVCodecContext *enc = ffmpeg->ctxt->streams[i]->codec;
     if (enc->codec_type == CODEC_TYPE_AUDIO && ffmpeg->audio_index < 0) {
       ffmpeg->audio_index = i;
@@ -192,16 +192,16 @@ static int startread(sox_format_t * ft)
   if (ffmpeg->audio_index < 0 ||
       stream_component_open(ffmpeg, ffmpeg->audio_index) < 0 ||
       ffmpeg->audio_stream < 0) {
-    sox_fail("ffmpeg could not open CODECs for %s", ft->filename);
+    lsx_fail("ffmpeg could not open CODECs for %s", ft->filename);
     return SOX_EOF;
   }
 
   /* Copy format info */
   ft->signal.rate = ffmpeg->audio_st->codec->sample_rate;
-  ft->signal.size = SOX_SIZE_16BIT;
-  ft->signal.encoding = SOX_ENCODING_SIGN2;
+  ft->encoding.bits_per_sample = 16;
+  ft->encoding.encoding = SOX_ENCODING_SIGN2;
   ft->signal.channels = ffmpeg->audio_st->codec->channels;
-  ft->length = 0; /* Currently we can't seek; no idea how to get this
+  ft->signal.length = 0; /* Currently we can't seek; no idea how to get this
                      info from ffmpeg anyway (in time, yes, but not in
                      samples); but ffmpeg *can* seek */
 
@@ -212,12 +212,12 @@ static int startread(sox_format_t * ft)
  * Read up to len samples of type sox_sample_t from file into buf[].
  * Return number of samples read.
  */
-static sox_size_t read(sox_format_t * ft, sox_sample_t *buf, sox_size_t len)
+static size_t read_samples(sox_format_t * ft, sox_sample_t *buf, size_t len)
 {
-  ffmpeg_t ffmpeg = (ffmpeg_t)ft->priv;
+  priv_t * ffmpeg = (priv_t *)ft->priv;
   AVPacket *pkt = &ffmpeg->audio_pkt;
   int ret;
-  sox_size_t nsamp = 0, nextra;
+  size_t nsamp = 0, nextra;
 
   /* Read data repeatedly until buf is full or no more can be read */
   do {
@@ -243,7 +243,7 @@ static sox_size_t read(sox_format_t * ft, sox_sample_t *buf, sox_size_t len)
  */
 static int stopread(sox_format_t * ft)
 {
-  ffmpeg_t ffmpeg = (ffmpeg_t)ft->priv;
+  priv_t * ffmpeg = (priv_t *)ft->priv;
 
   if (ffmpeg->audio_stream >= 0)
     stream_component_close(ffmpeg, ffmpeg->audio_stream);
@@ -251,7 +251,7 @@ static int stopread(sox_format_t * ft)
     av_close_input_file(ffmpeg->ctxt);
     ffmpeg->ctxt = NULL; /* safety */
   }
-  
+
   return SOX_SUCCESS;
 }
 
@@ -265,7 +265,7 @@ static AVStream *add_audio_stream(sox_format_t * ft, AVFormatContext *oc, enum C
 
   st = av_new_stream(oc, 1);
   if (!st) {
-    sox_fail("ffmpeg could not alloc stream");
+    lsx_fail("ffmpeg could not alloc stream");
     return NULL;
   }
 
@@ -282,7 +282,7 @@ static AVStream *add_audio_stream(sox_format_t * ft, AVFormatContext *oc, enum C
   return st;
 }
 
-static int open_audio(ffmpeg_t ffmpeg, AVStream *st)
+static int open_audio(priv_t * ffmpeg, AVStream *st)
 {
   AVCodecContext *c;
   AVCodec *codec;
@@ -292,17 +292,17 @@ static int open_audio(ffmpeg_t ffmpeg, AVStream *st)
   /* find the audio encoder */
   codec = avcodec_find_encoder(c->codec_id);
   if (!codec) {
-    sox_fail("ffmpeg CODEC not found");
+    lsx_fail("ffmpeg CODEC not found");
     return SOX_EOF;
   }
 
   /* open it */
   if (avcodec_open(c, codec) < 0) {
-    sox_fail("ffmpeg could not open CODEC");
+    lsx_fail("ffmpeg could not open CODEC");
     return SOX_EOF;
   }
 
-  ffmpeg->audio_buf = xmalloc(AVCODEC_MAX_AUDIO_FRAME_SIZE);
+  ffmpeg->audio_buf = lsx_malloc((size_t)AVCODEC_MAX_AUDIO_FRAME_SIZE);
 
   /* ugly hack for PCM codecs (will be removed ASAP with new PCM
      support to compute the input frame size in samples */
@@ -321,14 +321,14 @@ static int open_audio(ffmpeg_t ffmpeg, AVStream *st)
   } else
     ffmpeg->audio_input_frame_size = c->frame_size;
 
-  ffmpeg->samples = xmalloc((size_t)(ffmpeg->audio_input_frame_size * 2 * c->channels));
+  ffmpeg->samples = lsx_malloc((size_t)(ffmpeg->audio_input_frame_size * 2 * c->channels));
 
   return SOX_SUCCESS;
 }
 
 static int startwrite(sox_format_t * ft)
 {
-  ffmpeg_t ffmpeg = (ffmpeg_t)ft->priv;
+  priv_t * ffmpeg = (priv_t *)ft->priv;
 
   /* initialize libavcodec, and register all codecs and formats */
   av_register_all();
@@ -337,10 +337,10 @@ static int startwrite(sox_format_t * ft)
      mpeg. */
   ffmpeg->fmt = guess_format(NULL, ft->filename, NULL);
   if (!ffmpeg->fmt) {
-    sox_warn("ffmpeg could not deduce output format from file extension; using MPEG");
+    lsx_warn("ffmpeg could not deduce output format from file extension; using MPEG");
     ffmpeg->fmt = guess_format("mpeg", NULL, NULL);
     if (!ffmpeg->fmt) {
-      sox_fail("ffmpeg could not find suitable output format");
+      lsx_fail("ffmpeg could not find suitable output format");
       return SOX_EOF;
     }
   }
@@ -353,7 +353,7 @@ static int startwrite(sox_format_t * ft)
   }
   ffmpeg->ctxt->oformat = ffmpeg->fmt;
   snprintf(ffmpeg->ctxt->filename, sizeof(ffmpeg->ctxt->filename), "%s", ft->filename);
-  
+
   /* add the audio stream using the default format codecs
      and initialize the codecs */
   ffmpeg->audio_st = NULL;
@@ -362,34 +362,34 @@ static int startwrite(sox_format_t * ft)
     if (ffmpeg->audio_st == NULL)
       return SOX_EOF;
   }
-  
+
   /* set the output parameters (must be done even if no
      parameters). */
   if (av_set_parameters(ffmpeg->ctxt, NULL) < 0) {
-    sox_fail("ffmpeg invalid output format parameters");
+    lsx_fail("ffmpeg invalid output format parameters");
     return SOX_EOF;
   }
 
   /* Next line for debugging */
   /* dump_format(ffmpeg->ctxt, 0, ft->filename, 1); */
-  
+
   /* now that all the parameters are set, we can open the audio and
      codec and allocate the necessary encode buffers */
   if (ffmpeg->audio_st)
     if (open_audio(ffmpeg, ffmpeg->audio_st) == SOX_EOF)
       return SOX_EOF;
-  
+
   /* open the output file, if needed */
   if (!(ffmpeg->fmt->flags & AVFMT_NOFILE)) {
     if (url_fopen(&ffmpeg->ctxt->pb, ft->filename, URL_WRONLY) < 0) {
-      sox_fail("ffmpeg could not open `%s'", ft->filename);
+      lsx_fail("ffmpeg could not open `%s'", ft->filename);
       return SOX_EOF;
     }
   }
-  
+
   /* write the stream header, if any */
   av_write_header(ffmpeg->ctxt);
-  
+
   return SOX_SUCCESS;
 }
 
@@ -397,10 +397,10 @@ static int startwrite(sox_format_t * ft)
  * Write up to len samples of type sox_sample_t from buf[] into file.
  * Return number of samples written.
  */
-static sox_size_t write(sox_format_t * ft, const sox_sample_t *buf, sox_size_t len)
+static size_t write_samples(sox_format_t * ft, const sox_sample_t *buf, size_t len)
 {
-  ffmpeg_t ffmpeg = (ffmpeg_t)ft->priv;
-  sox_size_t nread = 0, nwritten = 0;
+  priv_t * ffmpeg = (priv_t *)ft->priv;
+  size_t nread = 0, nwritten = 0;
 
   /* Write data repeatedly until buf is empty */
   do {
@@ -422,10 +422,10 @@ static sox_size_t write(sox_format_t * ft, const sox_sample_t *buf, sox_size_t l
       pkt.flags |= PKT_FLAG_KEY;
       pkt.stream_index = ffmpeg->audio_st->index;
       pkt.data = ffmpeg->audio_buf;
-      
+
       /* write the compressed frame to the media file */
       if (av_write_frame(ffmpeg->ctxt, &pkt) != 0)
-        sox_fail("ffmpeg had error while writing audio frame");
+        lsx_fail("ffmpeg had error while writing audio frame");
 
       /* Increment nwritten whether write succeeded or not; we have to
          get rid of the input! */
@@ -442,8 +442,8 @@ static sox_size_t write(sox_format_t * ft, const sox_sample_t *buf, sox_size_t l
  */
 static int stopwrite(sox_format_t * ft)
 {
-  ffmpeg_t ffmpeg = (ffmpeg_t)ft->priv;
-  unsigned i;
+  priv_t * ffmpeg = (priv_t *)ft->priv;
+  int i;
 
   /* Close CODEC */
   if (ffmpeg->audio_st) {
@@ -451,19 +451,23 @@ static int stopwrite(sox_format_t * ft)
     free(ffmpeg->samples);
     free(ffmpeg->audio_buf);
   }
-  
+
   /* Write the trailer, if any */
   av_write_trailer(ffmpeg->ctxt);
-  
+
   /* Free the streams */
-  for (i = 0; i < ffmpeg->ctxt->nb_streams; i++) {
+  for (i = 0; (unsigned)i < ffmpeg->ctxt->nb_streams; i++) {
     av_freep(&ffmpeg->ctxt->streams[i]->codec);
     av_freep(&ffmpeg->ctxt->streams[i]);
   }
-  
+
   if (!(ffmpeg->fmt->flags & AVFMT_NOFILE)) {
     /* close the output file */
+#if (LIBAVFORMAT_VERSION_INT < 0x340000)
     url_fclose(&ffmpeg->ctxt->pb);
+#else
+    url_fclose(ffmpeg->ctxt->pb);
+#endif
   }
 
   /* Free the output context */
@@ -472,35 +476,28 @@ static int stopwrite(sox_format_t * ft)
   return SOX_SUCCESS;
 }
 
-
-/* Format file suffixes */
-/* For now, comment out formats built in to SoX */
-static const char *names[] = {
-  "ffmpeg", /* special type to force use of ffmpeg */
-  "mp4",
-  "m4a",
-  "avi",
-  "wmv",
-  "mpg",
-  NULL
-};
-
-/* Format descriptor */
-static sox_format_handler_t sox_ffmpeg_format = {
-  names,
-  SOX_FILE_NOSTDIO,
-  startread,
-  read,
-  stopread,
-  startwrite,
-  write,
-  stopwrite,
-  sox_format_nothing_seek
-};
-
-const sox_format_handler_t *sox_ffmpeg_format_fn(void);
-
-const sox_format_handler_t *sox_ffmpeg_format_fn(void)
+SOX_FORMAT_HANDLER(ffmpeg)
 {
-  return &sox_ffmpeg_format;
+  /* Format file suffixes */
+  /* For now, comment out formats built in to SoX */
+  static char const * const names[] = {
+    "ffmpeg", /* special type to force use of ffmpeg */
+    "mp4",
+    "m4a",
+    "avi",
+    "wmv",
+    "mpg",
+    NULL
+  };
+
+  static unsigned const write_encodings[] = {SOX_ENCODING_SIGN2, 16, 0, 0};
+
+  static sox_format_handler_t handler = {SOX_LIB_VERSION_CODE,
+    "Pseudo format to use libffmpeg", names, SOX_FILE_NOSTDIO,
+    startread, read_samples, stopread,
+    startwrite, write_samples, stopwrite,
+    NULL, write_encodings, NULL, sizeof(priv_t)
+  };
+
+  return &handler;
 }
