@@ -33,12 +33,16 @@
   #include <io.h>
 #endif
 
+#if HAVE_MAGIC
+  #include <magic.h>
+#endif
+
 static char const * detect_magic(sox_format_t * ft, char const * ext)
 {
   char data[256];
   size_t len = lsx_readbuf(ft, data, sizeof(data));
   #define MAGIC(type, p2, l2, d2, p1, l1, d1) if (len >= p1 + l1 && \
-      !memcmp(data + p1, d1, l1) && !memcmp(data + p2, d2, l2)) return #type;
+      !memcmp(data + p1, d1, (size_t)l1) && !memcmp(data + p2, d2, (size_t)l2)) return #type;
   MAGIC(voc   , 0, 0, ""     , 0, 20, "Creative Voice File\x1a")
   MAGIC(smp   , 0, 0, ""     , 0, 17, "SOUND SAMPLE DATA")
   MAGIC(wve   , 0, 0, ""     , 0, 15, "ALawSoundFile**")
@@ -76,6 +80,8 @@ static char const * detect_magic(sox_format_t * ft, char const * ext)
   MAGIC(sf    , 0, 0, ""     , 0,  4, "\144\243\003\0")
   MAGIC(sf    , 0, 0, ""     , 0,  4, "\0\003\243\144")
   MAGIC(sf    , 0, 0, ""     , 0,  4, "\144\243\004\0")
+  MAGIC(sox   , 0, 0, ""     , 0,  4, ".SoX")
+  MAGIC(sox   , 0, 0, ""     , 0,  4, "XoS.")
 
   if (ext && !strcasecmp(ext, "snd"))
   MAGIC(sndr  , 7, 1, ""     , 0,  2, "\0")
@@ -225,7 +231,7 @@ void sox_delete_comments(sox_comments_t * comments)
   *comments = 0;
 }
 
-char * sox_cat_comments(sox_comments_t comments)
+char * lsx_cat_comments(sox_comments_t comments)
 {
   sox_comments_t p = comments;
   size_t len = 0;
@@ -269,20 +275,20 @@ static void set_endiannesses(sox_format_t * ft)
   if (ft->handler.flags & SOX_FILE_ENDIAN) {
     if (ft->encoding.reverse_bytes == (sox_option_t)
         (!(ft->handler.flags & SOX_FILE_ENDBIG) != MACHINE_IS_BIGENDIAN))
-      sox_report("`%s': overriding file-type byte-order", ft->filename);
+      lsx_report("`%s': overriding file-type byte-order", ft->filename);
   } else if (ft->encoding.reverse_bytes == sox_true)
-    sox_report("`%s': overriding machine byte-order", ft->filename);
+    lsx_report("`%s': overriding machine byte-order", ft->filename);
 
   if (ft->encoding.reverse_bits == SOX_OPTION_DEFAULT)
     ft->encoding.reverse_bits = !!(ft->handler.flags & SOX_FILE_BIT_REV);
   else if (ft->encoding.reverse_bits == !(ft->handler.flags & SOX_FILE_BIT_REV))
-      sox_report("`%s': overriding file-type bit-order", ft->filename);
+      lsx_report("`%s': overriding file-type bit-order", ft->filename);
 
   if (ft->encoding.reverse_nibbles == SOX_OPTION_DEFAULT)
     ft->encoding.reverse_nibbles = !!(ft->handler.flags & SOX_FILE_NIB_REV);
   else
     if (ft->encoding.reverse_nibbles == !(ft->handler.flags & SOX_FILE_NIB_REV))
-      sox_report("`%s': overriding file-type nibble-order", ft->filename);
+      lsx_report("`%s': overriding file-type nibble-order", ft->filename);
 }
 
 static sox_bool is_seekable(sox_format_t const * ft)
@@ -312,47 +318,38 @@ static int sox_checkformat(sox_format_t * ft)
   return SOX_SUCCESS;
 }
 
-static char const * find_file_extension(char const * pathname)
+static sox_bool is_url(char const * text) /* detects only wget-supported URLs */
 {
-  /* First, chop off any path portions of filename.  This
-   * prevents the next search from considering that part. */
-  char const * result = LAST_SLASH(pathname);
-  if (!result)
-    result = pathname;
-
-  /* Now look for an filename extension */
-  result = strrchr(result, '.');
-  if (result)
-    ++result;
-  return result;
+  return !(
+      strncasecmp(text, "http:" , (size_t)5) &&
+      strncasecmp(text, "https:", (size_t)6) &&
+      strncasecmp(text, "ftp:"  , (size_t)4));
 }
 
-static sox_bool is_uri(char const * text)
-{
-  if (!isalpha((int)*text))
-    return sox_false;
-  ++text;
-  do {
-    if (!isalnum((int)*text) && !strchr("+-.", *text))
-      return sox_false;
-    ++text;
-  } while (*text && *text != ':');
-  return *text == ':';
-}
-
-static int xfclose(FILE * file, sox_bool is_process)
+static int xfclose(FILE * file, lsx_io_type io_type)
 {
   return
 #ifdef HAVE_POPEN
-    is_process? pclose(file) :
+    io_type != lsx_io_file? pclose(file) :
 #endif
     fclose(file);
 }
 
-static FILE * xfopen(char const * identifier, char const * mode, sox_bool * is_process)
+static FILE * xfopen(char const * identifier, char const * mode, lsx_io_type * io_type)
 {
-  *is_process = sox_false;
-  if (is_uri(identifier)) {
+  *io_type = lsx_io_file;
+
+  if (*identifier == '|') {
+    FILE * f = NULL;
+#ifdef HAVE_POPEN
+    f = popen(identifier + 1, "r");
+    *io_type = lsx_io_pipe;
+#else
+    lsx_fail("this build of SoX cannot open pipes");
+#endif
+    return f;
+  }
+  else if (is_url(identifier)) {
     FILE * f = NULL;
 #ifdef HAVE_POPEN
     char const * const command_format = "wget --no-check-certificate -q -O- \"%s\"";
@@ -360,9 +357,9 @@ static FILE * xfopen(char const * identifier, char const * mode, sox_bool * is_p
     sprintf(command, command_format, identifier);
     f = popen(command, "r");
     free(command);
-    *is_process = sox_true;
+    *io_type = lsx_io_url;
 #else
-    sox_fail("open URL support has not been built into SoX");
+    lsx_fail("this build of SoX cannot open URLs");
 #endif
     return f;
   }
@@ -377,34 +374,40 @@ sox_format_t * sox_open_read(
 {
   sox_format_t * ft = lsx_calloc(1, sizeof(*ft));
   sox_format_handler_t const * handler;
+  char const * const io_types[] = {"file", "pipe", "file URL"};
+  char const * type = "";
 
   if (filetype) {
     if (!(handler = sox_find_format(filetype, sox_false))) {
-      sox_fail("no handler for given file type `%s'", filetype);
+      lsx_fail("no handler for given file type `%s'", filetype);
       goto error;
     }
     ft->handler = *handler;
   }
 
   if (!(ft->handler.flags & SOX_FILE_NOSTDIO)) {
-    sox_size_t   input_bufsiz = sox_globals.input_bufsiz?
+    size_t   input_bufsiz = sox_globals.input_bufsiz?
         sox_globals.input_bufsiz : sox_globals.bufsiz;
 
     if (!strcmp(path, "-")) { /* Use stdin if the filename is "-" */
       if (sox_globals.stdin_in_use_by) {
-        sox_fail("`-' (stdin) already in use by `%s'", sox_globals.stdin_in_use_by);
+        lsx_fail("`-' (stdin) already in use by `%s'", sox_globals.stdin_in_use_by);
         goto error;
       }
       sox_globals.stdin_in_use_by = "audio input";
       SET_BINARY_MODE(stdin);
       ft->fp = stdin;
     }
-    else if ((ft->fp = xfopen(path, "rb", &ft->is_process)) == NULL) {
-      sox_fail("can't open input file `%s': %s", path, strerror(errno));
-      goto error;
+    else {
+      ft->fp = xfopen(path, "rb", &ft->io_type);
+      type = io_types[ft->io_type];
+      if (ft->fp == NULL) {
+        lsx_fail("can't open input %s `%s': %s", type, path, strerror(errno));
+        goto error;
+      }
     }
     if (setvbuf (ft->fp, NULL, _IOFBF, sizeof(char) * input_bufsiz)) {
-      sox_fail("Can't set read buffer");
+      lsx_fail("Can't set read buffer");
       goto error;
     }
     ft->seekable = is_seekable(ft);
@@ -412,34 +415,54 @@ sox_format_t * sox_open_read(
 
   if (!filetype) {
     if (ft->seekable) {
-      filetype = detect_magic(ft, find_file_extension(path));
+      filetype = detect_magic(ft, lsx_find_file_extension(path));
       lsx_rewind(ft);
+#if HAVE_MAGIC
+      if (!filetype) {
+        static magic_t magic;
+        if (!magic) {
+          magic = magic_open(MAGIC_MIME | MAGIC_SYMLINK);
+          if (magic)
+            magic_load(magic, NULL);
+        }
+        if (magic)
+          filetype = magic_file(magic, path);
+        if (filetype && (
+              !strcmp(filetype, "application/octet-stream") ||
+              !strncmp(filetype, "text/plain", (size_t)10) ))
+          filetype = NULL;
+      }
+#endif
     }
     if (filetype) {
-      sox_report("detected file format type `%s'", filetype);
+      lsx_report("detected file format type `%s'", filetype);
       if (!(handler = sox_find_format(filetype, sox_false))) {
-        sox_fail("no handler for detected file type `%s'", filetype);
+        lsx_fail("no handler for detected file type `%s'", filetype);
         goto error;
       }
     }
     else {
-      if (!(filetype = find_file_extension(path))) {
-        sox_fail("can't determine type of `%s'", path);
+      if (ft->io_type == lsx_io_pipe) {
+        filetype = "sox";
+        lsx_report("assuming input pipe `%s' has file-type `sox'", path);
+      }
+      else if (!(filetype = lsx_find_file_extension(path))) {
+        lsx_fail("can't determine type of %s `%s'", type, path);
         goto error;
       }
       if (!(handler = sox_find_format(filetype, sox_true))) {
-        sox_fail("no handler for file extension `%s'", filetype);
+        lsx_fail("no handler for file extension `%s'", filetype);
         goto error;
       }
     }
     ft->handler = *handler;
     if (ft->handler.flags & SOX_FILE_NOSTDIO) {
-      xfclose(ft->fp, ft->is_process);
+      xfclose(ft->fp, ft->io_type);
       ft->fp = NULL;
     }
   }
   if (!ft->handler.startread && !ft->handler.read) {
-    sox_fail("file type `%s' isn't readable", filetype);
+    lsx_fail("file type `%s' isn't readable", filetype);
     goto error;
   }
 
@@ -457,7 +480,7 @@ sox_format_t * sox_open_read(
   ft->priv = lsx_calloc(1, ft->handler.priv_size);
   /* Read and write starters can change their formats. */
   if (ft->handler.startread && (*ft->handler.startread)(ft) != SOX_SUCCESS) {
-    sox_fail("can't open input file `%s': %s", ft->filename, ft->sox_errstr);
+    lsx_fail("can't open input %s `%s': %s", type, ft->filename, ft->sox_errstr);
     goto error;
   }
 
@@ -469,11 +492,11 @@ sox_format_t * sox_open_read(
 
   if (sox_checkformat(ft) == SOX_SUCCESS)
     return ft;
-  sox_fail("bad input format for file `%s': %s", ft->filename, ft->sox_errstr);
+  lsx_fail("bad input format for %s `%s': %s", type, ft->filename, ft->sox_errstr);
 
 error:
   if (ft->fp && ft->fp != stdin)
-    xfclose(ft->fp, ft->is_process);
+    xfclose(ft->fp, ft->io_type);
   free(ft->priv);
   free(ft->filename);
   free(ft->filetype);
@@ -495,7 +518,7 @@ sox_bool sox_format_supports_encoding(
   assert(path);
   assert(encoding);
   if (!filetype)
-    filetype = find_file_extension(path);
+    filetype = lsx_find_file_extension(path);
 
   if (!filetype || !(handler = sox_find_format(filetype, is_file_extension)) ||
       !handler->write_formats)
@@ -541,7 +564,7 @@ static void set_output_format(sox_format_t * ft)
         }
         if (ft->signal.rate == HUGE_VAL)
           ft->signal.rate = max;
-        sox_warn("%s can't encode at %gHz; using %gHz", ft->handler.names[0], given, ft->signal.rate);
+        lsx_warn("%s can't encode at %gHz; using %gHz", ft->handler.names[0], given, ft->signal.rate);
       }
     }
   }
@@ -551,15 +574,15 @@ static void set_output_format(sox_format_t * ft)
   if (ft->handler.flags & SOX_FILE_CHANS) {
     if (ft->signal.channels == 1 && !(ft->handler.flags & SOX_FILE_MONO)) {
       ft->signal.channels = (ft->handler.flags & SOX_FILE_STEREO)? 2 : 4;
-      sox_warn("%s can't encode mono; setting channels to %u", ft->handler.names[0], ft->signal.channels);
+      lsx_warn("%s can't encode mono; setting channels to %u", ft->handler.names[0], ft->signal.channels);
     } else
     if (ft->signal.channels == 2 && !(ft->handler.flags & SOX_FILE_STEREO)) {
       ft->signal.channels = (ft->handler.flags & SOX_FILE_QUAD)? 4 : 1;
-      sox_warn("%s can't encode stereo; setting channels to %u", ft->handler.names[0], ft->signal.channels);
+      lsx_warn("%s can't encode stereo; setting channels to %u", ft->handler.names[0], ft->signal.channels);
     } else
     if (ft->signal.channels == 4 && !(ft->handler.flags & SOX_FILE_QUAD)) {
       ft->signal.channels = (ft->handler.flags & SOX_FILE_STEREO)? 2 : 1;
-      sox_warn("%s can't encode quad; setting channels to %u", ft->handler.names[0], ft->signal.channels);
+      lsx_warn("%s can't encode quad; setting channels to %u", ft->handler.names[0], ft->signal.channels);
     }
   } else ft->signal.channels = max(ft->signal.channels, 1);
 
@@ -574,7 +597,7 @@ static void set_output_format(sox_format_t * ft)
       while (enc_arg(unsigned));
     }
     if (e != ft->encoding.encoding) {
-      sox_warn("%s can't encode %s", ft->handler.names[0], sox_encodings_info[ft->encoding.encoding].desc);
+      lsx_warn("%s can't encode %s", ft->handler.names[0], sox_encodings_info[ft->encoding.encoding].desc);
       ft->encoding.encoding = 0;
     }
     else {
@@ -602,7 +625,7 @@ static void set_output_format(sox_format_t * ft)
       if (given_size) {
         if (found)
           ft->encoding.bits_per_sample = given_size;
-        else sox_warn("%s can't encode %s to %u-bit", ft->handler.names[0], sox_encodings_info[ft->encoding.encoding].desc, given_size);
+        else lsx_warn("%s can't encode %s to %u-bit", ft->handler.names[0], sox_encodings_info[ft->encoding.encoding].desc, given_size);
       }
     }
   }
@@ -614,7 +637,7 @@ static void set_output_format(sox_format_t * ft)
     while (s != ft->encoding.bits_per_sample && (e = enc_arg(sox_encoding_t)))
       while ((s = enc_arg(unsigned)) && s != ft->encoding.bits_per_sample);
     if (s != ft->encoding.bits_per_sample) {
-      sox_warn("%s can't encode to %u-bit", ft->handler.names[0], ft->encoding.bits_per_sample);
+      lsx_warn("%s can't encode to %u-bit", ft->handler.names[0], ft->encoding.bits_per_sample);
       ft->encoding.bits_per_sample = 0;
     }
     else ft->encoding.encoding = e;
@@ -676,37 +699,37 @@ sox_format_t * sox_open_write(
   sox_format_handler_t const * handler;
 
   if (!path || !signal) {
-    sox_fail("must specify file name and signal parameters to write file");
+    lsx_fail("must specify file name and signal parameters to write file");
     goto error;
   }
 
   if (filetype) {
     if (!(handler = sox_find_format(filetype, sox_false))) {
-      sox_fail("no handler for given file type `%s'", filetype);
+      lsx_fail("no handler for given file type `%s'", filetype);
       goto error;
     }
     ft->handler = *handler;
   }
   else {
-    if (!(filetype = find_file_extension(path))) {
-      sox_fail("can't determine type of `%s'", path);
+    if (!(filetype = lsx_find_file_extension(path))) {
+      lsx_fail("can't determine type of `%s'", path);
       goto error;
     }
     if (!(handler = sox_find_format(filetype, sox_true))) {
-      sox_fail("no handler for file extension `%s'", filetype);
+      lsx_fail("no handler for file extension `%s'", filetype);
       goto error;
     }
     ft->handler = *handler;
   }
   if (!ft->handler.startwrite && !ft->handler.write) {
-    sox_fail("file type `%s' isn't writeable", filetype);
+    lsx_fail("file type `%s' isn't writeable", filetype);
     goto error;
   }
 
   if (!(ft->handler.flags & SOX_FILE_NOSTDIO)) {
     if (!strcmp(path, "-")) { /* Use stdout if the filename is "-" */
       if (sox_globals.stdout_in_use_by) {
-        sox_fail("`-' (stdout) already in use by `%s'", sox_globals.stdout_in_use_by);
+        lsx_fail("`-' (stdout) already in use by `%s'", sox_globals.stdout_in_use_by);
         goto error;
       }
       sox_globals.stdout_in_use_by = "audio output";
@@ -717,11 +740,11 @@ sox_format_t * sox_open_write(
       struct stat st;
       if (!stat(path, &st) && (st.st_mode & S_IFMT) == S_IFREG &&
           (overwrite_permitted && !overwrite_permitted(path))) {
-        sox_fail("permission to overwrite '%s' denied", path);
+        lsx_fail("permission to overwrite '%s' denied", path);
         goto error;
       }
       if ((ft->fp = fopen(path, "wb")) == NULL) {
-        sox_fail("can't open output file `%s': %s", path, strerror(errno));
+        lsx_fail("can't open output file `%s': %s", path, strerror(errno));
         goto error;
       }
     }
@@ -729,7 +752,7 @@ sox_format_t * sox_open_write(
     /* stdout tends to be line-buffered.  Override this */
     /* to be Full Buffering. */
     if (setvbuf (ft->fp, NULL, _IOFBF, sizeof(char) * sox_globals.bufsiz)) {
-      sox_fail("Can't set write buffer");
+      lsx_fail("Can't set write buffer");
       goto error;
     }
     ft->seekable = is_seekable(ft);
@@ -759,23 +782,23 @@ sox_format_t * sox_open_write(
     ft->signal.length = ft->signal.length * ft->signal.rate / signal->rate *
       ft->signal.channels / signal->channels + .5;
 
-  if ((ft->handler.flags & SOX_FILE_REWIND) && !ft->signal.length && !ft->seekable)
-    sox_warn("can't seek in output file `%s'; length in file header will be unspecified", ft->filename);
+  if ((ft->handler.flags & SOX_FILE_REWIND) && strcmp(ft->filetype, "sox") && !ft->signal.length && !ft->seekable)
+    lsx_warn("can't seek in output file `%s'; length in file header will be unspecified", ft->filename);
 
   ft->priv = lsx_calloc(1, ft->handler.priv_size);
   /* Read and write starters can change their formats. */
   if (ft->handler.startwrite && (ft->handler.startwrite)(ft) != SOX_SUCCESS){
-    sox_fail("can't open output file `%s': %s", ft->filename, ft->sox_errstr);
+    lsx_fail("can't open output file `%s': %s", ft->filename, ft->sox_errstr);
     goto error;
   }
 
   if (sox_checkformat(ft) == SOX_SUCCESS)
     return ft;
-  sox_fail("bad format for output file `%s': %s", ft->filename, ft->sox_errstr);
+  lsx_fail("bad format for output file `%s': %s", ft->filename, ft->sox_errstr);
 
 error:
   if (ft->fp && ft->fp != stdout)
-    xfclose(ft->fp, ft->is_process);
+    xfclose(ft->fp, ft->io_type);
   free(ft->priv);
   free(ft->filename);
   free(ft->filetype);
@@ -783,48 +806,48 @@ error:
   return NULL;
 }
 
-sox_size_t sox_read(sox_format_t * ft, sox_sample_t * buf, sox_size_t len)
+size_t sox_read(sox_format_t * ft, sox_sample_t * buf, size_t len)
 {
-  sox_size_t actual = ft->handler.read? (*ft->handler.read)(ft, buf, len) : 0;
+  size_t actual = ft->handler.read? (*ft->handler.read)(ft, buf, len) : 0;
   return (actual > len? 0 : actual);
 }
 
-sox_size_t sox_write(sox_format_t * ft, const sox_sample_t *buf, sox_size_t len)
+size_t sox_write(sox_format_t * ft, const sox_sample_t *buf, size_t len)
 {
-  sox_size_t ret = ft->handler.write? (*ft->handler.write)(ft, buf, len) : 0;
+  size_t ret = ft->handler.write? (*ft->handler.write)(ft, buf, len) : 0;
   ft->olength += ret;
   return ret;
 }
 
 int sox_close(sox_format_t * ft)
 {
-  int rc = SOX_SUCCESS;
+  int result = SOX_SUCCESS;
 
   if (ft->mode == 'r')
-    rc = ft->handler.stopread? (*ft->handler.stopread)(ft) : SOX_SUCCESS;
+    result = ft->handler.stopread? (*ft->handler.stopread)(ft) : SOX_SUCCESS;
   else {
     if (ft->handler.flags & SOX_FILE_REWIND) {
       if (ft->olength != ft->signal.length && ft->seekable) {
-        rc = lsx_seeki(ft, 0, 0);
-        if (rc == SOX_SUCCESS)
-          rc = ft->handler.stopwrite? (*ft->handler.stopwrite)(ft)
+        result = lsx_seeki(ft, (off_t)0, 0);
+        if (result == SOX_SUCCESS)
+          result = ft->handler.stopwrite? (*ft->handler.stopwrite)(ft)
              : ft->handler.startwrite?(*ft->handler.startwrite)(ft) : SOX_SUCCESS;
       }
     }
-    else rc = ft->handler.stopwrite? (*ft->handler.stopwrite)(ft) : SOX_SUCCESS;
+    else result = ft->handler.stopwrite? (*ft->handler.stopwrite)(ft) : SOX_SUCCESS;
   }
 
   if (ft->fp && ft->fp != stdin && ft->fp != stdout)
-    xfclose(ft->fp, ft->is_process);
+    xfclose(ft->fp, ft->io_type);
   free(ft->filename);
   free(ft->filetype);
   sox_delete_comments(&ft->oob.comments);
 
   free(ft);
-  return rc;
+  return result;
 }
 
-int sox_seek(sox_format_t * ft, sox_size_t offset, int whence)
+int sox_seek(sox_format_t * ft, uint64_t offset, int whence)
 {
     /* FIXME: Implement SOX_SEEK_CUR and SOX_SEEK_END. */
     if (whence != SOX_SEEK_SET)
@@ -846,7 +869,8 @@ static int strcaseends(char const * str, char const * end)
 
 sox_bool sox_is_playlist(char const * filename)
 {
-  return strcaseends(filename, ".m3u") || strcaseends(filename, ".pls");
+  return *filename != '|' && 
+    (strcaseends(filename, ".m3u") || strcaseends(filename, ".pls"));
 }
 
 int sox_parse_playlist(sox_playlist_callback_t callback, void * p, char const * const listname)
@@ -857,8 +881,8 @@ int sox_parse_playlist(sox_playlist_callback_t callback, void * p, char const *
   char * text = lsx_malloc(text_length + 1);
   char * dirname = lsx_strdup(listname);
   char * slash_pos = LAST_SLASH(dirname);
-  sox_bool is_process;
-  FILE * file = xfopen(listname, "r", &is_process);
+  lsx_io_type io_type;
+  FILE * file = xfopen(listname, "r", &io_type);
   char * filename;
   int c, result = SOX_SUCCESS;
 
@@ -868,7 +892,7 @@ int sox_parse_playlist(sox_playlist_callback_t callback, void * p, char const *
     *slash_pos = '\0';
 
   if (file == NULL) {
-    sox_fail("Can't open playlist file `%s': %s", listname, strerror(errno));
+    lsx_fail("Can't open playlist file `%s': %s", listname, strerror(errno));
     result = SOX_EOF;
   }
   else {
@@ -898,14 +922,14 @@ int sox_parse_playlist(sox_playlist_callback_t callback, void * p, char const *
       text[end] = '\0';
       if (is_pls) {
         char dummy;
-        if (!strncasecmp(text, "file", 4) && sscanf(text + 4, "%*u=%c", &dummy) == 1)
+        if (!strncasecmp(text, "file", (size_t) 4) && sscanf(text + 4, "%*u=%c", &dummy) == 1)
           begin = strchr(text + 5, '=') - text + 1;
         else end = 0;
       }
       if (begin != end) {
         char const * id = text + begin;
 
-        if (!dirname[0] || is_uri(id) || IS_ABSOLUTE(id))
+        if (!dirname[0] || is_url(id) || IS_ABSOLUTE(id))
           filename = lsx_strdup(id);
         else {
           filename = lsx_malloc(strlen(dirname) + strlen(id) + 2);
@@ -920,10 +944,13 @@ int sox_parse_playlist(sox_playlist_callback_t callback, void * p, char const *
     } while (c != EOF);
 
     if (ferror(file)) {
-      sox_fail("error reading playlist file `%s': %s", listname, strerror(errno));
+      lsx_fail("error reading playlist file `%s': %s", listname, strerror(errno));
+      result = SOX_EOF;
+    }
+    if (xfclose(file, io_type) && io_type == lsx_io_url) {
+      lsx_fail("error reading playlist file URL `%s'", listname);
       result = SOX_EOF;
     }
-    xfclose(file, is_process);
   }
   free(text);
   free(dirname);
@@ -934,7 +961,7 @@ int sox_parse_playlist(sox_playlist_callback_t callback, void * p, char const *
 
 #include <ltdl.h>
 #define MAX_FORMATS 256 /* FIXME: Use a vector, not a fixed-size array */
-#define MAX_NAME_LEN 1024 /* FIXME: Use vasprintf */
+#define MAX_NAME_LEN (size_t)1024 /* FIXME: Use vasprintf */
 
 static sox_bool plugins_initted = sox_false;
 
@@ -952,11 +979,11 @@ static int init_format(const char *file, lt_ptr data)
 
   (void)data;
   if (start && (start += sizeof(prefix) - 1) < end) {
-    int ret = snprintf(fnname, MAX_NAME_LEN, "sox_%.*s_format_fn", end - start, start);
-    if (ret > 0 && ret < MAX_NAME_LEN) {
+    int ret = snprintf(fnname, MAX_NAME_LEN, "sox_%.*s_format_fn", (int)(end - start), start);
+    if (ret > 0 && ret < (int)MAX_NAME_LEN) {
       union {sox_format_fn_t fn; lt_ptr ptr;} ltptr;
       ltptr.ptr = lt_dlsym(lth, fnname);
-      sox_debug("opening format plugin `%s': library %p, entry point %p\n", fnname, (void *)lth, ltptr.ptr);
+      lsx_debug("opening format plugin `%s': library %p, entry point %p\n", fnname, (void *)lth, ltptr.ptr);
       if (nformats < MAX_FORMATS && ltptr.fn && (ltptr.fn()->sox_lib_version_code & ~255) == (SOX_LIB_VERSION_CODE & ~255))
         sox_format_fns[nformats++].fn = ltptr.fn;
     }
@@ -966,10 +993,13 @@ static int init_format(const char *file, lt_ptr data)
 
 int sox_format_init(void) /* Find & load format handlers.  */
 {
+  sox_format_handler_t const * sox_sox_format_fn(void);
   int ret;
 
+  sox_format_fns[nformats++].fn = sox_sox_format_fn;
+
   if ((ret = lt_dlinit()) != 0) {
-    sox_fail("lt_dlinit failed with %d error(s): %s", ret, lt_dlerror());
+    lsx_fail("lt_dlinit failed with %d error(s): %s", ret, lt_dlerror());
     return SOX_EOF;
   }
   plugins_initted = sox_true;
@@ -982,7 +1012,7 @@ void sox_format_quit(void) /* Cleanup things.  */
 {
   int ret;
   if (plugins_initted && (ret = lt_dlexit()) != 0)
-    sox_fail("lt_dlexit failed with %d error(s): %s", ret, lt_dlerror());
+    lsx_fail("lt_dlexit failed with %d error(s): %s", ret, lt_dlerror());
 }
 
 #else /* Static format handlers */
@@ -1016,7 +1046,7 @@ void sox_format_quit(void) {}
  */
 sox_format_handler_t const * sox_find_format(char const * name, sox_bool no_dev)
 {
-  sox_size_t f, n;
+  size_t f, n;
 
   if (name) for (f = 0; sox_format_fns[f].fn; ++f) {
     sox_format_handler_t const * handler = sox_format_fns[f].fn();
