@@ -6,20 +6,20 @@
         Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
-   The GNU C Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 2.1 of the License, or (at your option) any later version.
-
-   The GNU C Library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with the GNU C Library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307 USA.  */
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or (at
+ * your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 /* This tells Alpha OSF/1 not to define a getopt prototype in <stdio.h>.
    Ditto for AIX 3.2 and <stdlib.h>.  */
@@ -59,6 +59,11 @@
 # endif
 #endif
 
+/* For SoX, always compile this */
+#if defined ELIDE_CODE
+#undef ELIDE_CODE
+#endif
+
 #ifndef ELIDE_CODE
 
 
@@ -119,7 +124,7 @@
    Also, when `ordering' is RETURN_IN_ORDER,
    each non-option ARGV-element is returned here.  */
 
-char *optarg;
+char *optarg = NULL;
 
 /* Index in ARGV of the next element to be scanned.
    This is used for communication to and from the caller
@@ -218,13 +223,11 @@ static char *posixly_correct;
    whose names are inconsistent.  */
 
 #ifndef getenv
-extern char *getenv ();
+extern char *getenv (char const *);
 #endif
 
 static char *
-my_index (str, chr)
-     const char *str;
-     int chr;
+my_index (char const * str, int chr)
 {
   while (*str)
     {
@@ -261,7 +264,7 @@ static int last_nonopt;
 #ifdef _LIBC
 /* Stored original parameters.
    XXX This is no good solution.  We should rather copy the args so
-   that we can compare them later.  But we must not use xmalloc(3).  */
+   that we can compare them later.  But we must not use malloc(3).  */
 extern int __libc_argc;
 extern char **__libc_argv;
 
@@ -326,7 +329,7 @@ exchange (argv)
     {
       /* We must extend the array.  The user plays games with us and
          presents new arguments.  */
-      char *new_str = xmalloc (top + 1);
+      char *new_str = lsx_malloc (top + 1);
       if (new_str == NULL)
         nonoption_flags_len = nonoption_flags_max_len = 0;
       else
@@ -438,7 +441,7 @@ _getopt_initialize (argc, argv, optstring)
               if (nonoption_flags_max_len < argc)
                 nonoption_flags_max_len = argc;
               __getopt_nonoption_flags =
-                (char *) xmalloc (nonoption_flags_max_len);
+                lsx_malloc (nonoption_flags_max_len);
               if (__getopt_nonoption_flags == NULL)
                 nonoption_flags_max_len = -1;
               else
@@ -450,6 +453,8 @@ _getopt_initialize (argc, argv, optstring)
     }
   else
     nonoption_flags_len = 0;
+#else
+  (void)argc, (void)argv;
 #endif
 
   return optstring;
@@ -512,13 +517,21 @@ _getopt_initialize (argc, argv, optstring)
    long-named options.  */
 
 int
-_getopt_internal (argc, argv, optstring, longopts, longind, long_only)
-     int argc;
-     char *const *argv;
-     const char *optstring;
-     const struct option *longopts;
-     int *longind;
-     int long_only;
+_getopt_internal (
+     int argc,
+     char *const *argv,
+     const char *optstring,
+     const struct option *longopts,
+     int *longind,
+     int long_only);
+int
+_getopt_internal (
+     int argc,
+     char *const *argv,
+     const char *optstring,
+     const struct option *longopts,
+     int *longind,
+     int long_only)
 {
   int print_errors = opterr;
   if (optstring[0] == ':')
@@ -659,7 +672,7 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
       /* Test all long options for either exact match
          or abbreviated matches.  */
       for (p = longopts, option_index = 0; p->name; p++, option_index++)
-        if (!strncmp (p->name, nextchar, nameend - nextchar))
+        if (!strncmp (p->name, nextchar, (unsigned)(nameend - nextchar)))
           {
             if ((unsigned int) (nameend - nextchar)
                 == (unsigned int) strlen (p->name))
@@ -997,7 +1010,7 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
         /* Test all long options for either exact match
            or abbreviated matches.  */
         for (p = longopts, option_index = 0; p->name; p++, option_index++)
-          if (!strncmp (p->name, nextchar, nameend - nextchar))
+          if (!strncmp (p->name, nextchar, (unsigned)(nameend - nextchar)))
             {
               if ((unsigned int) (nameend - nextchar) == strlen (p->name))
                 {
