@@ -1,8 +1,7 @@
-/*
- * libSoX Macintosh HCOM format.
+/* libSoX Macintosh HCOM format.
  * These are really FSSD type files with Huffman compression,
  * in MacBinary format.
- * To do: make the MacBinary format optional (so that .data files
+ * TODO: make the MacBinary format optional (so that .data files
  * are also acceptable).  (How to do this on output?)
  *
  * September 25, 1991
@@ -27,6 +26,22 @@
 #include <stdlib.h>
 #include <errno.h>
 
+/* FIXME: eliminate these 2 functions */
+
+static void put32_be(unsigned char **p, int32_t val)
+{
+  *(*p)++ = (val >> 24) & 0xff;
+  *(*p)++ = (val >> 16) & 0xff;
+  *(*p)++ = (val >> 8) & 0xff;
+  *(*p)++ = val & 0xff;
+}
+
+static void put16_be(unsigned char **p, int val)
+{
+  *(*p)++ = (val >> 8) & 0xff;
+  *(*p)++ = val & 0xff;
+}
+
 /* Dictionary entry for Huffman (de)compression */
 typedef struct {
         long frequ;
@@ -34,29 +49,33 @@ typedef struct {
         short dict_rightson;
 } dictent;
 
-/* Private data used by reader */
-struct readpriv {
-        /* Static data from the header */
-        dictent *dictionary;
-        int32_t checksum;
-        int deltacompression;
-        /* Engine state */
-        long huffcount;
-        long cksum;
-        int dictentry;
-        int nrbits;
-        uint32_t current;
-        short sample;
-        /* Dictionary */
-        dictent *de;
-        int32_t new_checksum;
-        int nbits;
-        int32_t curword;
-};
+typedef struct {
+  /* Static data from the header */
+  dictent *dictionary;
+  int32_t checksum;
+  int deltacompression;
+  /* Engine state */
+  long huffcount;
+  long cksum;
+  int dictentry;
+  int nrbits;
+  uint32_t current;
+  short sample;
+  /* Dictionary */
+  dictent *de;
+  int32_t new_checksum;
+  int nbits;
+  int32_t curword;
 
-static int sox_hcomstartread(sox_format_t * ft)
+  /* Private data used by writer */
+  unsigned char *data;          /* Buffer allocated with lsx_malloc */
+  size_t size;               /* Size of allocated buffer */
+  size_t pos;                /* Where next byte goes */
+} priv_t;
+
+static int startread(sox_format_t * ft)
 {
-        struct readpriv *p = (struct readpriv *) ft->priv;
+        priv_t *p = (priv_t *) ft->priv;
         int i;
         char buf[5];
         uint32_t datasize, rsrcsize;
@@ -66,73 +85,73 @@ static int sox_hcomstartread(sox_format_t * ft)
 
 
         /* Skip first 65 bytes of header */
-        rc = sox_skipbytes(ft, 65);
+        rc = lsx_skipbytes(ft, (size_t) 65);
         if (rc)
             return rc;
 
         /* Check the file type (bytes 65-68) */
-        if (sox_reads(ft, buf, 4) == SOX_EOF || strncmp(buf, "FSSD", 4) != 0)
+        if (lsx_reads(ft, buf, (size_t)4) == SOX_EOF || strncmp(buf, "FSSD", (size_t)4) != 0)
         {
-                sox_fail_errno(ft,SOX_EHDR,"Mac header type is not FSSD");
+                lsx_fail_errno(ft,SOX_EHDR,"Mac header type is not FSSD");
                 return (SOX_EOF);
         }
 
         /* Skip to byte 83 */
-        rc = sox_skipbytes(ft, 83-69);
+        rc = lsx_skipbytes(ft, (size_t) 83-69);
         if (rc)
             return rc;
 
         /* Get essential numbers from the header */
-        sox_readdw(ft, &datasize); /* bytes 83-86 */
-        sox_readdw(ft, &rsrcsize); /* bytes 87-90 */
+        lsx_readdw(ft, &datasize); /* bytes 83-86 */
+        lsx_readdw(ft, &rsrcsize); /* bytes 87-90 */
 
         /* Skip the rest of the header (total 128 bytes) */
-        rc = sox_skipbytes(ft, 128-91);
+        rc = lsx_skipbytes(ft, (size_t) 128-91);
         if (rc != 0)
             return rc;
 
         /* The data fork must contain a "HCOM" header */
-        if (sox_reads(ft, buf, 4) == SOX_EOF || strncmp(buf, "HCOM", 4) != 0)
+        if (lsx_reads(ft, buf, (size_t)4) == SOX_EOF || strncmp(buf, "HCOM", (size_t)4) != 0)
         {
-                sox_fail_errno(ft,SOX_EHDR,"Mac data fork is not HCOM");
+                lsx_fail_errno(ft,SOX_EHDR,"Mac data fork is not HCOM");
                 return (SOX_EOF);
         }
 
         /* Then follow various parameters */
-        sox_readdw(ft, &huffcount);
-        sox_readdw(ft, &checksum);
-        sox_readdw(ft, &compresstype);
+        lsx_readdw(ft, &huffcount);
+        lsx_readdw(ft, &checksum);
+        lsx_readdw(ft, &compresstype);
         if (compresstype > 1)
         {
-                sox_fail_errno(ft,SOX_EHDR,"Bad compression type in HCOM header");
+                lsx_fail_errno(ft,SOX_EHDR,"Bad compression type in HCOM header");
                 return (SOX_EOF);
         }
-        sox_readdw(ft, &divisor);
+        lsx_readdw(ft, &divisor);
         if (divisor == 0 || divisor > 4)
         {
-                sox_fail_errno(ft,SOX_EHDR,"Bad sampling rate divisor in HCOM header");
+                lsx_fail_errno(ft,SOX_EHDR,"Bad sampling rate divisor in HCOM header");
                 return (SOX_EOF);
         }
-        sox_readw(ft, &dictsize);
+        lsx_readw(ft, &dictsize);
 
         /* Translate to sox parameters */
-        ft->signal.encoding = SOX_ENCODING_UNSIGNED;
-        ft->signal.size = SOX_SIZE_BYTE;
+        ft->encoding.encoding = SOX_ENCODING_HCOM;
+        ft->encoding.bits_per_sample = 8;
         ft->signal.rate = 22050 / divisor;
         ft->signal.channels = 1;
 
         /* Allocate memory for the dictionary */
-        p->dictionary = (dictent *)xmalloc(511 * sizeof(dictent));
+        p->dictionary = lsx_malloc(511 * sizeof(dictent));
 
         /* Read dictionary */
         for(i = 0; i < dictsize; i++) {
-                sox_readw(ft, (unsigned short *)&(p->dictionary[i].dict_leftson));
-                sox_readw(ft, (unsigned short *)&(p->dictionary[i].dict_rightson));
-                sox_debug("%d %d",
+                lsx_readsw(ft, &(p->dictionary[i].dict_leftson));
+                lsx_readsw(ft, &(p->dictionary[i].dict_rightson));
+                lsx_debug("%d %d",
                        p->dictionary[i].dict_leftson,
                        p->dictionary[i].dict_rightson);
         }
-        rc = sox_skipbytes(ft, 1); /* skip pad byte */
+        rc = lsx_skipbytes(ft, (size_t) 1); /* skip pad byte */
         if (rc)
             return rc;
 
@@ -140,7 +159,7 @@ static int sox_hcomstartread(sox_format_t * ft)
         p->checksum = checksum;
         p->deltacompression = compresstype;
         if (!p->deltacompression)
-                sox_debug("HCOM data using value compression");
+                lsx_debug("HCOM data using value compression");
         p->huffcount = huffcount;
         p->cksum = 0;
         p->dictentry = 0;
@@ -149,9 +168,9 @@ static int sox_hcomstartread(sox_format_t * ft)
         return (SOX_SUCCESS);
 }
 
-static sox_size_t sox_hcomread(sox_format_t * ft, sox_sample_t *buf, sox_size_t len)
+static size_t read_samples(sox_format_t * ft, sox_sample_t *buf, size_t len)
 {
-        register struct readpriv *p = (struct readpriv *) ft->priv;
+        register priv_t *p = (priv_t *) ft->priv;
         int done = 0;
         unsigned char sample_rate;
 
@@ -159,9 +178,8 @@ static sox_size_t sox_hcomread(sox_format_t * ft, sox_sample_t *buf, sox_size_t
                 /* The first byte is special */
                 if (p->huffcount == 0)
                         return 0; /* Don't know if this can happen... */
-                if (sox_readb(ft, &sample_rate) == SOX_EOF)
+                if (lsx_readb(ft, &sample_rate) == SOX_EOF)
                 {
-                        sox_fail_errno(ft,SOX_EOF,"unexpected EOF at start of HCOM data");
                         return (0);
                 }
                 p->sample = sample_rate;
@@ -176,10 +194,10 @@ static sox_size_t sox_hcomread(sox_format_t * ft, sox_sample_t *buf, sox_size_t
 
         while (p->huffcount > 0) {
                 if(p->nrbits == 0) {
-                        sox_readdw(ft, &(p->current));
-                        if (sox_eof(ft))
+                        lsx_readdw(ft, &(p->current));
+                        if (lsx_eof(ft))
                         {
-                                sox_fail_errno(ft,SOX_EOF,"unexpected EOF in HCOM data");
+                                lsx_fail_errno(ft,SOX_EOF,"unexpected EOF in HCOM data");
                                 return (0);
                         }
                         p->cksum += p->current;
@@ -213,69 +231,49 @@ static sox_size_t sox_hcomread(sox_format_t * ft, sox_sample_t *buf, sox_size_t
         return done;
 }
 
-static int sox_hcomstopread(sox_format_t * ft)
+static int stopread(sox_format_t * ft)
 {
-        register struct readpriv *p = (struct readpriv *) ft->priv;
+        register priv_t *p = (priv_t *) ft->priv;
 
         if (p->huffcount != 0)
         {
-                sox_fail_errno(ft,SOX_EFMT,"not all HCOM data read");
+                lsx_fail_errno(ft,SOX_EFMT,"not all HCOM data read");
                 return (SOX_EOF);
         }
         if(p->cksum != p->checksum)
         {
-                sox_fail_errno(ft,SOX_EFMT,"checksum error in HCOM data");
+                lsx_fail_errno(ft,SOX_EFMT,"checksum error in HCOM data");
                 return (SOX_EOF);
         }
-        free((char *)p->dictionary);
+        free(p->dictionary);
         p->dictionary = NULL;
         return (SOX_SUCCESS);
 }
 
-struct writepriv {
-  unsigned char *data;          /* Buffer allocated with xmalloc */
-  sox_size_t size;               /* Size of allocated buffer */
-  sox_size_t pos;                /* Where next byte goes */
-};
-
 #define BUFINCR (10*BUFSIZ)
 
-static int sox_hcomstartwrite(sox_format_t * ft)
+static int startwrite(sox_format_t * ft)
 {
-        register struct writepriv *p = (struct writepriv *) ft->priv;
+  priv_t * p = (priv_t *) ft->priv;
 
-        switch ((int)ft->signal.rate) {
-        case 22050:
-        case 22050/2:
-        case 22050/3:
-        case 22050/4:
-                break;
-        default:
-                sox_fail_errno(ft,SOX_EFMT,"unacceptable output rate for HCOM: try 5512, 7350, 11025 or 22050 hertz");
-                return (SOX_EOF);
-        }
-        ft->signal.size = SOX_SIZE_BYTE;
-        ft->signal.encoding = SOX_ENCODING_UNSIGNED;
-        ft->signal.channels = 1;
-
-        p->size = BUFINCR;
-        p->pos = 0;
-        p->data = (unsigned char *) xmalloc(p->size);
-        return (SOX_SUCCESS);
+  p->size = BUFINCR;
+  p->pos = 0;
+  p->data = lsx_malloc(p->size);
+  return SOX_SUCCESS;
 }
 
-static sox_size_t sox_hcomwrite(sox_format_t * ft, const sox_sample_t *buf, sox_size_t len)
+static size_t write_samples(sox_format_t * ft, const sox_sample_t *buf, size_t len)
 {
-  struct writepriv *p = (struct writepriv *) ft->priv;
+  priv_t *p = (priv_t *) ft->priv;
   sox_sample_t datum;
-  sox_size_t i;
+  size_t i;
 
   if (len == 0)
     return 0;
 
   if (p->pos + len > p->size) {
     p->size = ((p->pos + len) / BUFINCR + 1) * BUFINCR;
-    p->data = (unsigned char *)xrealloc(p->data, p->size);
+    p->data = lsx_realloc(p->data, p->size);
   }
 
   for (i = 0; i < len; i++) {
@@ -300,7 +298,7 @@ static void makecodes(int e, int c, int s, int b, dictent newdict[511], long cod
 
 static void putcode(sox_format_t * ft, long codes[256], long codesize[256], unsigned c, unsigned char **df)
 {
-  struct readpriv *p = (struct readpriv *) ft->priv;
+  priv_t *p = (priv_t *) ft->priv;
   long code, size;
   int i;
 
@@ -321,10 +319,10 @@ static void putcode(sox_format_t * ft, long codes[256], long codesize[256], unsi
   }
 }
 
-static void compress(sox_format_t * ft, unsigned char **df, int32_t *dl, sox_rate_t fr)
+static void compress(sox_format_t * ft, unsigned char **df, int32_t *dl)
 {
-  struct readpriv *p = (struct readpriv *) ft->priv;
-  int32_t samplerate;
+  priv_t *p = (priv_t *) ft->priv;
+  int samplerate;
   unsigned char *datafork = *df;
   unsigned char *ddf, *dfp;
   short dictsize;
@@ -338,7 +336,7 @@ static void compress(sox_format_t * ft, unsigned char **df, int32_t *dl, sox_rat
   memset(codes, 0, sizeof(codes));
   memset(codesize, 0, sizeof(codesize));
   memset(newdict, 0, sizeof(newdict));
-  
+
   for (i = 1; i < *dl; i++) {
     d = (datafork[i] - (sample & 0xff)) & 0xff; /* creates absolute entries LMS */
     sample = datafork[i];
@@ -391,9 +389,9 @@ static void compress(sox_format_t * ft, unsigned char **df, int32_t *dl, sox_rat
   for (i = 0; i < 256; i++)
     l += frequtable[i] * codesize[i];
   l = (((l + 31) >> 5) << 2) + 24 + dictsize * 4;
-  sox_debug("  Original size: %6d bytes", *dl);
-  sox_debug("Compressed size: %6d bytes", l);
-  datafork = (unsigned char *)xmalloc((unsigned)l);
+  lsx_debug("  Original size: %6d bytes", *dl);
+  lsx_debug("Compressed size: %6d bytes", l);
+  datafork = lsx_malloc((size_t)l);
   ddf = datafork + 22;
   for(i = 0; i < dictsize; i++) {
     put16_be(&ddf, newdict[i].dict_leftson);
@@ -411,12 +409,12 @@ static void compress(sox_format_t * ft, unsigned char **df, int32_t *dl, sox_rat
     codesize[0] = 32 - p->nbits;
     putcode(ft, codes, codesize, 0, &ddf);
   }
-  strncpy((char *)datafork, "HCOM", 4);
+  memcpy(datafork, "HCOM", (size_t)4);
   dfp = datafork + 4;
   put32_be(&dfp, *dl);
   put32_be(&dfp, p->new_checksum);
   put32_be(&dfp, 1);
-  samplerate = 22050 / fr;
+  samplerate = 22050 / ft->signal.rate + .5;
   put32_be(&dfp, samplerate);
   put16_be(&dfp, dictsize);
   *df = datafork;               /* reassign passed pointer to new datafork */
@@ -425,64 +423,55 @@ static void compress(sox_format_t * ft, unsigned char **df, int32_t *dl, sox_rat
 
 /* End of hcom utility routines */
 
-static int sox_hcomstopwrite(sox_format_t * ft)
+static int stopwrite(sox_format_t * ft)
 {
-  struct writepriv *p = (struct writepriv *) ft->priv;
+  priv_t *p = (priv_t *) ft->priv;
   unsigned char *compressed_data = p->data;
-  sox_size_t compressed_len = p->pos;
+  size_t compressed_len = p->pos;
   int rc = SOX_SUCCESS;
 
   /* Compress it all at once */
   if (compressed_len)
-    compress(ft, &compressed_data, (int32_t *)&compressed_len, ft->signal.rate);
-  free((char *)p->data);
+    compress(ft, &compressed_data, (int32_t *)&compressed_len);
+  free(p->data);
 
   /* Write the header */
-  sox_writebuf(ft, (void *)"\000\001A", 3); /* Dummy file name "A" */
-  sox_padbytes(ft, 65-3);
-  sox_writes(ft, "FSSD");
-  sox_padbytes(ft, 83-69);
-  sox_writedw(ft, compressed_len); /* compressed_data size */
-  sox_writedw(ft, 0); /* rsrc size */
-  sox_padbytes(ft, 128 - 91);
-  if (sox_error(ft)) {
-    sox_fail_errno(ft, errno, "write error in HCOM header");
+  lsx_writebuf(ft, "\000\001A", (size_t) 3); /* Dummy file name "A" */
+  lsx_padbytes(ft, (size_t) 65-3);
+  lsx_writes(ft, "FSSD");
+  lsx_padbytes(ft, (size_t) 83-69);
+  lsx_writedw(ft, (unsigned) compressed_len); /* compressed_data size */
+  lsx_writedw(ft, 0); /* rsrc size */
+  lsx_padbytes(ft, (size_t) 128 - 91);
+  if (lsx_error(ft)) {
+    lsx_fail_errno(ft, errno, "write error in HCOM header");
     rc = SOX_EOF;
-  } else if (sox_writebuf(ft, compressed_data, compressed_len) != compressed_len) {
+  } else if (lsx_writebuf(ft, compressed_data, compressed_len) != compressed_len) {
     /* Write the compressed_data fork */
-    sox_fail_errno(ft, errno, "can't write compressed HCOM data");
+    lsx_fail_errno(ft, errno, "can't write compressed HCOM data");
     rc = SOX_EOF;
   }
-  free((char *)compressed_data);
+  free(compressed_data);
 
   if (rc == SOX_SUCCESS)
     /* Pad the compressed_data fork to a multiple of 128 bytes */
-    sox_padbytes(ft, 128u - (compressed_len % 128));
+    lsx_padbytes(ft, 128u - (compressed_len % 128));
 
   return rc;
 }
 
-/* Mac FSSD/HCOM */
-static const char *hcomnames[] = {
-  "hcom",
-  NULL
-};
-
-static sox_format_handler_t sox_hcom_format = {
-  hcomnames,
-  SOX_FILE_BIG_END,
-  sox_hcomstartread,
-  sox_hcomread,
-  sox_hcomstopread,
-  sox_hcomstartwrite,
-  sox_hcomwrite,
-  sox_hcomstopwrite,
-  sox_format_nothing_seek
-};
-
-const sox_format_handler_t *sox_hcom_format_fn(void);
-
-const sox_format_handler_t *sox_hcom_format_fn(void)
+SOX_FORMAT_HANDLER(hcom)
 {
-    return &sox_hcom_format;
+  static char const * const names[]       = {"hcom", NULL};
+  static sox_rate_t   const write_rates[] = {22050,22050/2,22050/3,22050/4.,0};
+  static unsigned     const write_encodings[] = {
+    SOX_ENCODING_HCOM, 8, 0, 0};
+  static sox_format_handler_t handler = {SOX_LIB_VERSION_CODE,
+    "Mac FSSD files with Huffman compression",
+    names, SOX_FILE_BIG_END|SOX_FILE_MONO,
+    startread, read_samples, stopread,
+    startwrite, write_samples, stopwrite,
+    NULL, write_encodings, write_rates, sizeof(priv_t)
+  };
+  return &handler;
 }
