@@ -1,21 +1,20 @@
-/*
- * LADSPA effect support for sox
+/* LADSPA effect support for sox
  * (c) Reuben Thomas <rrt@sc3d.org> 2007
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or (at
+ * your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, write to the Free Software
- * Foundation, Fifth Floor, 51 Franklin Street, Boston, MA 02111-1301,
- * USA.  */
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 #include "sox_i.h"
 
@@ -24,7 +23,6 @@
 #include <assert.h>
 #include <limits.h>
 #include <string.h>
-#include <math.h>
 #include <ltdl.h>
 #include <ladspa.h>
 
@@ -38,12 +36,12 @@ typedef struct {
   LADSPA_Handle handle;         /* instantiated plugin handle */
   LADSPA_Data *control;         /* control ports */
   unsigned long input_port, output_port;
-} *ladspa_t;
+} priv_t;
 
 static LADSPA_Data ladspa_default(const LADSPA_PortRangeHint *p)
 {
   LADSPA_Data d;
-  
+
   if (LADSPA_IS_HINT_DEFAULT_0(p->HintDescriptor))
     d = 0.0;
   else if (LADSPA_IS_HINT_DEFAULT_1(p->HintDescriptor))
@@ -73,7 +71,7 @@ static LADSPA_Data ladspa_default(const LADSPA_PortRangeHint *p)
       d = p->LowerBound * 0.25 + p->UpperBound * 0.75;
   } else { /* shouldn't happen */
     /* FIXME: Deal with this at a higher level */
-    sox_fail("non-existent default value; using 0.1");
+    lsx_fail("non-existent default value; using 0.1");
     d = 0.1; /* Should at least avoid divide by 0 */
   }
 
@@ -85,9 +83,9 @@ static LADSPA_Data ladspa_default(const LADSPA_PortRangeHint *p)
  */
 static int sox_ladspa_getopts(sox_effect_t *effp, int n, char **argv)
 {
-  ladspa_t l_st = (ladspa_t)effp->priv;
+  priv_t * l_st = (priv_t *)effp->priv;
   char *path;
-  LADSPA_Descriptor_Function l_fn;
+  union {LADSPA_Descriptor_Function fn; lt_ptr ptr;} ltptr;
   unsigned long index = 0, i;
   double arg;
 
@@ -106,63 +104,63 @@ static int sox_ladspa_getopts(sox_effect_t *effp, int n, char **argv)
     path = LADSPA_PATH;
   lt_dlsetsearchpath(path);
   if ((l_st->lth = lt_dlopenext(l_st->name)) == NULL) {
-    sox_fail("could not open LADSPA plugin %s", l_st->name);
+    lsx_fail("could not open LADSPA plugin %s", l_st->name);
     return SOX_EOF;
   }
 
   /* Get descriptor function */
-  if ((l_fn = (LADSPA_Descriptor_Function)lt_dlsym(l_st->lth, "ladspa_descriptor")) == NULL) {
-    sox_fail("could not find ladspa_descriptor");
+  if ((ltptr.ptr = lt_dlsym(l_st->lth, "ladspa_descriptor")) == NULL) {
+    lsx_fail("could not find ladspa_descriptor");
     return SOX_EOF;
   }
 
   /* If no plugins in this module, complain */
-  if (l_fn(0) == NULL) {
-    sox_fail("no plugins found");
+  if (ltptr.fn(0UL) == NULL) {
+    lsx_fail("no plugins found");
     return SOX_EOF;
   }
 
   /* Get first plugin descriptor */
-  l_st->desc = l_fn(0);
+  l_st->desc = ltptr.fn(0UL);
   assert(l_st->desc);           /* We already know this will work */
 
   /* If more than one plugin, or first argument is not a number, try
      to use first argument as plugin label. */
-  if (n > 0 && (l_fn(1) != NULL || !sscanf(argv[0], "%lf", &arg))) {
+  if (n > 0 && (ltptr.fn(1UL) != NULL || !sscanf(argv[0], "%lf", &arg))) {
     while (l_st->desc && strcmp(l_st->desc->Label, argv[0]) != 0)
-      l_st->desc = l_fn(++index);
+      l_st->desc = ltptr.fn(++index);
     if (l_st->desc == NULL) {
-      sox_fail("no plugin called `%s' found", argv[0]);
+      lsx_fail("no plugin called `%s' found", argv[0]);
       return SOX_EOF;
     } else
       n--; argv++;
   }
 
   /* Scan the ports to check there's one input and one output */
-  l_st->control = xcalloc(l_st->desc->PortCount, sizeof(LADSPA_Data));
+  l_st->control = lsx_calloc(l_st->desc->PortCount, sizeof(LADSPA_Data));
   for (i = 0; i < l_st->desc->PortCount; i++) {
     const LADSPA_PortDescriptor port = l_st->desc->PortDescriptors[i];
 
     /* Check port is well specified. All control ports should be
        inputs, but don't bother checking, as we never rely on this. */
     if (LADSPA_IS_PORT_INPUT(port) && LADSPA_IS_PORT_OUTPUT(port)) {
-      sox_fail("port %lu is both input and output", i);
+      lsx_fail("port %lu is both input and output", i);
       return SOX_EOF;
     } else if (LADSPA_IS_PORT_CONTROL(port) && LADSPA_IS_PORT_AUDIO(port)) {
-      sox_fail("port %lu is both audio and control", i);
+      lsx_fail("port %lu is both audio and control", i);
       return SOX_EOF;
     }
-               
+
     if (LADSPA_IS_PORT_AUDIO(port)) {
       if (LADSPA_IS_PORT_INPUT(port)) {
         if (l_st->input_port != ULONG_MAX) {
-          sox_fail("can't use a plugin with more than one audio input port");
+          lsx_fail("can't use a plugin with more than one audio input port");
           return SOX_EOF;
         }
         l_st->input_port = i;
       } else if (LADSPA_IS_PORT_OUTPUT(port)) {
         if (l_st->output_port != ULONG_MAX) {
-          sox_fail("can't use a plugin with more than one audio output port");
+          lsx_fail("can't use a plugin with more than one audio output port");
           return SOX_EOF;
         }
         l_st->output_port = i;
@@ -170,23 +168,23 @@ static int sox_ladspa_getopts(sox_effect_t *effp, int n, char **argv)
     } else {                    /* Control port */
       if (n == 0) {
         if (!LADSPA_IS_HINT_HAS_DEFAULT(l_st->desc->PortRangeHints[i].HintDescriptor)) {
-          sox_fail("not enough arguments for control ports");
+          lsx_fail("not enough arguments for control ports");
           return SOX_EOF;
         }
         l_st->control[i] = ladspa_default(&(l_st->desc->PortRangeHints[i]));
-        sox_debug("default argument for port %lu is %f", i, l_st->control[i]);
+        lsx_debug("default argument for port %lu is %f", i, l_st->control[i]);
       } else {
         if (!sscanf(argv[0], "%lf", &arg))
-          return sox_usage(effp);
+          return lsx_usage(effp);
         l_st->control[i] = (LADSPA_Data)arg;
-        sox_debug("argument for port %lu is %f", i, l_st->control[i]);
+        lsx_debug("argument for port %lu is %f", i, l_st->control[i]);
         n--; argv++;
       }
     }
   }
 
   /* Stop if we have any unused arguments */
-  return n? sox_usage(effp) : SOX_SUCCESS;
+  return n? lsx_usage(effp) : SOX_SUCCESS;
 }
 
 /*
@@ -194,14 +192,14 @@ static int sox_ladspa_getopts(sox_effect_t *effp, int n, char **argv)
  */
 static int sox_ladspa_start(sox_effect_t * effp)
 {
-  ladspa_t l_st = (ladspa_t)effp->priv;
+  priv_t * l_st = (priv_t *)effp->priv;
   unsigned long i;
 
   /* Instantiate the plugin */
-  sox_debug("rate for plugin is %g", effp->ininfo.rate);
-  l_st->handle = l_st->desc->instantiate(l_st->desc, (unsigned long)effp->ininfo.rate);
+  lsx_debug("rate for plugin is %g", effp->in_signal.rate);
+  l_st->handle = l_st->desc->instantiate(l_st->desc, (unsigned long)effp->in_signal.rate);
   if (l_st->handle == NULL) {
-    sox_fail("could not instantiate plugin");
+    lsx_fail("could not instantiate plugin");
     return SOX_EOF;
   }
 
@@ -223,40 +221,40 @@ static int sox_ladspa_start(sox_effect_t * effp)
  * Process one bufferful of data.
  */
 static int sox_ladspa_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf,
-                           sox_size_t *isamp, sox_size_t *osamp)
+                           size_t *isamp, size_t *osamp)
 {
-  ladspa_t l_st = (ladspa_t)effp->priv;
-  sox_size_t i, len = min(*isamp, *osamp);
+  priv_t * l_st = (priv_t *)effp->priv;
+  size_t i, len = min(*isamp, *osamp);
 
   *osamp = *isamp = len;
 
   if (len) {
-    LADSPA_Data *buf = xmalloc(sizeof(LADSPA_Data) * len);
-    
+    LADSPA_Data *buf = lsx_malloc(sizeof(LADSPA_Data) * len);
+
     /* Insert input if effect takes it */
     if (l_st->input_port != ULONG_MAX) {
       /* Copy the input; FIXME: Assume LADSPA_Data == float! */
       for (i = 0; i < len; i++)
         buf[i] = SOX_SAMPLE_TO_FLOAT_32BIT(ibuf[i], effp->clips);
-      
+
       /* Connect the input port */
       l_st->desc->connect_port(l_st->handle, l_st->input_port, buf);
     }
-    
+
     /* Connect the output port if used */
     if (l_st->output_port != ULONG_MAX)
       l_st->desc->connect_port(l_st->handle, l_st->output_port, buf);
-    
+
     /* Run the plugin */
     l_st->desc->run(l_st->handle, len);
-    
+
     /* Grab output if effect produces it */
     if (l_st->output_port != ULONG_MAX)
       /* FIXME: Assume LADSPA_Data == float! */
       for (i = 0; i < len; i++) {
         obuf[i] = SOX_FLOAT_32BIT_TO_SAMPLE(buf[i], effp->clips);
       }
-    
+
     free(buf);
   }
 
@@ -266,7 +264,7 @@ static int sox_ladspa_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sa
 /*
  * Nothing to do.
  */
-static int sox_ladspa_drain(sox_effect_t * effp UNUSED, sox_sample_t *obuf UNUSED, sox_size_t *osamp)
+static int sox_ladspa_drain(sox_effect_t * effp UNUSED, sox_sample_t *obuf UNUSED, size_t *osamp)
 {
   *osamp = 0;
 
@@ -279,7 +277,7 @@ static int sox_ladspa_drain(sox_effect_t * effp UNUSED, sox_sample_t *obuf UNUSE
  */
 static int sox_ladspa_stop(sox_effect_t * effp)
 {
-  ladspa_t l_st = (ladspa_t)effp->priv;
+  priv_t * l_st = (priv_t *)effp->priv;
 
   /* If needed, deactivate the plugin */
   if (l_st->desc->deactivate)
@@ -297,7 +295,7 @@ static sox_effect_handler_t sox_ladspa_effect = {
   sox_ladspa_flow,
   sox_ladspa_drain,
   sox_ladspa_stop,
-  NULL
+  NULL, sizeof(priv_t)
 };
 
 const sox_effect_handler_t *sox_ladspa_effect_fn(void)
