@@ -1,9 +1,8 @@
-/*
- * July 5, 1991
+/* July 5, 1991
  * Copyright 1991 Lance Norskog And Sundry Contributors
  * This source code is freely redistributable and may be used for
- * any purpose.  This copyright notice must be maintained. 
- * Lance Norskog And Sundry Contributors are not responsible for 
+ * any purpose.  This copyright notice must be maintained.
+ * Lance Norskog And Sundry Contributors are not responsible for
  * the consequences of using this software.
  *
  * Channel duplication code by Graeme W. Gill - 93/5/18
@@ -20,7 +19,7 @@
 #include <string.h>
 #include <stdlib.h>
 
-typedef struct mixerstuff {
+typedef struct {
         /* How to generate each output channel.  sources[i][j] */
         /* represents the fraction of channel i that should be passed */
         /* through to channel j on output, and so forth.  Channel 0 is */
@@ -29,7 +28,7 @@ typedef struct mixerstuff {
         double  sources[4][4];
         int     num_pans;
         int     mix;                    /* How are we mixing it? */
-} *mixer_t;
+} priv_t;
 
 /* MIX_CENTER is shorthand to mix channels together at 50% each */
 #define MIX_CENTER      0
@@ -39,9 +38,9 @@ typedef struct mixerstuff {
 /*
  * Process options
  */
-static int getopts(sox_effect_t * effp, int n, char **argv) 
+static int getopts(sox_effect_t * effp, int n, char **argv)
 {
-    mixer_t mixer = (mixer_t) effp->priv;
+    priv_t * mixer = (priv_t *) effp->priv;
     double* pans = &mixer->sources[0][0];
     int i;
 
@@ -64,7 +63,7 @@ static int getopts(sox_effect_t * effp, int n, char **argv)
         else if (!strcmp(argv[0], "-4")) mixer->mix = '4';
         else if (argv[0][0] == '-' && !isdigit((int)argv[0][1])
                 && argv[0][1] != '.')
-          return sox_usage(effp);
+          return lsx_usage(effp);
         else {
             int commas;
             char *s;
@@ -74,7 +73,7 @@ static int getopts(sox_effect_t * effp, int n, char **argv)
                 if (*s == ',') {
                     ++commas;
                     if (commas >= 16) {
-                        sox_fail("mixer can only take up to 16 pan values");
+                        lsx_fail("mixer can only take up to 16 pan values");
                         return (SOX_EOF);
                     }
                     pans[commas] = atof(s+1);
@@ -87,7 +86,7 @@ static int getopts(sox_effect_t * effp, int n, char **argv)
         mixer->mix = MIX_CENTER;
     }
     else
-      return sox_usage(effp);
+      return lsx_usage(effp);
 
     return (SOX_SUCCESS);
 }
@@ -159,7 +158,7 @@ static int start(sox_effect_t * effp)
 
        GHK 2000/11/28
      */
-     mixer_t mixer = (mixer_t) effp->priv;
+     priv_t * mixer = (priv_t *) effp->priv;
      double pans[16];
      int i, j;
      int ichan, ochan;
@@ -167,17 +166,17 @@ static int start(sox_effect_t * effp)
      for (i = 0;  i < 16;  i++)
          pans[i] = ((double*)&mixer->sources[0][0])[i];
 
-     ichan = effp->ininfo.channels;
-     ochan = effp->outinfo.channels;
+     ichan = effp->in_signal.channels;
+     ochan = effp->out_signal.channels;
      if (ochan == -1) {
-         sox_fail("Output must have known number of channels");
+         lsx_fail("Output must have known number of channels");
          return(SOX_EOF);
      }
 
      if ((ichan != 1 && ichan != 2 && ichan != 4 &&
           mixer->mix != MIX_CENTER && ochan != 1)
              ||  (ochan != 1 && ochan != 2 && ochan != 4)) {
-         sox_fail("Can't mix %d -> %d channels", ichan, ochan);
+         lsx_fail("Can't mix %d -> %d channels", ichan, ochan);
          return (SOX_EOF);
      }
 
@@ -204,7 +203,7 @@ static int start(sox_effect_t * effp)
              }
              else
              {
-                 sox_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
+                 lsx_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
                  return SOX_EOF;
              }
              break;
@@ -225,7 +224,7 @@ static int start(sox_effect_t * effp)
              }
              else
              {
-                 sox_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
+                 lsx_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
                  return SOX_EOF;
              }
              break;
@@ -246,7 +245,7 @@ static int start(sox_effect_t * effp)
              }
              else
              {
-                 sox_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
+                 lsx_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
                  return SOX_EOF;
              }
              break;
@@ -267,7 +266,7 @@ static int start(sox_effect_t * effp)
              }
              else
              {
-                 sox_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
+                 lsx_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
                  return SOX_EOF;
              }
              break;
@@ -288,7 +287,7 @@ static int start(sox_effect_t * effp)
              }
              else
              {
-                 sox_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
+                 lsx_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
                  return SOX_EOF;
              }
              break;
@@ -309,7 +308,7 @@ static int start(sox_effect_t * effp)
              }
              else
              {
-                 sox_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
+                 lsx_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
                  return SOX_EOF;
              }
              break;
@@ -325,7 +324,7 @@ static int start(sox_effect_t * effp)
              }
              else
              {
-                 sox_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
+                 lsx_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
                  return SOX_EOF;
              }
              break;
@@ -341,7 +340,7 @@ static int start(sox_effect_t * effp)
              }
              else
              {
-                 sox_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
+                 lsx_fail("Can't mix -%c %d -> %d channels", mixer->mix, ichan, ochan);
                  return SOX_EOF;
              }
              break;
@@ -349,7 +348,7 @@ static int start(sox_effect_t * effp)
          case MIX_SPECIFIED:
              break;
          default:
-             sox_fail("Unknown mix option");
+             lsx_fail("Unknown mix option");
              return SOX_EOF;
      }
 
@@ -362,7 +361,7 @@ static int start(sox_effect_t * effp)
      {
          for (i = 0; i < ichan; i++)
          {
-             for (j = 0; j < ochan; j++) 
+             for (j = 0; j < ochan; j++)
              {
                  mixer->sources[i][j] = 0;
              }
@@ -399,7 +398,7 @@ static int start(sox_effect_t * effp)
              mixer->sources[3][1] = 0.5;
          }
          else {
-             sox_fail("You must specify at least one mix level when using mixer with an unusual number of channels.");
+             lsx_fail("You must specify at least one mix level when using mixer with an unusual number of channels.");
              return(SOX_EOF);
          }
      }
@@ -427,7 +426,7 @@ static int start(sox_effect_t * effp)
          }
          else
          {
-             sox_fail("Invalid options while not mixing");
+             lsx_fail("Invalid options while not mixing");
              return SOX_EOF;
          }
      }
@@ -454,12 +453,12 @@ static int start(sox_effect_t * effp)
          }
          else
          {
-             sox_fail("Invalid options for this channel combination");
+             lsx_fail("Invalid options for this channel combination");
              return SOX_EOF;
          }
      }
      else if (mixer->num_pans == 3) {
-       sox_fail("Invalid options while not mixing");
+       lsx_fail("Invalid options while not mixing");
        return SOX_EOF;
      }
      else if (mixer->num_pans == 4) {
@@ -480,16 +479,16 @@ static int start(sox_effect_t * effp)
          }
          else
          {
-             sox_fail("Invalid options for this channel combination");
+             lsx_fail("Invalid options for this channel combination");
              return SOX_EOF;
          }
      }
 
-     if (effp->ininfo.channels != effp->outinfo.channels)
+     if (effp->in_signal.channels != effp->out_signal.channels)
        return SOX_SUCCESS;
 
-     for (i = 0; i < (int)effp->ininfo.channels; ++i)
-       for (j = 0; j < (int)effp->outinfo.channels; ++j)
+     for (i = 0; i < (int)effp->in_signal.channels; ++i)
+       for (j = 0; j < (int)effp->out_signal.channels; ++j)
          if (mixer->sources[i][j] != (i == j))
            return SOX_SUCCESS;
 
@@ -500,17 +499,17 @@ static int start(sox_effect_t * effp)
  * Process either isamp or osamp samples, whichever is smaller.
  */
 
-static int flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf, 
-                sox_size_t *isamp, sox_size_t *osamp)
+static int flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf,
+                size_t *isamp, size_t *osamp)
 {
-    mixer_t mixer = (mixer_t) effp->priv;
-    sox_size_t len, done;
+    priv_t * mixer = (priv_t *) effp->priv;
+    size_t len, done;
     int ichan, ochan;
     int i, j;
     double samp;
 
-    ichan = effp->ininfo.channels;
-    ochan = effp->outinfo.channels;
+    ichan = effp->in_signal.channels;
+    ochan = effp->out_signal.channels;
     len = *isamp / ichan;
     if (len > *osamp / ochan)
         len = *osamp / ochan;
@@ -534,32 +533,15 @@ sox_effect_handler_t const * sox_mixer_effect_fn(void)
     "mixer",
     "[ -l | -r | -f | -b | -1 | -2 | -3 | -4 | n,n,n...,n ]",
     SOX_EFF_MCHAN | SOX_EFF_CHAN,
-    getopts, start, flow, 0, 0, 0
+    getopts, start, flow, 0, 0, 0, sizeof(priv_t)
   };
   return &handler;
 }
 
-sox_effect_handler_t const * sox_avg_effect_fn(void)
-{
-  static sox_effect_handler_t handler;
-  handler = *sox_mixer_effect_fn();
-  handler.name = "avg";
-  handler.flags |= SOX_EFF_DEPRECATED;
-  return &handler;
-}
-
-sox_effect_handler_t const * sox_pick_effect_fn(void)
-{
-  static sox_effect_handler_t handler;
-  handler = *sox_avg_effect_fn();
-  handler.name = "pick";
-  return &handler;
-}
-
-static int oops_getopts(sox_effect_t * effp, int argc, char * * argv UNUSED) 
+static int oops_getopts(sox_effect_t * effp, int argc, char * * argv UNUSED)
 {
   char * args[] = {"1,1,-1,-1"};
-  return argc? sox_usage(effp) : sox_mixer_effect_fn()->getopts(effp, array_length(args), args);
+  return argc? lsx_usage(effp) : sox_mixer_effect_fn()->getopts(effp, (int)array_length(args), args);
 }
 
 sox_effect_handler_t const * sox_oops_effect_fn(void)
