@@ -1,11 +1,10 @@
-/*
- * noiseprof - Noise Profiling Effect. 
+/* noiseprof - Noise Profiling Effect.
  *
  * Written by Ian Turner (vectro@vectro.org)
  *
  * Copyright 1999 Ian Turner
  * This source code is freely redistributable and may be used for
- * any purpose.  This copyright notice must be maintained. 
+ * any purpose.  This copyright notice must be maintained.
  * Authors are not responsible for the consequences of using this software.
  */
 
@@ -16,7 +15,7 @@
 #include <string.h>
 #include <assert.h>
 
-typedef struct chandata {
+typedef struct {
     float *window;
     float *lastwindow;
     float *noisegate;
@@ -24,13 +23,35 @@ typedef struct chandata {
 } chandata_t;
 
 /* Holds profile information */
-typedef struct reddata {
+typedef struct {
     char* profile_filename;
     float threshold;
 
     chandata_t *chandata;
-    sox_size_t bufdata;
-} * reddata_t;
+    size_t bufdata;
+} priv_t;
+
+static void FFT(unsigned NumSamples,
+         int InverseTransform,
+         const float *RealIn, float *ImagIn, float *RealOut, float *ImagOut)
+{
+  unsigned i;
+  double * work = malloc(2 * NumSamples * sizeof(*work));
+  for (i = 0; i < 2 * NumSamples; i += 2) {
+    work[i] = RealIn[i >> 1];
+    work[i + 1] = ImagIn? ImagIn[i >> 1] : 0;
+  }
+  lsx_safe_cdft(2 * (int)NumSamples, InverseTransform? -1 : 1, work);
+  if (InverseTransform) for (i = 0; i < 2 * NumSamples; i += 2) {
+    RealOut[i >> 1] = work[i] / NumSamples;
+    ImagOut[i >> 1] = work[i + 1] / NumSamples;
+  }
+  else for (i = 0; i < 2 * NumSamples; i += 2) {
+    RealOut[i >> 1] = work[i];
+    ImagOut[i >> 1] = work[i + 1];
+  }
+  free(work);
+}
 
 /*
  * Get the options. Default file is stdin (if the audio
@@ -38,7 +59,7 @@ typedef struct reddata {
  */
 static int sox_noisered_getopts(sox_effect_t * effp, int argc, char **argv)
 {
-  reddata_t p = (reddata_t) effp->priv;
+  priv_t * p = (priv_t *) effp->priv;
 
   if (argc > 0) {
     p->profile_filename = argv[0];
@@ -51,7 +72,7 @@ static int sox_noisered_getopts(sox_effect_t * effp, int argc, char **argv)
     NUMERIC_PARAMETER(threshold, 0, 1);
   } while (0);
 
-  return argc? sox_usage(effp) : SOX_SUCCESS;
+  return argc? lsx_usage(effp) : SOX_SUCCESS;
 }
 
 /*
@@ -60,51 +81,53 @@ static int sox_noisered_getopts(sox_effect_t * effp, int argc, char **argv)
  */
 static int sox_noisered_start(sox_effect_t * effp)
 {
-    reddata_t data = (reddata_t) effp->priv;
-    sox_size_t fchannels = 0;
-    sox_size_t channels = effp->ininfo.channels;
-    sox_size_t i;
+    priv_t * data = (priv_t *) effp->priv;
+    size_t fchannels = 0;
+    size_t channels = effp->in_signal.channels;
+    size_t i;
     FILE* ifp;
 
-    data->chandata = (chandata_t*)xcalloc(channels, sizeof(*(data->chandata)));
+    data->chandata = lsx_calloc(channels, sizeof(*(data->chandata)));
     data->bufdata = 0;
     for (i = 0; i < channels; i ++) {
-        data->chandata[i].noisegate = (float*)xcalloc(FREQCOUNT, sizeof(float));
-        data->chandata[i].smoothing = (float*)xcalloc(FREQCOUNT, sizeof(float));
+        data->chandata[i].noisegate = lsx_calloc(FREQCOUNT, sizeof(float));
+        data->chandata[i].smoothing = lsx_calloc(FREQCOUNT, sizeof(float));
         data->chandata[i].lastwindow = NULL;
     }
 
     /* Here we actually open the input file. */
     if (!data->profile_filename || !strcmp(data->profile_filename, "-")) {
       if (effp->global_info->global_info->stdin_in_use_by) {
-        sox_fail("stdin already in use by '%s'", effp->global_info->global_info->stdin_in_use_by);
+        lsx_fail("stdin already in use by '%s'", effp->global_info->global_info->stdin_in_use_by);
         return SOX_EOF;
       }
       effp->global_info->global_info->stdin_in_use_by = effp->handler.name;
       ifp = stdin;
     }
     else if ((ifp = fopen(data->profile_filename, "r")) == NULL) {
-        sox_fail("Couldn't open profile file %s: %s",
+        lsx_fail("Couldn't open profile file %s: %s",
                 data->profile_filename, strerror(errno));
         return SOX_EOF;
     }
 
     while (1) {
-        sox_size_t i1;
+        unsigned long i1_ul;
+        size_t i1;
         float f1;
-        if (2 != fscanf(ifp, " Channel %u: %f", &i1, &f1))
+        if (2 != fscanf(ifp, " Channel %lu: %f", &i1_ul, &f1))
             break;
+        i1 = i1_ul;
         if (i1 != fchannels) {
-            sox_fail("noisered: Got channel %d, expected channel %d.",
-                    i1, fchannels);
+            lsx_fail("noisered: Got channel %lu, expected channel %lu.",
+                    (unsigned long)i1, (unsigned long)fchannels);
             return SOX_EOF;
         }
 
         data->chandata[fchannels].noisegate[0] = f1;
         for (i = 1; i < FREQCOUNT; i ++) {
             if (1 != fscanf(ifp, ", %f", &f1)) {
-                sox_fail("noisered: Not enough datums for channel %d "
-                        "(expected %d, got %d)", fchannels, FREQCOUNT, i);
+                lsx_fail("noisered: Not enough datums for channel %lu "
+                        "(expected %d, got %lu)", (unsigned long)fchannels, FREQCOUNT, (unsigned long)i);
                 return SOX_EOF;
             }
             data->chandata[fchannels].noisegate[i] = f1;
@@ -112,8 +135,8 @@ static int sox_noisered_start(sox_effect_t * effp)
         fchannels ++;
     }
     if (fchannels != channels) {
-        sox_fail("noisered: channel mismatch: %d in input, %d in profile.",
-                channels, fchannels);
+        lsx_fail("noisered: channel mismatch: %lu in input, %lu in profile.",
+                (unsigned long)channels, (unsigned long)fchannels);
         return SOX_EOF;
     }
     if (ifp != stdin)
@@ -123,7 +146,7 @@ static int sox_noisered_start(sox_effect_t * effp)
 }
 
 /* Mangle a single window. Each output sample (except the first and last
- * half-window) is the result of two distinct calls to this function, 
+ * half-window) is the result of two distinct calls to this function,
  * due to overlapping windows. */
 static void reduce_noise(chandata_t* chan, float* window, double level)
 {
@@ -131,12 +154,12 @@ static void reduce_noise(chandata_t* chan, float* window, double level)
     float *smoothing = chan->smoothing;
     int i;
 
-    inr = (float*)xcalloc(WINDOWSIZE * 5, sizeof(float));
+    inr = lsx_calloc(WINDOWSIZE * 5, sizeof(float));
     ini = inr + WINDOWSIZE;
     outr = ini + WINDOWSIZE;
     outi = outr + WINDOWSIZE;
     power = outi + WINDOWSIZE;
-    
+
     for (i = 0; i < FREQCOUNT; i ++)
         assert(smoothing[i] >= 0 && smoothing[i] <= 1);
 
@@ -145,8 +168,8 @@ static void reduce_noise(chandata_t* chan, float* window, double level)
     FFT(WINDOWSIZE, 0, inr, NULL, outr, outi);
 
     memcpy(inr, window, WINDOWSIZE*sizeof(float));
-    WindowFunc(HANNING, WINDOWSIZE, inr);
-    PowerSpectrum(WINDOWSIZE, inr, power);
+    lsx_apply_hann_f(inr, WINDOWSIZE);
+    lsx_power_spectrum_f(WINDOWSIZE, inr, power);
 
     for (i = 0; i < FREQCOUNT; i ++) {
         float smooth;
@@ -156,10 +179,10 @@ static void reduce_noise(chandata_t* chan, float* window, double level)
             smooth = 0.0;
         else
             smooth = 1.0;
-        
+
         smoothing[i] = smooth * 0.5 + smoothing[i] * 0.5;
     }
-    
+
     /* Audacity says this code will eliminate tinkle bells.
      * I have no idea what that means. */
     for (i = 2; i < FREQCOUNT - 2; i ++) {
@@ -171,25 +194,25 @@ static void reduce_noise(chandata_t* chan, float* window, double level)
             smoothing[i+2]<0.1)
             smoothing[i] = 0.0;
     }
-    
+
     outr[0] *= smoothing[0];
     outi[0] *= smoothing[0];
     outr[FREQCOUNT-1] *= smoothing[FREQCOUNT-1];
     outi[FREQCOUNT-1] *= smoothing[FREQCOUNT-1];
-    
+
     for (i = 1; i < FREQCOUNT-1; i ++) {
         int j = WINDOWSIZE - i;
         float smooth = smoothing[i];
-        
+
         outr[i] *= smooth;
         outi[i] *= smooth;
         outr[j] *= smooth;
         outi[j] *= smooth;
     }
-    
+
     FFT(WINDOWSIZE, 1, outr, outi, inr, ini);
-    WindowFunc(HANNING, WINDOWSIZE, inr);
-    
+    lsx_apply_hann_f(inr, WINDOWSIZE);
+
     memcpy(window, inr, WINDOWSIZE*sizeof(float));
 
     for (i = 0; i < FREQCOUNT; i ++)
@@ -200,7 +223,7 @@ static void reduce_noise(chandata_t* chan, float* window, double level)
 
 /* Do window management once we have a complete window, including mangling
  * the current window. */
-static int process_window(sox_effect_t * effp, reddata_t data, unsigned chan_num, unsigned num_chans,
+static int process_window(sox_effect_t * effp, priv_t * data, unsigned chan_num, unsigned num_chans,
                           sox_sample_t *obuf, unsigned len) {
     int j;
     float* nextwindow;
@@ -208,9 +231,9 @@ static int process_window(sox_effect_t * effp, reddata_t data, unsigned chan_num
     chandata_t *chan = &(data->chandata[chan_num]);
     int first = (chan->lastwindow == NULL);
 
-    if ((nextwindow = (float*)xcalloc(WINDOWSIZE, sizeof(float))) == NULL)
+    if ((nextwindow = lsx_calloc(WINDOWSIZE, sizeof(float))) == NULL)
         return SOX_EOF;
-    
+
     memcpy(nextwindow, chan->window+WINDOWSIZE/2,
            sizeof(float)*(WINDOWSIZE/2));
 
@@ -231,27 +254,27 @@ static int process_window(sox_effect_t * effp, reddata_t data, unsigned chan_num
     }
     chan->lastwindow = chan->window;
     chan->window = nextwindow;
-    
+
     return use;
 }
 
 /*
  * Read in windows, and call process_window once we get a whole one.
  */
-static int sox_noisered_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf, 
-                    sox_size_t *isamp, sox_size_t *osamp)
+static int sox_noisered_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf,
+                    size_t *isamp, size_t *osamp)
 {
-    reddata_t data = (reddata_t) effp->priv;
-    sox_size_t samp = min(*isamp, *osamp);
-    sox_size_t tracks = effp->ininfo.channels;
-    sox_size_t track_samples = samp / tracks;
-    sox_size_t ncopy = min(track_samples, WINDOWSIZE-data->bufdata);
-    sox_size_t whole_window = (ncopy + data->bufdata == WINDOWSIZE);
+    priv_t * data = (priv_t *) effp->priv;
+    size_t samp = min(*isamp, *osamp);
+    size_t tracks = effp->in_signal.channels;
+    size_t track_samples = samp / tracks;
+    size_t ncopy = min(track_samples, WINDOWSIZE-data->bufdata);
+    size_t whole_window = (ncopy + data->bufdata == WINDOWSIZE);
     int oldbuf = data->bufdata;
-    sox_size_t i;
+    size_t i;
 
     /* FIXME: Make this automatic for all effects */
-    assert(effp->ininfo.channels == effp->outinfo.channels);
+    assert(effp->in_signal.channels == effp->out_signal.channels);
 
     if (whole_window)
         data->bufdata = WINDOWSIZE/2;
@@ -261,11 +284,11 @@ static int sox_noisered_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_
     /* Reduce noise on every channel. */
     for (i = 0; i < tracks; i ++) {
         chandata_t* chan = &(data->chandata[i]);
-        sox_size_t j;
+        size_t j;
 
         if (chan->window == NULL)
-            chan->window = (float*)xcalloc(WINDOWSIZE, sizeof(float));
-        
+            chan->window = lsx_calloc(WINDOWSIZE, sizeof(float));
+
         for (j = 0; j < ncopy; j ++)
             chan->window[oldbuf + j] =
                 SOX_SAMPLE_TO_FLOAT_32BIT(ibuf[i + tracks * j], effp->clips);
@@ -273,9 +296,9 @@ static int sox_noisered_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_
         if (!whole_window)
             continue;
         else
-            process_window(effp, data, i, tracks, obuf, oldbuf + ncopy);
+            process_window(effp, data, (unsigned) i, (unsigned) tracks, obuf, (unsigned) (oldbuf + ncopy));
     }
-    
+
     *isamp = tracks*ncopy;
     if (whole_window)
         *osamp = tracks*(WINDOWSIZE/2);
@@ -289,13 +312,13 @@ static int sox_noisered_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_
  * We have up to half a window left to dump.
  */
 
-static int sox_noisered_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *osamp)
+static int sox_noisered_drain(sox_effect_t * effp, sox_sample_t *obuf, size_t *osamp)
 {
-    reddata_t data = (reddata_t)effp->priv;
+    priv_t * data = (priv_t *)effp->priv;
     unsigned i;
-    unsigned tracks = effp->ininfo.channels;
+    unsigned tracks = effp->in_signal.channels;
     for (i = 0; i < tracks; i ++)
-        *osamp = process_window(effp, data, i, tracks, obuf, data->bufdata);
+        *osamp = process_window(effp, data, i, tracks, obuf, (unsigned) data->bufdata);
 
     /* FIXME: This is very picky.  osamp needs to be big enough to get all
      * remaining data or it will be discarded.
@@ -308,17 +331,17 @@ static int sox_noisered_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_
  */
 static int sox_noisered_stop(sox_effect_t * effp)
 {
-    reddata_t data = (reddata_t) effp->priv;
-    sox_size_t i;
+    priv_t * data = (priv_t *) effp->priv;
+    size_t i;
 
-    for (i = 0; i < effp->ininfo.channels; i ++) {
+    for (i = 0; i < effp->in_signal.channels; i ++) {
         chandata_t* chan = &(data->chandata[i]);
         free(chan->lastwindow);
         free(chan->window);
         free(chan->smoothing);
         free(chan->noisegate);
     }
-    
+
     free(data->chandata);
 
     return (SOX_SUCCESS);
@@ -333,7 +356,7 @@ static sox_effect_handler_t sox_noisered_effect = {
   sox_noisered_flow,
   sox_noisered_drain,
   sox_noisered_stop,
-  NULL
+  NULL, sizeof(priv_t)
 };
 
 const sox_effect_handler_t *sox_noisered_effect_fn(void)
