@@ -1,5 +1,4 @@
-/*
- * Copyright 1997 Chris Bagwell And Sundry Contributors
+/* Copyright 1997 Chris Bagwell And Sundry Contributors
  * This source code is freely redistributable and may be used for
  * any purpose.  This copyright notice must be maintained.
  * Chris Bagwell And Sundry Contributors are not
@@ -27,7 +26,6 @@
 
 #include "sox_i.h"
 
-#include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <fcntl.h>
@@ -40,60 +38,60 @@
 
 #include <sys/ioctl.h>
 
+typedef sox_fileinfo_t priv_t;
 /* common r/w initialization code */
 static int ossinit(sox_format_t * ft)
 {
     int sampletype, samplesize, dsp_stereo;
     int tmp, rc;
-    sox_fileinfo_t *file = (sox_fileinfo_t *)ft->priv;
+    priv_t *file = (priv_t *)ft->priv;
+    sox_signalinfo_t client_signal = ft->signal;
 
-    if (ft->signal.rate == 0.0) ft->signal.rate = 8000;
-    if (ft->signal.size == -1) ft->signal.size = SOX_SIZE_BYTE;
-    if (ft->signal.size == SOX_SIZE_BYTE) {
+    lsx_set_signal_defaults(&ft->signal);
+    if (ft->encoding.bits_per_sample == 8) {
         sampletype = AFMT_U8;
         samplesize = 8;
-        if (ft->signal.encoding == SOX_ENCODING_UNKNOWN)
-            ft->signal.encoding = SOX_ENCODING_UNSIGNED;
-        if (ft->signal.encoding != SOX_ENCODING_UNSIGNED) {
-            sox_report("OSS driver only supports unsigned with bytes");
-            sox_report("Forcing to unsigned");
-            ft->signal.encoding = SOX_ENCODING_UNSIGNED;
+        if (ft->encoding.encoding == SOX_ENCODING_UNKNOWN)
+            ft->encoding.encoding = SOX_ENCODING_UNSIGNED;
+        if (ft->encoding.encoding != SOX_ENCODING_UNSIGNED) {
+            lsx_report("OSS driver only supports unsigned with bytes");
+            lsx_report("Forcing to unsigned");
+            ft->encoding.encoding = SOX_ENCODING_UNSIGNED;
         }
     }
-    else if (ft->signal.size == SOX_SIZE_16BIT) {
+    else if (ft->encoding.bits_per_sample == 16) {
         /* Attempt to use endian that user specified */
-        if (ft->signal.reverse_bytes)
-            sampletype = (SOX_IS_BIGENDIAN) ? AFMT_S16_LE : AFMT_S16_BE;
+        if (ft->encoding.reverse_bytes)
+            sampletype = (MACHINE_IS_BIGENDIAN) ? AFMT_S16_LE : AFMT_S16_BE;
         else
-            sampletype = (SOX_IS_BIGENDIAN) ? AFMT_S16_BE : AFMT_S16_LE;
+            sampletype = (MACHINE_IS_BIGENDIAN) ? AFMT_S16_BE : AFMT_S16_LE;
         samplesize = 16;
-        if (ft->signal.encoding == SOX_ENCODING_UNKNOWN)
-            ft->signal.encoding = SOX_ENCODING_SIGN2;
-        if (ft->signal.encoding != SOX_ENCODING_SIGN2) {
-            sox_report("OSS driver only supports signed with words");
-            sox_report("Forcing to signed linear");
-            ft->signal.encoding = SOX_ENCODING_SIGN2;
+        if (ft->encoding.encoding == SOX_ENCODING_UNKNOWN)
+            ft->encoding.encoding = SOX_ENCODING_SIGN2;
+        if (ft->encoding.encoding != SOX_ENCODING_SIGN2) {
+            lsx_report("OSS driver only supports signed with words");
+            lsx_report("Forcing to signed linear");
+            ft->encoding.encoding = SOX_ENCODING_SIGN2;
         }
     }
     else {
         /* Attempt to use endian that user specified */
-        if (ft->signal.reverse_bytes)
-            sampletype = (SOX_IS_BIGENDIAN) ? AFMT_S16_LE : AFMT_S16_BE;
+        if (ft->encoding.reverse_bytes)
+            sampletype = (MACHINE_IS_BIGENDIAN) ? AFMT_S16_LE : AFMT_S16_BE;
         else
-            sampletype = (SOX_IS_BIGENDIAN) ? AFMT_S16_BE : AFMT_S16_LE;
+            sampletype = (MACHINE_IS_BIGENDIAN) ? AFMT_S16_BE : AFMT_S16_LE;
         samplesize = 16;
-        ft->signal.size = SOX_SIZE_16BIT;
-        ft->signal.encoding = SOX_ENCODING_SIGN2;
-        sox_report("OSS driver only supports bytes and words");
-        sox_report("Forcing to signed linear word");
+        ft->encoding.bits_per_sample = 16;
+        ft->encoding.encoding = SOX_ENCODING_SIGN2;
+        lsx_report("OSS driver only supports bytes and words");
+        lsx_report("Forcing to signed linear word");
     }
 
-    if (ft->signal.channels == 0) ft->signal.channels = 1;
-    else if (ft->signal.channels > 2) ft->signal.channels = 2;
+    if (ft->signal.channels > 2) ft->signal.channels = 2;
 
-    if (ioctl(fileno(ft->fp), SNDCTL_DSP_RESET, 0) < 0)
+    if (ioctl(fileno(ft->fp), (size_t) SNDCTL_DSP_RESET, 0) < 0)
     {
-        sox_fail_errno(ft,SOX_EOF,"Unable to reset OSS driver.  Possibly accessing an invalid file/device");
+        lsx_fail_errno(ft,SOX_EOF,"Unable to reset OSS driver.  Possibly accessing an invalid file/device");
         return(SOX_EOF);
     }
 
@@ -107,21 +105,21 @@ static int ossinit(sox_format_t * ft)
             if (samplesize == 16 && (tmp & (AFMT_S16_LE|AFMT_S16_BE)) == 0)
             {
                 /* Must not like 16-bits, try 8-bits */
-                ft->signal.size = SOX_SIZE_BYTE;
-                ft->signal.encoding = SOX_ENCODING_UNSIGNED;
-                sox_report("OSS driver doesn't like signed words");
-                sox_report("Forcing to unsigned bytes");
+                ft->encoding.bits_per_sample = 8;
+                ft->encoding.encoding = SOX_ENCODING_UNSIGNED;
+                lsx_report("OSS driver doesn't like signed words");
+                lsx_report("Forcing to unsigned bytes");
                 tmp = sampletype = AFMT_U8;
                 samplesize = 8;
             }
             /* is 8-bit supported */
             else if (samplesize == 8 && (tmp & AFMT_U8) == 0)
             {
-                ft->signal.size = SOX_SIZE_16BIT;
-                ft->signal.encoding = SOX_ENCODING_SIGN2;
-                sox_report("OSS driver doesn't like unsigned bytes");
-                sox_report("Forcing to signed words");
-                sampletype = (SOX_IS_BIGENDIAN) ? AFMT_S16_BE : AFMT_S16_LE;
+                ft->encoding.bits_per_sample = 16;
+                ft->encoding.encoding = SOX_ENCODING_SIGN2;
+                lsx_report("OSS driver doesn't like unsigned bytes");
+                lsx_report("Forcing to signed words");
+                sampletype = (MACHINE_IS_BIGENDIAN) ? AFMT_S16_BE : AFMT_S16_LE;
                 samplesize = 16;
             }
             /* determine which 16-bit format to use */
@@ -133,7 +131,7 @@ static int ossinit(sox_format_t * ft)
                  * it supports at least one of the two endians.
                  */
                 sampletype = (sampletype == AFMT_S16_BE) ? AFMT_S16_LE : AFMT_S16_BE;
-                ft->signal.reverse_bytes = !ft->signal.reverse_bytes;
+                ft->encoding.reverse_bytes = !ft->encoding.reverse_bytes;
             }
 
         }
@@ -143,7 +141,7 @@ static int ossinit(sox_format_t * ft)
     /* Give up and exit */
     if (rc < 0 || tmp != sampletype)
     {
-        sox_fail_errno(ft,SOX_EOF,"Unable to set the sample size to %d", samplesize);
+        lsx_fail_errno(ft,SOX_EOF,"Unable to set the sample size to %d", samplesize);
         return (SOX_EOF);
     }
 
@@ -153,18 +151,19 @@ static int ossinit(sox_format_t * ft)
     tmp = dsp_stereo;
     if (ioctl(fileno(ft->fp), SNDCTL_DSP_STEREO, &tmp) < 0)
     {
-        sox_warn("Couldn't set to %s", dsp_stereo?  "stereo":"mono");
+        lsx_warn("Couldn't set to %s", dsp_stereo?  "stereo":"mono");
         dsp_stereo = 0;
     }
 
     if (tmp != dsp_stereo)
     {
-        sox_warn("Sound card appears to only support %d channels.  Overriding format", tmp+1);
+      if (client_signal.channels != 0)
+        lsx_warn("Sound card appears to support only %d channels.  Overriding format", tmp+1);
         ft->signal.channels = tmp + 1;
     }
 
     tmp = ft->signal.rate;
-    if (ioctl (fileno(ft->fp), SNDCTL_DSP_SPEED, &tmp) < 0 || 
+    if (ioctl (fileno(ft->fp), SNDCTL_DSP_SPEED, &tmp) < 0 ||
         (int)ft->signal.rate != tmp) {
         /* If the rate the sound card is using is not within 1% of what
          * the user specified then override the user setting.
@@ -174,9 +173,10 @@ static int ossinit(sox_format_t * ft)
          * this and having strange output file rates for something that
          * we can't hear anyways.
          */
-        if ((int)ft->signal.rate - tmp > (tmp * .01) || 
+        if ((int)ft->signal.rate - tmp > (tmp * .01) ||
             tmp - (int)ft->signal.rate > (tmp * .01)) {
-            sox_warn("Unable to set audio speed to %g (set to %d)",
+          if (client_signal.rate != 0)
+            lsx_warn("Unable to set audio speed to %g (set to %d)",
                      ft->signal.rate, tmp);
             ft->signal.rate = tmp;
         }
@@ -188,15 +188,15 @@ static int ossinit(sox_format_t * ft)
     file->size = 0;
     ioctl (fileno(ft->fp), SNDCTL_DSP_GETBLKSIZE, &file->size);
     if (file->size < 4 || file->size > 65536) {
-            sox_fail_errno(ft,SOX_EOF,"Invalid audio buffer size %d", file->size);
+            lsx_fail_errno(ft,SOX_EOF,"Invalid audio buffer size %lu", (unsigned long)file->size);
             return (SOX_EOF);
     }
     file->count = 0;
     file->pos = 0;
-    file->buf = (char *)xmalloc(file->size);
+    file->buf = lsx_malloc(file->size);
 
-    if (ioctl(fileno(ft->fp), SNDCTL_DSP_SYNC, NULL) < 0) {
-        sox_fail_errno(ft,SOX_EOF,"Unable to sync dsp");
+    if (ioctl(fileno(ft->fp), (size_t) SNDCTL_DSP_SYNC, NULL) < 0) {
+        lsx_fail_errno(ft,SOX_EOF,"Unable to sync dsp");
         return (SOX_EOF);
     }
 
@@ -205,47 +205,19 @@ static int ossinit(sox_format_t * ft)
     return(SOX_SUCCESS);
 }
 
-/*
- * Do anything required before you start reading samples.
- * Read file header.
- *      Find out sampling rate,
- *      size and encoding of samples,
- *      mono/stereo/quad.
- */
-static int sox_ossstartread(sox_format_t * ft)
+SOX_FORMAT_HANDLER(oss)
 {
-    int rc;
-    rc = ossinit(ft);
-    return rc;
-}
-
-static int sox_ossstartwrite(sox_format_t * ft)
-{
-    return ossinit(ft);
-}
-
-/* OSS /dev/dsp player */
-static const char *ossnames[] = {
-  "ossdsp",
-  "oss",
-  NULL
-};
-
-static sox_format_handler_t sox_oss_format = {
-  ossnames,
-  SOX_FILE_DEVICE,
-  sox_ossstartread,
-  sox_rawread,
-  sox_rawstopread,
-  sox_ossstartwrite,
-  sox_rawwrite,
-  sox_rawstopwrite,
-  sox_format_nothing_seek
-};
-
-const sox_format_handler_t *sox_oss_format_fn(void);
-
-const sox_format_handler_t *sox_oss_format_fn(void)
-{
-    return &sox_oss_format;
+  static char const * const names[] = {"ossdsp", "oss", NULL};
+  static unsigned const write_encodings[] = {
+    SOX_ENCODING_SIGN2, 16, 0,
+    SOX_ENCODING_UNSIGNED, 8, 0,
+    0};
+  static sox_format_handler_t const handler = {SOX_LIB_VERSION_CODE,
+    "Open Sound Sytem device driver for unix-like systems",
+    names, SOX_FILE_DEVICE,
+    ossinit, lsx_rawread, lsx_rawstopread,
+    ossinit, lsx_rawwrite, lsx_rawstopwrite,
+    NULL, write_encodings, NULL, sizeof(priv_t)
+  };
+  return &handler;
 }
