@@ -1,33 +1,32 @@
-/*
- * libSoX rate change effect file.
+/* libSoX rate change effect file.
  *
  * July 14, 1998
  * Copyright 1998  K. Bradley, Carnegie Mellon University
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License as published by the Free Software Foundation; either
- *   version 2 of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or (at
+ * your option) any later version.
  *
- *   This library is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *   Lesser General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+ * General Public License for more details.
  *
- *   You should have received a copy of the GNU Lesser General Public
- *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * October 29, 1999
  * Various changes, bugfixes, speedups, by Stan Brooks.
  *
  */
 
-#include <math.h>
+#include "sox_i.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "sox_i.h"
 
 #define Float float/*double*/
 #define ISCALE 0x10000
@@ -43,29 +42,26 @@ typedef struct {
   Float *window;  /* this is past_hist[hsize], then input[size]     */
 } polystage;
 
-typedef struct polyphase {
-  sox_rate_t lcmrate;             /* least common multiple of rates */
-  sox_rate_t inskip, outskip;     /* LCM increments for I & O rates */
+typedef struct {
+  unsigned lcmrate;             /* least common multiple of rates */
+  unsigned inskip, outskip;     /* LCM increments for I & O rates */
   double Factor;                 /* out_rate/in_rate               */
   unsigned long total;           /* number of filter stages        */
-  sox_size_t oskip;               /* output samples to skip at start*/
+  size_t oskip;               /* output samples to skip at start*/
   double inpipe;                 /* output samples 'in the pipe'   */
   polystage *stage[MF];          /* array of pointers to polystage structs */
   int win_type;
   int win_width;
   Float cutoff;
   int m1[MF], m2[MF], b1[MF], b2[MF]; /* arrays used in optimize_factors */
-} *poly_t;
-
-assert_static(sizeof(struct polyphase) <= SOX_MAX_EFFECT_PRIVSIZE, 
-              /* else */ skeleff_PRIVSIZE_too_big);
+} priv_t;
 
 /*
  * Process options
  */
 static int sox_poly_getopts(sox_effect_t * effp, int n, char **argv)
 {
-  poly_t rate = (poly_t) effp->priv;
+  priv_t * rate = (priv_t *) effp->priv;
 
   rate->win_type = 0;           /* 0: nuttall, 1: hamming */
   rate->win_width = 1024;
@@ -99,10 +95,10 @@ static int sox_poly_getopts(sox_effect_t * effp, int n, char **argv)
       continue;
     }
 
-    sox_fail("Polyphase: unknown argument (%s %s)!", argv[0], argv[1]);
+    lsx_fail("Polyphase: unknown argument (%s %s)!", argv[0], argv[1]);
     return SOX_EOF;
   }
-  
+
   return SOX_SUCCESS;
 }
 
@@ -134,19 +130,19 @@ static int prime(unsigned n, int *q0)
 
   p = primes;
   q = q0;
-  sox_debug("factors(%d) =",n);
+  lsx_debug("factors(%d) =",n);
   while (n > 1) {
     while ((pr = *p) && (n % pr)) p++;
     if (!pr) {
-      sox_fail("Number %d too large of a prime.",n);
+      lsx_fail("Number %d too large of a prime.",n);
       pr = n;
     }
     *q++ = pr;
     n /= pr;
   }
   *q = 0;
-  for (pr=0; pr<q-q0; pr++) sox_debug(" %d",q0[pr]);
-  sox_debug(" ");
+  for (pr=0; pr<q-q0; pr++) lsx_debug(" %d",q0[pr]);
+  lsx_debug(" ");
   return (q-q0);
 }
 
@@ -184,12 +180,12 @@ static int permute(int *m, int *l, int ct, int ct1, size_t amalg)
   }
   if (n) *p++=n;
   *p = 0;
-  /*for (k=0; k<p-m; k++) sox_debug(" %d",m[k]);*/
-  /*sox_debug("");*/
+  /*for (k=0; k<p-m; k++) lsx_debug(" %d",m[k]);*/
+  /*lsx_debug("");*/
   return (p-m);
 }
 
-static int optimize_factors(poly_t rate, unsigned numer, unsigned denom, int *l1, int *l2)
+static int optimize_factors(priv_t * rate, unsigned numer, unsigned denom, int *l1, int *l2)
 {
   unsigned f_min;
   int c_min,u_min,ct1,ct2;
@@ -216,9 +212,9 @@ static int optimize_factors(poly_t rate, unsigned numer, unsigned denom, int *l1
       cost = 0;
       f = denom;
       u = min(ct1,ct2) + 1;
-      /*sox_debug("pfacts(%d): ", numer);*/
+      /*lsx_debug("pfacts(%d): ", numer);*/
       u1 = permute(rate->m1,l1,ct1,u,amalg);
-      /*sox_debug("pfacts(%d): ", denom);*/
+      /*lsx_debug("pfacts(%d): ", denom);*/
       u2 = permute(rate->m2,l2,ct2,u,amalg);
       u = max(u1,u2);
       for (j=0; j<u; j++) {
@@ -232,10 +228,10 @@ static int optimize_factors(poly_t rate, unsigned numer, unsigned denom, int *l1
         c_min = cost;
         u_min = u;
         if (sox_globals.verbosity >= 4) {
-          sox_debug("c_min %d, [%d-%d]:",c_min,numer,denom);
+          lsx_debug("c_min %d, [%d-%d]:",c_min,numer,denom);
           for (j=0; j<u; j++)
-            sox_debug(" (%d,%d)",rate->m1[j],rate->m2[j]);
-          sox_debug(" ");
+            lsx_debug(" (%d,%d)",rate->m1[j],rate->m2[j]);
+          lsx_debug(" ");
         }
         memcpy(rate->b1,rate->m1,u*sizeof(int));
         memcpy(rate->b2,rate->m2,u*sizeof(int));
@@ -265,7 +261,7 @@ static void nuttall(Float *buffer, int length)
   int N1;
 
   if(buffer == NULL || length <= 0)
-    sox_fail("Illegal buffer %p or length %d to nuttall.", (void *)buffer, length);
+    lsx_fail("Illegal buffer %p or length %d to nuttall.", (void *)buffer, length);
 
   /* Initial variable setups. */
   N = length;
@@ -288,7 +284,7 @@ static void hamming(Float *buffer, int length)
     int N1;
 
     if(buffer == NULL || length <= 0)
-      sox_fail("Illegal buffer %p or length %d to hamming.",(void *)buffer,length);
+      lsx_fail("Illegal buffer %p or length %d to hamming.",(void *)buffer,length);
 
     N1 = length/2;
     for(j=0;j<length;j++)
@@ -308,13 +304,13 @@ static Float sinc(double value)
 
    buffer must already be allocated.
 */
-static void fir_design(poly_t rate, Float *buffer, int length, double cutoff)
+static void fir_design(priv_t * rate, Float *buffer, int length, double cutoff)
 {
     int j;
     double sum;
 
     if(buffer == NULL || length < 0 || cutoff < 0 || cutoff > M_PI)
-      sox_fail("Illegal buffer %p, length %d, or cutoff %f.",(void *)buffer,length,cutoff);
+      lsx_fail("Illegal buffer %p, length %d, or cutoff %f.",(void *)buffer,length,cutoff);
 
     /* Use the user-option of window type */
     if (rate->win_type == 0)
@@ -322,12 +318,12 @@ static void fir_design(poly_t rate, Float *buffer, int length, double cutoff)
     else
       hamming(buffer,length);  /* Design Hamming window:  43 dB cutoff */
 
-    /* sox_debug("# fir_design length=%d, cutoff=%8.4f",length,cutoff); */
+    /* lsx_debug("# fir_design length=%d, cutoff=%8.4f",length,cutoff); */
     /* Design filter:  windowed sinc function */
     sum = 0.0;
     for(j=0;j<length;j++) {
       buffer[j] *= sinc(M_PI*cutoff*(j-length/2)); /* center at length/2 */
-      /* sox_debug("%.1f %.6f",(float)j,buffer[j]); */
+      /* lsx_debug("%.1f %.6f",(float)j,buffer[j]); */
       sum += buffer[j];
     }
     sum = (double)1.0/sum;
@@ -335,34 +331,34 @@ static void fir_design(poly_t rate, Float *buffer, int length, double cutoff)
     for(j=0;j<length;j++) {
       buffer[j] *= sum;
     }
-    /* sox_debug("# end"); */
+    /* lsx_debug("# end"); */
 }
 
 #define RIBLEN 2048
 
 static int sox_poly_start(sox_effect_t * effp)
 {
-    poly_t rate = (poly_t) effp->priv;
+    priv_t * rate = (priv_t *) effp->priv;
     static int l1[MF], l2[MF];
     double skip = 0;
     int total, size, uprate;
     int k;
 
-    if (effp->ininfo.rate == effp->outinfo.rate)
+    if (effp->in_signal.rate == effp->out_signal.rate)
       return SOX_EFF_NULL;
 
-    effp->outinfo.channels = effp->ininfo.channels;
+    effp->out_signal.channels = effp->in_signal.channels;
 
-    rate->lcmrate = sox_lcm((sox_sample_t)effp->ininfo.rate,
-                           (sox_sample_t)effp->outinfo.rate);
+    rate->lcmrate = lsx_lcm((unsigned)effp->in_signal.rate,
+                           (unsigned)effp->out_signal.rate);
 
     /* Cursory check for LCM overflow.
      * If both rates are below 65k, there should be no problem.
      * 16 bits x 16 bits = 32 bits, which we can handle.
      */
 
-    rate->inskip = rate->lcmrate / (sox_sample_t)effp->ininfo.rate;
-    rate->outskip = rate->lcmrate / (sox_sample_t)effp->outinfo.rate;
+    rate->inskip = rate->lcmrate / (sox_sample_t)effp->in_signal.rate;
+    rate->outskip = rate->lcmrate / (sox_sample_t)effp->out_signal.rate;
     rate->Factor = (double)rate->inskip / (double)rate->outskip;
     rate->inpipe = 0;
     {
@@ -376,18 +372,18 @@ static int sox_poly_start(sox_effect_t * effp)
     rate->total = total;
     /* l1 and l2 are now lists of the up/down factors for conversion */
 
-    sox_debug("Poly:  input rate %g, output rate %g.  %d stages.",
-            effp->ininfo.rate, effp->outinfo.rate,total);
-    sox_debug("Poly:  window: %s  size: %d  cutoff: %f.",
+    lsx_debug("Poly:  input rate %g, output rate %g.  %d stages.",
+            effp->in_signal.rate, effp->out_signal.rate,total);
+    lsx_debug("Poly:  window: %s  size: %d  cutoff: %f.",
             (rate->win_type == 0) ? ("nut") : ("ham"), rate->win_width, rate->cutoff);
 
     /* Create an array of filters and past history */
-    uprate = effp->ininfo.rate;
+    uprate = effp->in_signal.rate;
     for (k = 0; k < total; k++) {
       int j, prod, f_cutoff, f_len;
       polystage *s;
 
-      rate->stage[k] = s = (polystage*) xmalloc(sizeof(polystage));
+      rate->stage[k] = s = lsx_malloc(sizeof(polystage));
       s->up = l1[k];
       s->down = l2[k];
       f_cutoff = max(s->up, s->down);
@@ -398,17 +394,17 @@ static int sox_poly_start(sox_effect_t * effp)
       s->size = size;
       s->hsize = f_len/s->up; /* this much of window is past-history */
       s->held = 0;
-      sox_debug("Poly:  stage %d:  Up by %d, down by %d,  i_samps %d, hsize %d",
+      lsx_debug("Poly:  stage %d:  Up by %d, down by %d,  i_samps %d, hsize %d",
               k+1,s->up,s->down,size, s->hsize);
       s->filt_len = f_len;
-      s->filt_array = (Float *) xmalloc(sizeof(Float) * f_len);
-      s->window = (Float *) xmalloc(sizeof(Float) * (s->hsize+size));
+      s->filt_array = lsx_malloc(sizeof(Float) * f_len);
+      s->window = lsx_malloc(sizeof(Float) * (s->hsize+size));
       /* zero past_history section of window */
       for(j = 0; j < s->hsize; j++)
         s->window[j] = 0.0;
 
       uprate *= s->up;
-      sox_debug("Poly:         :  filt_len %d, cutoff freq %.1f",
+      lsx_debug("Poly:         :  filt_len %d, cutoff freq %.1f",
               f_len, uprate * rate->cutoff / f_cutoff);
       uprate /= s->down;
       fir_design(rate, s->filt_array, f_len, rate->cutoff / f_cutoff);
@@ -416,22 +412,22 @@ static int sox_poly_start(sox_effect_t * effp)
       skip *= s->up;
       skip += f_len;
       skip /= s->down;
-      
+
       size = (size * s->up) / s->down;  /* this is integer */
     }
     rate->oskip = skip/2;
     { /* bogus last stage is for output buffering */
       polystage *s;
-      rate->stage[k] = s = (polystage*) xmalloc(sizeof(polystage));
+      rate->stage[k] = s = lsx_malloc(sizeof(polystage));
       s->up = s->down = 0;
       s->size = size;
       s->hsize = 0;
       s->held = 0;
       s->filt_len = 0;
       s->filt_array = NULL;
-      s->window = (Float *) xmalloc(sizeof(Float) * size);
+      s->window = lsx_malloc(sizeof(Float) * size);
     }
-    sox_debug("Poly:  output samples %d, oskip %d",size, rate->oskip);
+    lsx_debug("Poly:  output samples %d, oskip %lu",size, (unsigned long)rate->oskip);
     return (SOX_SUCCESS);
 }
 
@@ -465,10 +461,10 @@ static void polyphase(Float *output, polystage *s)
   Float *o_top;
 
   in = s->window + s->hsize;
-  /*for (mm=0; mm<s->filt_len; mm++) sox_debug("cf_%d %f",mm,s->filt_array[mm]);*/
+  /*for (mm=0; mm<s->filt_len; mm++) lsx_debug("cf_%d %f",mm,s->filt_array[mm]);*/
   /* assumes s->size divisible by down (now true) */
   o_top = output + (s->size * up) / down;
-  /*sox_debug(" isize %d, osize %d, up %d, down %d, N %d", s->size, o_top-output, up, down, f_len);*/
+  /*lsx_debug(" isize %d, osize %d, up %d, down %d, N %d", s->size, o_top-output, up, down, f_len);*/
   for (mm=0, o=output; o < o_top; mm+=down, o++) {
     double sum;
     const Float *p, *q;
@@ -491,17 +487,17 @@ static void update_hist(Float *hist, int hist_size, int in_size)
 }
 
 static int sox_poly_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf,
-                 sox_size_t *isamp, sox_size_t *osamp)
+                 size_t *isamp, size_t *osamp)
 {
-  poly_t rate = (poly_t) effp->priv;
+  priv_t * rate = (priv_t *) effp->priv;
   polystage *s0,*s1;
 
   /* Sanity check:  how much can we tolerate? */
-  /* sox_debug("*isamp=%d *osamp=%d",*isamp,*osamp); */
+  /* lsx_debug("*isamp=%d *osamp=%d",*isamp,*osamp); */
   s0 = rate->stage[0];            /* the first stage */
   s1 = rate->stage[rate->total];  /* the 'last' stage is output buffer */
   {
-    sox_size_t in_size, gap, k;
+    size_t in_size, gap, k;
 
     in_size = *isamp;
     gap = s0->size - s0->held; /* space available in this 'input' buffer */
@@ -525,7 +521,7 @@ static int sox_poly_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_samp
   }
 
   if (s0->held == s0->size && s1->held == 0) {
-    sox_size_t k;
+    size_t k;
     /* input buffer full, output buffer empty, so do process */
 
     for(k=0; k<rate->total; k++) {
@@ -536,7 +532,7 @@ static int sox_poly_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_samp
 
       out = rate->stage[k+1]->window + rate->stage[k+1]->hsize;
 
-      /* sox_debug("k=%d  insize=%d",k,in_size); */
+      /* lsx_debug("k=%d  insize=%d",k,in_size); */
       polyphase(out, s);
 
       /* copy input history into lower portion of rate->window[k] */
@@ -551,10 +547,10 @@ static int sox_poly_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_samp
 
   {
     sox_sample_t *q;
-    sox_size_t out_size;
-    sox_size_t oskip;
+    size_t out_size;
+    size_t oskip;
     Float *out_buf;
-    sox_size_t k;
+    size_t k;
 
     oskip = rate->oskip;
                 out_size = s1->held;
@@ -593,9 +589,9 @@ static int sox_poly_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_samp
 /*
  * Process tail of input samples.
  */
-static int sox_poly_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *osamp)
+static int sox_poly_drain(sox_effect_t * effp, sox_sample_t *obuf, size_t *osamp)
 {
-  sox_size_t in_size;
+  size_t in_size;
   /* Call "flow" with NULL input. */
   sox_poly_flow(effp, NULL, obuf, &in_size, osamp);
   return (SOX_SUCCESS);
@@ -607,8 +603,8 @@ static int sox_poly_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *o
  */
 static int sox_poly_stop(sox_effect_t * effp)
 {
-    poly_t rate = (poly_t)effp->priv;
-    sox_size_t k;
+    priv_t * rate = (priv_t *)effp->priv;
+    size_t k;
 
     for (k = 0; k <= rate->total; k++) {
       free(rate->stage[k]->window);
@@ -625,13 +621,13 @@ static sox_effect_handler_t sox_polyphase_effect = {
   "       -width n       window width in samples [default 1024]\n"
   "\n"
   "       -cutoff float  frequency cutoff for base bandwidth [default 0.95]",
-  SOX_EFF_RATE,
+  SOX_EFF_RATE | SOX_EFF_DEPRECATED,
   sox_poly_getopts,
   sox_poly_start,
   sox_poly_flow,
   sox_poly_drain,
   sox_poly_stop,
-  NULL
+  NULL, sizeof(priv_t)
 };
 
 const sox_effect_handler_t *sox_polyphase_effect_fn(void)
