@@ -4,7 +4,7 @@
  * Minor addition by Donnie Smith 13.08.2003
  *
  * This effect can delete samples from the start of a sound file
- * until it sees a specified count of samples exceed a given threshold 
+ * until it sees a specified count of samples exceed a given threshold
  * (any of the channels).
  * This effect can also delete samples from the end of a sound file
  * when it sees a specified count of samples below a given threshold
@@ -13,10 +13,9 @@
  * Thesholds can be given as either a percentage or in decibels.
  */
 
+#include "sox_i.h"
 
 #include <string.h>
-#include <math.h>
-#include "sox_i.h"
 
 /* Private data for silence effect. */
 
@@ -26,51 +25,50 @@
 #define SILENCE_COPY_FLUSH  3
 #define SILENCE_STOP        4
 
-typedef struct silencestuff
-{
+typedef struct {
     char        start;
     int         start_periods;
     char        *start_duration_str;
-    sox_size_t   start_duration;
+    size_t   start_duration;
     double      start_threshold;
     char        start_unit; /* "d" for decibels or "%" for percent. */
     int         restart;
 
     sox_sample_t *start_holdoff;
-    sox_size_t   start_holdoff_offset;
-    sox_size_t   start_holdoff_end;
+    size_t   start_holdoff_offset;
+    size_t   start_holdoff_end;
     int         start_found_periods;
 
     char        stop;
     int         stop_periods;
     char        *stop_duration_str;
-    sox_size_t   stop_duration;
+    size_t   stop_duration;
     double      stop_threshold;
     char        stop_unit;
 
     sox_sample_t *stop_holdoff;
-    sox_size_t   stop_holdoff_offset;
-    sox_size_t   stop_holdoff_end;
+    size_t   stop_holdoff_offset;
+    size_t   stop_holdoff_end;
     int         stop_found_periods;
 
     double      *window;
     double      *window_current;
     double      *window_end;
-    sox_size_t   window_size;
+    size_t   window_size;
     double      rms_sum;
 
     char        leave_silence;
 
     /* State Machine */
     char        mode;
-} *silence_t;
+} priv_t;
 
 static void clear_rms(sox_effect_t * effp)
 
 {
-    silence_t silence = (silence_t) effp->priv;
+    priv_t * silence = (priv_t *) effp->priv;
 
-    memset(silence->window, 0, 
+    memset(silence->window, 0,
            silence->window_size * sizeof(double));
 
     silence->window_current = silence->window;
@@ -80,7 +78,7 @@ static void clear_rms(sox_effect_t * effp)
 
 static int sox_silence_getopts(sox_effect_t * effp, int n, char **argv)
 {
-    silence_t   silence = (silence_t) effp->priv;
+    priv_t *   silence = (priv_t *) effp->priv;
     int parse_count;
 
     /* check for option switches */
@@ -94,15 +92,15 @@ static int sox_silence_getopts(sox_effect_t * effp, int n, char **argv)
     }
 
     if (n < 1)
-      return sox_usage(effp);
+      return lsx_usage(effp);
 
     /* Parse data related to trimming front side */
     silence->start = sox_false;
     if (sscanf(argv[0], "%d", &silence->start_periods) != 1)
-      return sox_usage(effp);
+      return lsx_usage(effp);
     if (silence->start_periods < 0)
     {
-        sox_fail("Periods must not be negative");
+        lsx_fail("Periods must not be negative");
         return(SOX_EOF);
     }
     argv++;
@@ -112,23 +110,23 @@ static int sox_silence_getopts(sox_effect_t * effp, int n, char **argv)
     {
         silence->start = sox_true;
         if (n < 2)
-          return sox_usage(effp);
+          return lsx_usage(effp);
 
         /* We do not know the sample rate so we can not fully
          * parse the duration info yet.  So save argument off
          * for future processing.
          */
-        silence->start_duration_str = (char *)xmalloc(strlen(argv[0])+1);
+        silence->start_duration_str = lsx_malloc(strlen(argv[0])+1);
         strcpy(silence->start_duration_str,argv[0]);
         /* Perform a fake parse to do error checking */
-        if (sox_parsesamples(0,silence->start_duration_str,
+        if (lsx_parsesamples(0.,silence->start_duration_str,
                     &silence->start_duration,'s') == NULL)
-          return sox_usage(effp);
+          return lsx_usage(effp);
 
-        parse_count = sscanf(argv[1], "%lf%c", &silence->start_threshold, 
+        parse_count = sscanf(argv[1], "%lf%c", &silence->start_threshold,
                 &silence->start_unit);
         if (parse_count < 1)
-          return sox_usage(effp);
+          return lsx_usage(effp);
         else if (parse_count < 2)
             silence->start_unit = '%';
 
@@ -141,9 +139,9 @@ static int sox_silence_getopts(sox_effect_t * effp, int n, char **argv)
     if (n > 0)
     {
         if (n < 3)
-          return sox_usage(effp);
+          return lsx_usage(effp);
         if (sscanf(argv[0], "%d", &silence->stop_periods) != 1)
-          return sox_usage(effp);
+          return lsx_usage(effp);
         if (silence->stop_periods < 0)
         {
             silence->stop_periods = -silence->stop_periods;
@@ -159,17 +157,17 @@ static int sox_silence_getopts(sox_effect_t * effp, int n, char **argv)
          * parse the duration info yet.  So save argument off
          * for future processing.
          */
-        silence->stop_duration_str = (char *)xmalloc(strlen(argv[0])+1);
+        silence->stop_duration_str = lsx_malloc(strlen(argv[0])+1);
         strcpy(silence->stop_duration_str,argv[0]);
         /* Perform a fake parse to do error checking */
-        if (sox_parsesamples(0,silence->stop_duration_str,
+        if (lsx_parsesamples(0.,silence->stop_duration_str,
                     &silence->stop_duration,'s') == NULL)
-          return sox_usage(effp);
+          return lsx_usage(effp);
 
-        parse_count = sscanf(argv[1], "%lf%c", &silence->stop_threshold, 
+        parse_count = sscanf(argv[1], "%lf%c", &silence->stop_threshold,
                              &silence->stop_unit);
         if (parse_count < 1)
-          return sox_usage(effp);
+          return lsx_usage(effp);
         else if (parse_count < 2)
             silence->stop_unit = '%';
 
@@ -182,18 +180,18 @@ static int sox_silence_getopts(sox_effect_t * effp, int n, char **argv)
     {
         if ((silence->start_unit != '%') && (silence->start_unit != 'd'))
         {
-            sox_fail("Invalid unit specified");
-            return sox_usage(effp);
+            lsx_fail("Invalid unit specified");
+            return lsx_usage(effp);
         }
         if ((silence->start_unit == '%') && ((silence->start_threshold < 0.0)
             || (silence->start_threshold > 100.0)))
         {
-            sox_fail("silence threshold should be between 0.0 and 100.0 %%");
+            lsx_fail("silence threshold should be between 0.0 and 100.0 %%");
             return (SOX_EOF);
         }
         if ((silence->start_unit == 'd') && (silence->start_threshold >= 0.0))
         {
-            sox_fail("silence threshold should be less than 0.0 dB");
+            lsx_fail("silence threshold should be less than 0.0 dB");
             return(SOX_EOF);
         }
     }
@@ -202,18 +200,18 @@ static int sox_silence_getopts(sox_effect_t * effp, int n, char **argv)
     {
         if ((silence->stop_unit != '%') && (silence->stop_unit != 'd'))
         {
-            sox_fail("Invalid unit specified");
+            lsx_fail("Invalid unit specified");
             return(SOX_EOF);
         }
-        if ((silence->stop_unit == '%') && ((silence->stop_threshold < 0.0) || 
+        if ((silence->stop_unit == '%') && ((silence->stop_threshold < 0.0) ||
                     (silence->stop_threshold > 100.0)))
         {
-            sox_fail("silence threshold should be between 0.0 and 100.0 %%");
+            lsx_fail("silence threshold should be between 0.0 and 100.0 %%");
             return (SOX_EOF);
         }
         if ((silence->stop_unit == 'd') && (silence->stop_threshold >= 0.0))
         {
-            sox_fail("silence threshold should be less than 0.0 dB");
+            lsx_fail("silence threshold should be less than 0.0 dB");
             return(SOX_EOF);
         }
     }
@@ -222,31 +220,30 @@ static int sox_silence_getopts(sox_effect_t * effp, int n, char **argv)
 
 static int sox_silence_start(sox_effect_t * effp)
 {
-        silence_t       silence = (silence_t) effp->priv;
+        priv_t *       silence = (priv_t *) effp->priv;
 
         /* When you want to remove silence, small window sizes are
          * better or else RMS will look like non-silence at
          * aburpt changes from load to silence.
          */
-        silence->window_size = (effp->ininfo.rate / 50) * 
-                               effp->ininfo.channels;
-        silence->window = (double *)xmalloc(silence->window_size *
-                                           sizeof(double));
+        silence->window_size = (effp->in_signal.rate / 50) *
+                               effp->in_signal.channels;
+        silence->window = lsx_malloc(silence->window_size * sizeof(double));
 
         clear_rms(effp);
 
         /* Now that we know sample rate, reparse duration. */
         if (silence->start)
         {
-            if (sox_parsesamples(effp->ininfo.rate, silence->start_duration_str,
+            if (lsx_parsesamples(effp->in_signal.rate, silence->start_duration_str,
                                 &silence->start_duration, 's') == NULL)
-              return sox_usage(effp);
+              return lsx_usage(effp);
         }
         if (silence->stop)
         {
-            if (sox_parsesamples(effp->ininfo.rate,silence->stop_duration_str,
+            if (lsx_parsesamples(effp->in_signal.rate,silence->stop_duration_str,
                                 &silence->stop_duration,'s') == NULL)
-              return sox_usage(effp);
+              return lsx_usage(effp);
         }
 
         if (silence->start)
@@ -254,12 +251,12 @@ static int sox_silence_start(sox_effect_t * effp)
         else
             silence->mode = SILENCE_COPY;
 
-        silence->start_holdoff = (sox_sample_t *)xmalloc(sizeof(sox_sample_t)*silence->start_duration);
+        silence->start_holdoff = lsx_malloc(sizeof(sox_sample_t)*silence->start_duration);
         silence->start_holdoff_offset = 0;
         silence->start_holdoff_end = 0;
         silence->start_found_periods = 0;
 
-        silence->stop_holdoff = (sox_sample_t *)xmalloc(sizeof(sox_sample_t)*silence->stop_duration);
+        silence->stop_holdoff = lsx_malloc(sizeof(sox_sample_t)*silence->stop_duration);
         silence->stop_holdoff_offset = 0;
         silence->stop_holdoff_end = 0;
         silence->stop_found_periods = 0;
@@ -275,23 +272,23 @@ static int aboveThreshold(sox_effect_t * effp, sox_sample_t value, double thresh
 
     /* When scaling low bit data, noise values got scaled way up */
     /* Only consider the original bits when looking for silence */
-    switch(effp->ininfo.size)
+    switch(effp->in_signal.precision)
     {
-        case SOX_SIZE_BYTE:
+        case 8:
             value = SOX_SAMPLE_TO_SIGNED_8BIT(value, dummy_clipped_count);
             ratio = (double)abs(value) / (double)SOX_INT8_MAX;
             break;
-        case SOX_SIZE_16BIT:
+        case 16:
             value = SOX_SAMPLE_TO_SIGNED_16BIT(value, dummy_clipped_count);
             ratio = (double)abs(value) / (double)SOX_INT16_MAX;
             break;
-        case SOX_SIZE_24BIT:
+        case 24:
             value = SOX_SAMPLE_TO_SIGNED_24BIT(value, dummy_clipped_count);
             ratio = (double)abs(value) / (double)SOX_INT24_MAX;
             break;
-        case SOX_SIZE_32BIT:
+        case 32:
             value = SOX_SAMPLE_TO_SIGNED_32BIT(value,);
-            ratio = (double)labs(value) / (double)SOX_INT32_MAX;
+            ratio = (double)abs(value) / (double)SOX_INT32_MAX;
             break;
         default:
             ratio = 0;
@@ -308,7 +305,7 @@ static int aboveThreshold(sox_effect_t * effp, sox_sample_t value, double thresh
 
 static sox_sample_t compute_rms(sox_effect_t * effp, sox_sample_t sample)
 {
-    silence_t silence = (silence_t) effp->priv;
+    priv_t * silence = (priv_t *) effp->priv;
     double new_sum;
     sox_sample_t rms;
 
@@ -323,7 +320,7 @@ static sox_sample_t compute_rms(sox_effect_t * effp, sox_sample_t sample)
 
 static void update_rms(sox_effect_t * effp, sox_sample_t sample)
 {
-    silence_t silence = (silence_t) effp->priv;
+    priv_t * silence = (priv_t *) effp->priv;
 
     silence->rms_sum -= *silence->window_current;
     *silence->window_current = ((double)sample * (double)sample);
@@ -336,13 +333,13 @@ static void update_rms(sox_effect_t * effp, sox_sample_t sample)
 
 /* Process signed long samples from ibuf to obuf. */
 /* Return number of samples processed in isamp and osamp. */
-static int sox_silence_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf, 
-                    sox_size_t *isamp, sox_size_t *osamp)
+static int sox_silence_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf,
+                    size_t *isamp, size_t *osamp)
 {
-    silence_t silence = (silence_t) effp->priv;
+    priv_t * silence = (priv_t *) effp->priv;
     int threshold;
-    sox_size_t i, j;
-    sox_size_t nrOfTicks, nrOfInSamplesRead, nrOfOutSamplesWritten;
+    size_t i, j;
+    size_t nrOfTicks, nrOfInSamplesRead, nrOfOutSamplesWritten;
 
     nrOfInSamplesRead = 0;
     nrOfOutSamplesWritten = 0;
@@ -357,24 +354,24 @@ static int sox_silence_flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_s
              * prevent getting buffers out of sync.
              */
 silence_trim:
-            nrOfTicks = min((*isamp-nrOfInSamplesRead), 
-                            (*osamp-nrOfOutSamplesWritten)) / 
-                           effp->ininfo.channels;
+            nrOfTicks = min((*isamp-nrOfInSamplesRead),
+                            (*osamp-nrOfOutSamplesWritten)) /
+                           effp->in_signal.channels;
             for(i = 0; i < nrOfTicks; i++)
             {
                 threshold = 0;
-                for (j = 0; j < effp->ininfo.channels; j++)
+                for (j = 0; j < effp->in_signal.channels; j++)
                 {
                     threshold |= aboveThreshold(effp,
                                                 compute_rms(effp, ibuf[j]),
-                                                silence->start_threshold, 
+                                                silence->start_threshold,
                                                 silence->start_unit);
                 }
 
                 if (threshold)
                 {
                     /* Add to holdoff buffer */
-                    for (j = 0; j < effp->ininfo.channels; j++)
+                    for (j = 0; j < effp->in_signal.channels; j++)
                     {
                         update_rms(effp, *ibuf);
                         silence->start_holdoff[
@@ -401,12 +398,12 @@ silence_trim:
                 else /* !above Threshold */
                 {
                     silence->start_holdoff_end = 0;
-                    for (j = 0; j < effp->ininfo.channels; j++)
+                    for (j = 0; j < effp->in_signal.channels; j++)
                     {
                         update_rms(effp, ibuf[j]);
                     }
-                    ibuf += effp->ininfo.channels; 
-                    nrOfInSamplesRead += effp->ininfo.channels;
+                    ibuf += effp->in_signal.channels;
+                    nrOfInSamplesRead += effp->in_signal.channels;
                 }
             } /* for nrOfTicks */
             break;
@@ -414,8 +411,8 @@ silence_trim:
         case SILENCE_TRIM_FLUSH:
 silence_trim_flush:
             nrOfTicks = min((silence->start_holdoff_end -
-                             silence->start_holdoff_offset), 
-                             (*osamp-nrOfOutSamplesWritten)); 
+                             silence->start_holdoff_offset),
+                             (*osamp-nrOfOutSamplesWritten));
             for(i = 0; i < nrOfTicks; i++)
             {
                 *obuf++ = silence->start_holdoff[silence->start_holdoff_offset++];
@@ -442,9 +439,9 @@ silence_trim_flush:
              * Case A:
              *
              * Case 1a:
-             * If previous silence was detect then see if input sample is 
+             * If previous silence was detect then see if input sample is
              * above threshold.  If found then flush out hold off buffer
-             * and copy over to output buffer.  
+             * and copy over to output buffer.
              *
              * Case 1b:
              * If no previous silence detect then see if input sample
@@ -457,7 +454,7 @@ silence_trim_flush:
              * buffer.  Even though it wasn't put in output
              * buffer, inform user that input was consumed.
              *
-             * If hold off buffer is full after this then stop 
+             * If hold off buffer is full after this then stop
              * copying data and discard data in hold off buffer.
              *
              * Special leave_silence logic:
@@ -472,20 +469,20 @@ silence_trim_flush:
              *
              */
 silence_copy:
-            nrOfTicks = min((*isamp-nrOfInSamplesRead), 
-                            (*osamp-nrOfOutSamplesWritten)) / 
-                           effp->ininfo.channels;
+            nrOfTicks = min((*isamp-nrOfInSamplesRead),
+                            (*osamp-nrOfOutSamplesWritten)) /
+                           effp->in_signal.channels;
             if (silence->stop)
             {
                 /* Case A */
                 for(i = 0; i < nrOfTicks; i++)
                 {
                     threshold = 1;
-                    for (j = 0; j < effp->ininfo.channels; j++)
+                    for (j = 0; j < effp->in_signal.channels; j++)
                     {
-                        threshold &= aboveThreshold(effp, 
+                        threshold &= aboveThreshold(effp,
                                                     compute_rms(effp, ibuf[j]),
-                                                    silence->stop_threshold, 
+                                                    silence->stop_threshold,
                                                     silence->stop_unit);
                     }
 
@@ -510,7 +507,7 @@ silence_copy:
                     else if (threshold)
                     {
                         /* Not holding off so copy into output buffer */
-                        for (j = 0; j < effp->ininfo.channels; j++)
+                        for (j = 0; j < effp->in_signal.channels; j++)
                         {
                             update_rms(effp, *ibuf);
                             *obuf++ = *ibuf++;
@@ -522,7 +519,7 @@ silence_copy:
                     else if (!threshold)
                     {
                         /* Add to holdoff buffer */
-                        for (j = 0; j < effp->ininfo.channels; j++)
+                        for (j = 0; j < effp->in_signal.channels; j++)
                         {
                             update_rms(effp, *ibuf);
                             if (silence->leave_silence) {
@@ -534,15 +531,15 @@ silence_copy:
                             nrOfInSamplesRead++;
                         }
 
-                        /* Check if holdoff buffer is greater than duration 
+                        /* Check if holdoff buffer is greater than duration
                          */
-                        if (silence->stop_holdoff_end >= 
+                        if (silence->stop_holdoff_end >=
                                 silence->stop_duration)
                         {
                             /* Increment found counter and see if this
                              * is the last period.  If so then exit.
                              */
-                            if (++silence->stop_found_periods >= 
+                            if (++silence->stop_found_periods >=
                                     silence->stop_periods)
                             {
                                 silence->stop_holdoff_offset = 0;
@@ -569,7 +566,7 @@ silence_copy:
                             }
                             else
                             {
-                                /* Flush this buffer and start 
+                                /* Flush this buffer and start
                                  * looking again.
                                  */
                                 silence->mode = SILENCE_COPY_FLUSH;
@@ -584,16 +581,16 @@ silence_copy:
             {
                 /* Case B */
                 memcpy(obuf, ibuf, sizeof(sox_sample_t)*nrOfTicks*
-                                   effp->ininfo.channels);
-                nrOfInSamplesRead += (nrOfTicks*effp->ininfo.channels);
-                nrOfOutSamplesWritten += (nrOfTicks*effp->ininfo.channels);
+                                   effp->in_signal.channels);
+                nrOfInSamplesRead += (nrOfTicks*effp->in_signal.channels);
+                nrOfOutSamplesWritten += (nrOfTicks*effp->in_signal.channels);
             }
             break;
 
         case SILENCE_COPY_FLUSH:
 silence_copy_flush:
             nrOfTicks = min((silence->stop_holdoff_end -
-                                silence->stop_holdoff_offset), 
+                                silence->stop_holdoff_offset),
                             (*osamp-nrOfOutSamplesWritten));
 
             for(i = 0; i < nrOfTicks; i++)
@@ -623,19 +620,19 @@ silence_copy_flush:
         return (SOX_SUCCESS);
 }
 
-static int sox_silence_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t *osamp)
+static int sox_silence_drain(sox_effect_t * effp, sox_sample_t *obuf, size_t *osamp)
 {
-    silence_t silence = (silence_t) effp->priv;
-    sox_size_t i;
-    sox_size_t nrOfTicks, nrOfOutSamplesWritten = 0;
+    priv_t * silence = (priv_t *) effp->priv;
+    size_t i;
+    size_t nrOfTicks, nrOfOutSamplesWritten = 0;
 
     /* Only if in flush mode will there be possible samples to write
      * out during drain() call.
      */
-    if (silence->mode == SILENCE_COPY_FLUSH || 
+    if (silence->mode == SILENCE_COPY_FLUSH ||
         silence->mode == SILENCE_COPY)
     {
-        nrOfTicks = min((silence->stop_holdoff_end - 
+        nrOfTicks = min((silence->stop_holdoff_end -
                             silence->stop_holdoff_offset), *osamp);
         for(i = 0; i < nrOfTicks; i++)
         {
@@ -661,7 +658,7 @@ static int sox_silence_drain(sox_effect_t * effp, sox_sample_t *obuf, sox_size_t
 
 static int sox_silence_stop(sox_effect_t * effp)
 {
-    silence_t silence = (silence_t) effp->priv;
+    priv_t * silence = (priv_t *) effp->priv;
 
     free(silence->window);
     free(silence->start_holdoff);
@@ -671,7 +668,7 @@ static int sox_silence_stop(sox_effect_t * effp)
 
 static int kill(sox_effect_t * effp)
 {
-  silence_t silence = (silence_t) effp->priv;
+  priv_t * silence = (priv_t *) effp->priv;
 
   free(silence->start_duration_str);
   free(silence->stop_duration_str);
@@ -687,7 +684,7 @@ static sox_effect_handler_t sox_silence_effect = {
   sox_silence_flow,
   sox_silence_drain,
   sox_silence_stop,
-  kill
+  kill, sizeof(priv_t)
 };
 
 const sox_effect_handler_t *sox_silence_effect_fn(void)
