@@ -1,11 +1,10 @@
-/*
- * Effect: change tempo (alter duration, maintain pitch) with a WSOLA method.
- * Copyright (c) 2007 robs@users.sourceforge.net
+/* libSoX effect: change tempo (and duration) or pitch (maintain duration)
+ * Copyright (c) 2007,8 robs@users.sourceforge.net
  * Based on ideas from Olli Parviainen's SoundTouch Library.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 2.1 of the License, or (at
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -14,13 +13,13 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this library.  If not, write to the Free Software Foundation,
- * Fifth Floor, 51 Franklin Street, Boston, MA 02111-1301, USA.
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "sox_i.h"
 #include "fifo.h"
-#include "xmalloc.h"
+#include "getopt.h"
 #include <math.h>
 
 typedef struct {
@@ -46,7 +45,7 @@ typedef struct {
   size_t skip_total;
 } tempo_t;
 
-/* For the Waveform Similarity part of WSOLA */
+/* Waveform Similarity by least squares; works across multi-channels */
 static float difference(const float * a, const float * b, size_t length)
 {
   float diff = 0;
@@ -86,7 +85,6 @@ static size_t tempo_best_overlap_position(tempo_t * t, float const * new_win)
   return best_pos;
 }
 
-/* For the Over-Lap part of WSOLA */
 static void tempo_overlap(
     tempo_t * t, const float * in1, const float * in2, float * output)
 {
@@ -104,22 +102,22 @@ static void tempo_overlap(
 static void tempo_process(tempo_t * t)
 {
   while (fifo_occupancy(&t->input_fifo) >= t->process_size) {
-    size_t skip, offset = 0;
+    size_t skip, offset;
 
     /* Copy or overlap the first bit to the output */
-    if (!t->segments_total)
-      fifo_write(&t->output_fifo, t->overlap, fifo_read_ptr(&t->input_fifo));
-    else {
+    if (!t->segments_total) {
+      offset = t->search / 2;
+      fifo_write(&t->output_fifo, t->overlap, (float *) fifo_read_ptr(&t->input_fifo) + t->channels * offset);
+    } else {
       offset = tempo_best_overlap_position(t, fifo_read_ptr(&t->input_fifo));
       tempo_overlap(t, t->overlap_buf,
           (float *) fifo_read_ptr(&t->input_fifo) + t->channels * offset,
           fifo_write(&t->output_fifo, t->overlap, NULL));
     }
     /* Copy the middle bit to the output */
-    if (t->segment > 2 * t->overlap)
-      fifo_write(&t->output_fifo, t->segment - 2 * t->overlap,
-                 (float *) fifo_read_ptr(&t->input_fifo) +
-                 t->channels * (offset + t->overlap));
+    fifo_write(&t->output_fifo, t->segment - 2 * t->overlap,
+               (float *) fifo_read_ptr(&t->input_fifo) +
+               t->channels * (offset + t->overlap));
 
     /* Copy the end bit to overlap_buf ready to be mixed with
      * the beginning of the next segment. */
@@ -128,9 +126,8 @@ static void tempo_process(tempo_t * t)
            t->channels * (offset + t->segment - t->overlap),
            t->channels * t->overlap * sizeof(*(t->overlap_buf)));
 
-    /* The Advance part of WSOLA */
+    /* Advance through the input stream */
     skip = t->factor * (++t->segments_total * (t->segment - t->overlap)) + 0.5;
-    skip -= (t->search + 1) >> 1; /* So search straddles nominal skip point. */
     t->skip_total += skip -= t->skip_total;
     fifo_read(&t->input_fifo, skip, NULL);
   }
@@ -153,14 +150,14 @@ static void tempo_flush(tempo_t * t)
 {
   size_t samples_out = t->samples_in / t->factor + .5;
   size_t remaining = samples_out - t->samples_out;
-  float * buff = xcalloc(128 * t->channels, sizeof(*buff));
+  float * buff = lsx_calloc(128 * t->channels, sizeof(*buff));
 
   if ((int)remaining > 0) {
     while (fifo_occupancy(&t->output_fifo) < remaining) {
-      tempo_input(t, buff, 128);
+      tempo_input(t, buff, (size_t) 128);
       tempo_process(t);
     }
-    fifo_trim(&t->output_fifo, remaining);
+    fifo_trim_to(&t->output_fifo, remaining);
     t->samples_in = 0;
   }
   free(buff);
@@ -177,9 +174,12 @@ static void tempo_setup(tempo_t * t,
   t->search  = sample_rate * search_ms / 1000 + .5;
   t->overlap = max(sample_rate * overlap_ms / 1000 + 4.5, 16);
   t->overlap &= ~7; /* Make divisible by 8 for loop optimisation */
-  t->overlap_buf = xmalloc(t->overlap * t->channels * sizeof(*t->overlap_buf));
+  if (t->overlap * 2 > t->segment)
+    t->overlap -= 8;
+  t->overlap_buf = lsx_malloc(t->overlap * t->channels * sizeof(*t->overlap_buf));
   max_skip = ceil(factor * (t->segment - t->overlap));
   t->process_size = max(max_skip + t->overlap, t->segment) + t->search;
+  memset(fifo_reserve(&t->input_fifo, t->search / 2), 0, (t->search / 2) * t->channels * sizeof(float));
 }
 
 static void tempo_delete(tempo_t * t)
@@ -192,7 +192,7 @@ static void tempo_delete(tempo_t * t)
 
 static tempo_t * tempo_create(size_t channels)
 {
-  tempo_t * t = xcalloc(1, sizeof(*t));
+  tempo_t * t = lsx_calloc(1, sizeof(*t));
   t->channels = channels;
   fifo_create(&t->input_fifo, t->channels * sizeof(float));
   fifo_create(&t->output_fifo, t->channels * sizeof(float));
@@ -201,80 +201,97 @@ static tempo_t * tempo_create(size_t channels)
 
 /*------------------------------- SoX Wrapper --------------------------------*/
 
-typedef struct tempo {
+typedef struct {
   tempo_t     * tempo;
   sox_bool    quick_search;
   double      factor, segment_ms, search_ms, overlap_ms;
 } priv_t;
 
-assert_static(sizeof(struct tempo) <= SOX_MAX_EFFECT_PRIVSIZE,
-              /* else */ tempo_PRIVSIZE_too_big);
-
 static int getopts(sox_effect_t * effp, int argc, char **argv)
 {
-  priv_t * p = (priv_t *) effp->priv;
+  priv_t * p = (priv_t *)effp->priv;
+  enum {Default, Music, Speech, Linear} profile = Default;
+  static const double segments_ms [] = {   82,82,  35  , 20};
+  static const double segments_pow[] = {    0, 1, .33  , 1};
+  static const double overlaps_div[] = {6.833, 7,  2.5 , 2};
+  static const double searches_div[] = {5.587, 6,  2.14, 2};
+  int c;
 
-  p->segment_ms = 82; /* Set non-zero defaults: */
-  p->search_ms  = 14;
-  p->overlap_ms = 12;
-
-  p->quick_search = argc && !strcmp(*argv, "-q") && (--argc, ++argv, sox_true);
+  p->segment_ms = p->search_ms = p->overlap_ms = HUGE_VAL;
+  while ((c = getopt(argc, argv, "+qmls")) != -1) switch (c) {
+    case 'q': p->quick_search  = sox_true;   break;
+    case 'm': profile = Music; break;
+    case 's': profile = Speech; break;
+    case 'l': profile = Linear; p->search_ms = 0; break;
+    default: lsx_fail("unknown option `-%c'", optopt); return lsx_usage(effp);
+  }
+  argc -= optind, argv += optind;
   do {                    /* break-able block */
-    NUMERIC_PARAMETER(factor      ,0.25, 4  )
+    NUMERIC_PARAMETER(factor      ,0.1 , 100 )
     NUMERIC_PARAMETER(segment_ms  , 10 , 120)
     NUMERIC_PARAMETER(search_ms   , 0  , 30 )
     NUMERIC_PARAMETER(overlap_ms  , 0  , 30 )
   } while (0);
 
-  return argc || !p->factor || p->overlap_ms + p->search_ms >= p->segment_ms ?
-    sox_usage(effp) : SOX_SUCCESS;
+  if (p->segment_ms == HUGE_VAL)
+    p->segment_ms = max(10, segments_ms[profile] / max(pow(p->factor, segments_pow[profile]), 1));
+  if (p->overlap_ms == HUGE_VAL)
+    p->overlap_ms = p->segment_ms / overlaps_div[profile];
+  if (p->search_ms == HUGE_VAL)
+    p->search_ms = p->segment_ms / searches_div[profile];
+
+  p->overlap_ms = min(p->overlap_ms, p->segment_ms / 2);
+  lsx_report("quick_search=%u factor=%g segment=%g search=%g overlap=%g",
+    p->quick_search, p->factor, p->segment_ms, p->search_ms, p->overlap_ms);
+  return argc? lsx_usage(effp) : SOX_SUCCESS;
 }
 
 static int start(sox_effect_t * effp)
 {
-  priv_t * p = (priv_t *) effp->priv;
-
+  priv_t * p = (priv_t *)effp->priv;
   if (p->factor == 1)
     return SOX_EFF_NULL;
 
-  p->tempo = tempo_create(effp->ininfo.channels);
-  tempo_setup(p->tempo, effp->ininfo.rate, p->quick_search, p->factor,
+  p->tempo = tempo_create((size_t)effp->in_signal.channels);
+  tempo_setup(p->tempo, effp->in_signal.rate, p->quick_search, p->factor,
       p->segment_ms, p->search_ms, p->overlap_ms);
   return SOX_SUCCESS;
 }
 
 static int flow(sox_effect_t * effp, const sox_sample_t * ibuf,
-                sox_sample_t * obuf, sox_size_t * isamp, sox_size_t * osamp)
+                sox_sample_t * obuf, size_t * isamp, size_t * osamp)
 {
-  priv_t * p = (priv_t *) effp->priv;
-  sox_size_t i, odone = *osamp /= effp->ininfo.channels;
+  priv_t * p = (priv_t *)effp->priv;
+  size_t i, odone = *osamp /= effp->in_signal.channels;
   float const * s = tempo_output(p->tempo, NULL, &odone);
 
-  for (i = 0; i < odone * effp->ininfo.channels; ++i)
+  for (i = 0; i < odone * effp->in_signal.channels; ++i)
     *obuf++ = SOX_FLOAT_32BIT_TO_SAMPLE(*s++, effp->clips);
 
   if (*isamp && odone < *osamp) {
-    float * t = tempo_input(p->tempo, NULL, *isamp / effp->ininfo.channels);
+    float * t = tempo_input(p->tempo, NULL, *isamp / effp->in_signal.channels);
     for (i = *isamp; i; --i)
       *t++ = SOX_SAMPLE_TO_FLOAT_32BIT(*ibuf++, effp->clips);
     tempo_process(p->tempo);
   }
   else *isamp = 0;
 
-  *osamp = odone * effp->ininfo.channels;
+  *osamp = odone * effp->in_signal.channels;
   return SOX_SUCCESS;
 }
 
-static int drain(sox_effect_t * effp, sox_sample_t * obuf, sox_size_t * osamp)
+static int drain(sox_effect_t * effp, sox_sample_t * obuf, size_t * osamp)
 {
-  static sox_size_t isamp = 0;
-  tempo_flush(((priv_t *)effp->priv)->tempo);
+  priv_t * p = (priv_t *)effp->priv;
+  static size_t isamp = 0;
+  tempo_flush(p->tempo);
   return flow(effp, 0, obuf, &isamp, osamp);
 }
 
 static int stop(sox_effect_t * effp)
 {
-  tempo_delete(((priv_t *)effp->priv)->tempo);
+  priv_t * p = (priv_t *)effp->priv;
+  tempo_delete(p->tempo);
   return SOX_SUCCESS;
 }
 
@@ -282,7 +299,60 @@ sox_effect_handler_t const * sox_tempo_effect_fn(void)
 {
   static sox_effect_handler_t handler = {
     "tempo", "[-q] factor [segment-ms [search-ms [overlap-ms]]]",
-    SOX_EFF_MCHAN | SOX_EFF_LENGTH, getopts, start, flow, drain, stop, NULL
+    SOX_EFF_MCHAN | SOX_EFF_LENGTH | SOX_EFF_GETOPT,
+    getopts, start, flow, drain, stop, NULL, sizeof(priv_t)
   };
   return &handler;
 }
+
+/*---------------------------------- pitch -----------------------------------*/
+
+static int pitch_getopts(sox_effect_t * effp, int argc, char **argv)
+{
+  double d;
+  char dummy, arg[100], **argv2 = malloc(argc * sizeof(*argv2));
+  int result, pos = (argc > 1 && !strcmp(argv[1], "-q"))? 2 : 1;
+
+  if (argc <= pos || sscanf(argv[pos], "%lf %c", &d, &dummy) != 1)
+    return lsx_usage(effp);
+
+  d = pow(2., d / 1200);  /* cents --> factor */
+  sprintf(arg, "%g", 1 / d);
+  memcpy(argv2, argv, argc * sizeof(*argv2));
+  argv2[pos] = arg;
+  result = getopts(effp, argc, argv2);
+  free(argv2);
+  return result;
+}
+
+static int pitch_start(sox_effect_t * effp)
+{
+  priv_t * p = (priv_t *) effp->priv;
+  int result = start(effp);
+
+  effp->out_signal.rate = effp->in_signal.rate / p->factor;
+  return result;
+}
+
+sox_effect_handler_t const * sox_pitch_effect_fn(void)
+{
+  static sox_effect_handler_t handler;
+  handler = *sox_tempo_effect_fn();
+  handler.name = "pitch";
+  handler.usage = "[-q] shift-in-cents [segment-ms [search-ms [overlap-ms]]]",
+  handler.getopts = pitch_getopts;
+  handler.start = pitch_start;
+  handler.flags &= ~SOX_EFF_LENGTH;
+  return &handler;
+}
+
+/*------------------------- key (old name for pitch) -------------------------*/
+
+sox_effect_handler_t const * sox_key_effect_fn(void)
+{
+  static sox_effect_handler_t handler;
+  handler = *sox_pitch_effect_fn();
+  handler.name = "key";
+  handler.flags |= SOX_EFF_DEPRECATED;
+  return &handler;
+}
