@@ -1,7 +1,8 @@
-/*
+/* libSoX effect: tremolo  (c) 2007 robs@users.sourceforge.net
+ *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 2.1 of the License, or (at
  * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -10,30 +11,28 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this library.  If not, write to the Free Software Foundation,
- * Fifth Floor, 51 Franklin Street, Boston, MA 02111-1301, USA.
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/* Effect: tremolo  (c) 2007 robs@users.sourceforge.net */
-
 #include "sox_i.h"
 
-static int getopts(sox_effect_t * effp, int n, char * * argv) 
+static int getopts(sox_effect_t * effp, int n, char * * argv)
 {
   double speed, depth = 40;
   char dummy;     /* To check for extraneous chars. */
   char offset[100];
-  char * args[] = {"sine", "fmod", 0, 0};
+  char * args[] = {"sine", "fmod", 0, 0, "25"};
 
   if (n < 1 || n > 2 ||
       sscanf(argv[0], "%lf %c", &speed, &dummy) != 1 || speed < 0 ||
       (n > 1 && sscanf(argv[1], "%lf %c", &depth, &dummy) != 1) ||
       depth <= 0 || depth > 100)
-    return sox_usage(effp);
+    return lsx_usage(effp);
   args[2] = argv[0];
   sprintf(offset, "%g", 100 - depth / 2);
   args[3] = offset;
-  return sox_synth_effect_fn()->getopts(effp, array_length(args), args);
+  return sox_synth_effect_fn()->getopts(effp, (int)array_length(args), args);
 }
 
 sox_effect_handler_t const * sox_tremolo_effect_fn(void)
