@@ -1,9 +1,9 @@
-/* Yamaha TX-16W sampler file support
+/* libSoX Yamaha TX-16W sampler file support
  *
  * May 20, 1993
  * Copyright 1993 Rob Talley   (rob@aii.com)
  * This source code is freely redistributable and may be used for
- * any purpose. This copyright notice and the following copyright 
+ * any purpose. This copyright notice and the following copyright
  * notice must be maintained intact. No warranty whatsoever is
  * provided. This code is furnished AS-IS as a component of the
  * larger work Copyright 1991 Lance Norskog and Sundry Contributors.
@@ -33,16 +33,20 @@
  *
  */
 
-#define TXMAXLEN 0x3FF80
-
+#include "sox_i.h"
 #include <stdio.h>
 #include <string.h>
-#include "sox_i.h"
+
+#define TXMAXLEN 0x3FF80
 
 /* Private data for TX16 file */
-typedef struct txwstuff {
-        sox_size_t rest;                 /* bytes remaining in sample file */
-} *txw_t;
+typedef struct {
+  size_t   samples_out;
+  size_t   bytes_out;
+  size_t   rest;                 /* bytes remaining in sample file */
+  sox_sample_t odd;
+  sox_bool     odd_flag;
+} priv_t;
 
 struct WaveHeader_ {
   char filetype[6]; /* = "LM8953", */
@@ -54,75 +58,66 @@ struct WaveHeader_ {
     atc_length[3],   /* I'll get to this... */
     rpt_length[3],
     unused[2];       /* set these to null, to be on the safe side */
-} ;
+};
 
 static const unsigned char magic1[4] = {0, 0x06, 0x10, 0xF6};
 static const unsigned char magic2[4] = {0, 0x52, 0x00, 0x52};
 
-/* SJB: dangerous static variables */
-static sox_size_t tx16w_len=0;
-static sox_size_t writedone=0;
-
 /*
  * Do anything required before you start reading samples.
- * Read file header. 
- *      Find out sampling rate, 
+ * Read file header.
+ *      Find out sampling rate,
  *      size and encoding of samples,
  *      mono/stereo/quad.
  */
-static int sox_txwstartread(sox_format_t * ft)
+static int startread(sox_format_t * ft)
 {
     int c;
     char filetype[7];
-    char format;
-    char sample_rate;
-    sox_size_t num_samp_bytes = 0;
-    char gunk[8];
+    int8_t format;
+    unsigned char sample_rate;
+    size_t num_samp_bytes = 0;
+    unsigned char gunk[8];
     int blewIt;
-    uint32_t trash;
+    uint8_t trash;
 
-    txw_t sk = (txw_t) ft->priv;
+    priv_t * sk = (priv_t *) ft->priv;
     /* If you need to seek around the input file. */
     if (! ft->seekable)
     {
-        sox_fail_errno(ft,SOX_EOF,"txw input file must be a file, not a pipe");
+        lsx_fail_errno(ft,SOX_EOF,"txw input file must be a file, not a pipe");
         return(SOX_EOF);
     }
 
     /* This is dumb but portable, just count the bytes til EOF */
-    while (sox_readb(ft, (unsigned char *)&trash) != SOX_EOF)
-        num_samp_bytes++; 
+    while (lsx_read_b_buf(ft, &trash, (size_t) 1) == 1)
+        num_samp_bytes++;
     num_samp_bytes -= 32;         /* calculate num samples by sub header size */
-    sox_seeki(ft, 0, 0);           /* rewind file */
+    lsx_seeki(ft, (off_t)0, 0);   /* rewind file */
     sk->rest = num_samp_bytes;    /* set how many sample bytes to read */
 
     /* first 6 bytes are file type ID LM8953 */
-    sox_readb(ft, (unsigned char *)&filetype[0]);
-    sox_readb(ft, (unsigned char *)&filetype[1]);
-    sox_readb(ft, (unsigned char *)&filetype[2]);
-    sox_readb(ft, (unsigned char *)&filetype[3]);
-    sox_readb(ft, (unsigned char *)&filetype[4]);
-    sox_readb(ft, (unsigned char *)&filetype[5]);
+    lsx_readchars(ft, filetype, sizeof(filetype) - 1);
     filetype[6] = '\0';
     for( c = 16; c > 0 ; c-- )    /* Discard next 16 bytes */
-        sox_readb(ft, (unsigned char *)&trash);
-    sox_readb(ft, (unsigned char *)&format);
-    sox_readb(ft, (unsigned char *)&sample_rate);
+        lsx_readb(ft, &trash);
+    lsx_readsb(ft, &format);
+    lsx_readb(ft, &sample_rate);
     /*
      * save next 8 bytes - if sample rate is 0, then we need
      *  to look at gunk[2] and gunk[5] to get real rate
      */
     for( c = 0; c < 8; c++ )
-        sox_readb(ft, (unsigned char *)&(gunk[c]));
+        lsx_readb(ft, &(gunk[c]));
     /*
-     * We should now be pointing at start of raw sample data in file 
+     * We should now be pointing at start of raw sample data in file
      */
 
     /* Check to make sure we got a good filetype ID from file */
-    sox_debug("Found header filetype %s",filetype);
+    lsx_debug("Found header filetype %s",filetype);
     if(strcmp(filetype,"LM8953"))
     {
-        sox_fail_errno(ft,SOX_EHDR,"Invalid filetype ID in input file header, != LM8953");
+        lsx_fail_errno(ft,SOX_EHDR,"Invalid filetype ID in input file header, != LM8953");
         return(SOX_EOF);
     }
     /*
@@ -131,13 +126,13 @@ static int sox_txwstartread(sox_format_t * ft)
 
     switch( sample_rate ) {
         case 1:
-            ft->signal.rate = 33333;
+            ft->signal.rate = 1e5 / 3;
             break;
         case 2:
-            ft->signal.rate = 50000;
+            ft->signal.rate = 1e5 / 2;
             break;
         case 3:
-            ft->signal.rate = 16667;
+            ft->signal.rate = 1e5 / 6;
             break;
         default:
             blewIt = 1;
@@ -145,32 +140,32 @@ static int sox_txwstartread(sox_format_t * ft)
                 case 0x06:
                     if ( (gunk[5] & 0xFE) == 0x52 ) {
                         blewIt = 0;
-                        ft->signal.rate = 33333;
+                        ft->signal.rate = 1e5 / 3;
                     }
                     break;
                 case 0x10:
                     if ( (gunk[5] & 0xFE) == 0x00 ) {
                         blewIt = 0;
-                        ft->signal.rate = 50000;
+                        ft->signal.rate = 1e5 / 2;
                     }
                     break;
                 case 0xF6:
                     if ( (gunk[5] & 0xFE) == 0x52 ) {
                         blewIt = 0;
-                        ft->signal.rate = 16667;
+                        ft->signal.rate = 1e5 / 6;
                     }
                     break;
             }
             if ( blewIt ) {
-                sox_debug("Invalid sample rate identifier found %d", (int)sample_rate);
-                ft->signal.rate = 33333;
+                lsx_debug("Invalid sample rate identifier found %d", sample_rate);
+                ft->signal.rate = 1e5 / 3;
             }
     }
-    sox_debug("Sample rate = %g", ft->signal.rate);
+    lsx_debug("Sample rate = %g", ft->signal.rate);
 
     ft->signal.channels = 1 ; /* not sure about stereo sample data yet ??? */
-    ft->signal.size = SOX_SIZE_16BIT; /* this is close enough */
-    ft->signal.encoding = SOX_ENCODING_SIGN2;
+    ft->encoding.bits_per_sample = 12;
+    ft->encoding.encoding = SOX_ENCODING_SIGN2;
 
     return(SOX_SUCCESS);
 }
@@ -182,10 +177,10 @@ static int sox_txwstartread(sox_format_t * ft)
  * Return number of samples read.
  */
 
-static sox_size_t sox_txwread(sox_format_t * ft, sox_sample_t *buf, sox_size_t len)
+static size_t read_samples(sox_format_t * ft, sox_sample_t *buf, size_t len)
 {
-    txw_t sk = (txw_t) ft->priv;
-    sox_size_t done = 0;
+    priv_t * sk = (priv_t *) ft->priv;
+    size_t done = 0;
     unsigned char uc1,uc2,uc3;
     unsigned short s1,s2;
 
@@ -204,7 +199,7 @@ static sox_size_t sox_txwread(sox_format_t * ft, sox_sample_t *buf, sox_size_t l
     /*
      * This is ugly but it's readable!
      * Read three bytes from stream, then decompose these into
-     * two unsigned short samples. 
+     * two unsigned short samples.
      * TCC 3.0 appeared to do unwanted things, so we really specify
      *  exactly what we want to happen.
      * Convert unsigned short to sox_sample_t then shift up the result
@@ -215,9 +210,9 @@ static sox_size_t sox_txwread(sox_format_t * ft, sox_sample_t *buf, sox_size_t l
      */
     for(done = 0; done < len; ) {
         if(sk->rest < 3) break; /* Finished reading from file? */
-        sox_readb(ft, &uc1);
-        sox_readb(ft, &uc2);
-        sox_readb(ft, &uc3);
+        lsx_readb(ft, &uc1);
+        lsx_readb(ft, &uc2);
+        lsx_readb(ft, &uc3);
         sk->rest -= 3; /* adjust remaining for bytes we just read */
         s1 = (unsigned short) (uc1 << 4) | (((uc2 >> 4) & 017));
         s2 = (unsigned short) (uc3 << 4) | (( uc2 & 017 ));
@@ -232,73 +227,82 @@ static sox_size_t sox_txwread(sox_format_t * ft, sox_sample_t *buf, sox_size_t l
     return done;
 }
 
-static int sox_txwstartwrite(sox_format_t * ft)
+static int startwrite(sox_format_t * ft)
 {
+  priv_t * sk = (priv_t *) ft->priv;
     struct WaveHeader_ WH;
 
-    sox_debug("tx16w selected output");
+    lsx_debug("tx16w selected output");
 
     memset(&WH, 0, sizeof(struct WaveHeader_));
 
-    if (ft->signal.channels != 1)
-        sox_report("tx16w is overriding output format to 1 channel.");
-    ft->signal.channels = 1 ; /* not sure about stereo sample data yet ??? */
-    if (ft->signal.size != SOX_SIZE_16BIT || ft->signal.encoding != SOX_ENCODING_SIGN2)
-        sox_report("tx16w is overriding output format to size Signed Word format.");
-    ft->signal.size = SOX_SIZE_16BIT; /* this is close enough */
-    ft->signal.encoding = SOX_ENCODING_SIGN2;
-
     /* If you have to seek around the output file */
     if (! ft->seekable)
     {
-        sox_fail_errno(ft,SOX_EOF,"Output .txw file must be a file, not a pipe");
+        lsx_fail_errno(ft,SOX_EOF,"Output .txw file must be a file, not a pipe");
         return(SOX_EOF);
     }
 
     /* dummy numbers, just for place holder, real header is written
        at end of processing, since byte count is needed */
 
-    sox_writebuf(ft, &WH, 32);
-    writedone = 32;
+    lsx_writebuf(ft, &WH, (size_t) 32);
+    sk->bytes_out = 32;
     return(SOX_SUCCESS);
 }
 
-static sox_size_t sox_txwwrite(sox_format_t * ft, const sox_sample_t *buf, sox_size_t len)
+static size_t write_samples(sox_format_t * ft, const sox_sample_t *buf, size_t len0)
 {
-    sox_size_t i;
-    sox_sample_t w1,w2;
+  priv_t * sk = (priv_t *) ft->priv;
+  size_t last_i, i = 0, len = min(len0, TXMAXLEN - sk->samples_out);
+  sox_sample_t w1, w2;
 
-    tx16w_len += len;
-    if (tx16w_len > TXMAXLEN) return 0;
-
-    for (i=0;i<len;i+=2) {
-        w1 =  *buf++ >> 20;
-        if (i+1==len)
-            w2 = 0;
-        else {
-            w2 =  *buf++ >> 20;
-        }
-        sox_writeb(ft, (w1 >> 4) & 0xFF);
-        sox_writeb(ft, (((w1 & 0x0F) << 4) | (w2 & 0x0F)) & 0xFF);
-        sox_writeb(ft, (w2 >> 4) & 0xFF);
-        writedone += 3;
+  while (i < len) {
+    last_i = i;
+    if (sk->odd_flag) {
+      w1 = sk->odd;
+      sk->odd_flag = sox_false;
     }
-    return(len);
+    else w1 = *buf++ >> 20, ++i;
+
+    if (i < len) {
+      w2 = *buf++ >> 20, ++i;
+      if (lsx_writesb(ft, (w1 >> 4) & 0xFF) ||
+          lsx_writesb(ft, (((w1 & 0x0F) << 4) | (w2 & 0x0F)) & 0xFF) ||
+          lsx_writesb(ft, (w2 >> 4) & 0xFF)) {
+        i = last_i;
+        break;
+      }
+      sk->samples_out += 2;
+      sk->bytes_out += 3;
+    }
+    else {
+      sk->odd = w1;
+      sk->odd_flag = sox_true;
+    }
+  }
+  return i;
 }
 
-static int sox_txwstopwrite(sox_format_t * ft)
+static int stopwrite(sox_format_t * ft)
 {
+  priv_t * sk = (priv_t *) ft->priv;
     struct WaveHeader_ WH;
     int AttackLength, LoopLength, i;
 
+    if (sk->odd_flag) {
+      sox_sample_t pad = 0;
+      write_samples(ft, &pad, (size_t) 1);
+    }
+
     /* All samples are already written out. */
     /* If file header needs fixing up, for example it needs the */
     /* the number of samples in a field, seek back and write them here. */
 
-    sox_debug("tx16w:output finished");
+    lsx_debug("tx16w:output finished");
 
     memset(&WH, 0, sizeof(struct WaveHeader_));
-    strncpy(WH.filetype,"LM8953",6);
+    strncpy(WH.filetype,"LM8953",(size_t)6);
     for (i=0;i<10;i++) WH.nulls[i]=0;
     for (i=0;i<6;i++)  WH.dummy_aeg[i]=0;
     for (i=0;i<2;i++)  WH.unused[i]=0;
@@ -307,44 +311,44 @@ static int sox_txwstopwrite(sox_format_t * ft)
 
     WH.format = 0xC9;   /* loop off */
 
-    /* the actual sample rate is not that important ! */  
+    /* the actual sample rate is not that important ! */
     if (ft->signal.rate < 24000)      WH.sample_rate = 3;
     else if (ft->signal.rate < 41000) WH.sample_rate = 1;
     else                            WH.sample_rate = 2;
 
-    if (tx16w_len >= TXMAXLEN) {
-        sox_warn("Sound too large for TX16W. Truncating, Loop Off");
+    if (sk->samples_out >= TXMAXLEN) {
+        lsx_warn("Sound too large for TX16W. Truncating, Loop Off");
         AttackLength       = TXMAXLEN/2;
         LoopLength         = TXMAXLEN/2;
     }
-    else if (tx16w_len >=TXMAXLEN/2) {
+    else if (sk->samples_out >=TXMAXLEN/2) {
         AttackLength       = TXMAXLEN/2;
-        LoopLength         = tx16w_len - TXMAXLEN/2;
+        LoopLength         = sk->samples_out - TXMAXLEN/2;
         if (LoopLength < 0x40) {
             LoopLength   +=0x40;
             AttackLength -= 0x40;
         }
-    }    
-    else if (tx16w_len >= 0x80) {
-        AttackLength                       = tx16w_len -0x40;
+    }
+    else if (sk->samples_out >= 0x80) {
+        AttackLength                       = sk->samples_out -0x40;
         LoopLength                         = 0x40;
     }
     else {
         AttackLength                       = 0x40;
         LoopLength                         = 0x40;
-        for(i=tx16w_len;i<0x80;i++) {
-            sox_writeb(ft, 0);
-            sox_writeb(ft, 0);
-            sox_writeb(ft, 0);
-            writedone += 3;
+        for(i=sk->samples_out;i<0x80;i++) {
+            lsx_writeb(ft, 0);
+            lsx_writeb(ft, 0);
+            lsx_writeb(ft, 0);
+            sk->bytes_out += 3;
         }
     }
 
     /* Fill up to 256 byte blocks; the TX16W seems to like that */
 
-    while ((writedone % 0x100) != 0) {
-        sox_writeb(ft, 0);
-        writedone++;
+    while ((sk->bytes_out % 0x100) != 0) {
+        lsx_writeb(ft, 0);
+        sk->bytes_out++;
     }
 
     WH.atc_length[0] = 0xFF & AttackLength;
@@ -357,33 +361,22 @@ static int sox_txwstopwrite(sox_format_t * ft)
     WH.rpt_length[2] = (0x01 & (LoopLength >> 16)) +
         magic2[WH.sample_rate];
 
-    sox_rewind(ft);
-    sox_writebuf(ft, &WH, 32);
+    lsx_rewind(ft);
+    lsx_writebuf(ft, &WH, (size_t) 32);
 
     return(SOX_SUCCESS);
 }
 
-/* Yamaha TX16W and SY99 waves */
-static const char *txwnames[] = {
-  "txw",
-  NULL
-};
-
-static sox_format_handler_t sox_txw_format = {
-   txwnames,
-   0,
-   sox_txwstartread,
-   sox_txwread,
-   sox_format_nothing,
-   sox_txwstartwrite,
-   sox_txwwrite,
-   sox_txwstopwrite,
-   sox_format_nothing_seek
-};
-
-const sox_format_handler_t *sox_txw_format_fn(void);
-
-const sox_format_handler_t *sox_txw_format_fn(void)
+SOX_FORMAT_HANDLER(txw)
 {
-    return &sox_txw_format;
+  static char const * const names[] = {"txw", NULL};
+  static sox_rate_t   const write_rates[] = {1e5/6, 1e5/3, 1e5/2, 0};
+  static unsigned const write_encodings[] = {SOX_ENCODING_SIGN2, 12, 0, 0};
+  static sox_format_handler_t const handler = {SOX_LIB_VERSION_CODE,
+    "Yamaha TX-16W sampler", names, SOX_FILE_MONO,
+    startread, read_samples, NULL,
+    startwrite, write_samples, stopwrite,
+    NULL, write_encodings, write_rates, sizeof(priv_t)
+  };
+  return &handler;
 }
