@@ -17,6 +17,7 @@
  */
 
 #include "sox_i.h"
+#include "getopt.h"
 #include <ctype.h>
 #include <stdio.h>
 
@@ -36,9 +37,24 @@ int strncasecmp(char const * s1, char const * s2, size_t n)
 }
 #endif
 
-enum_item const * find_enum_text(char const * text, enum_item const * enum_items)
+char const * lsx_find_file_extension(char const * pathname)
 {
-  enum_item const * result = NULL; /* Assume not found */
+  /* First, chop off any path portions of filename.  This
+   * prevents the next search from considering that part. */
+  char const * result = LAST_SLASH(pathname);
+  if (!result)
+    result = pathname;
+
+  /* Now look for an filename extension */
+  result = strrchr(result, '.');
+  if (result)
+    ++result;
+  return result;
+}
+
+lsx_enum_item const * lsx_find_enum_text(char const * text, lsx_enum_item const * enum_items)
+{
+  lsx_enum_item const * result = NULL; /* Assume not found */
 
   while (enum_items->text) {
     if (strcasecmp(text, enum_items->text) == 0)
@@ -53,7 +69,7 @@ enum_item const * find_enum_text(char const * text, enum_item const * enum_items
   return result;
 }
 
-enum_item const * find_enum_value(unsigned value, enum_item const * enum_items)
+lsx_enum_item const * lsx_find_enum_value(unsigned value, lsx_enum_item const * enum_items)
 {
   for (;enum_items->text; ++enum_items)
     if (value == enum_items->value)
@@ -61,7 +77,25 @@ enum_item const * find_enum_value(unsigned value, enum_item const * enum_items)
   return NULL;
 }
 
-char const * sigfigs3(size_t number)
+int lsx_enum_option(int c, lsx_enum_item const * items)
+{
+  lsx_enum_item const * p = lsx_find_enum_text(optarg, items);
+  if (p == NULL) {
+    size_t len = 1;
+    char * set = lsx_malloc(len);
+    *set = 0;
+    for (p = items; p->text; ++p) {
+      set = lsx_realloc(set, len += 2 + strlen(p->text));
+      strcat(set, ", "); strcat(set, p->text);
+    }
+    lsx_fail("-%c: '%s' is not one of: %s.", c, optarg, set + 2);
+    free(set);
+    return INT_MAX;
+  }
+  return p->value;
+}
+
+char const * lsx_sigfigs3(size_t number)
 {
   static char string[16][10];
   static unsigned n;
@@ -77,7 +111,7 @@ char const * sigfigs3(size_t number)
   return string[n];
 }
 
-char const * sigfigs3p(double percentage)
+char const * lsx_sigfigs3p(double percentage)
 {
   static char string[16][10];
   static unsigned n;
