@@ -1,5 +1,4 @@
-/*
- * Copyright (c) 20/03/2000 Fabien COELHO <fabien@coelho.net>
+/* Copyright (c) 20/03/2000 Fabien COELHO <fabien@coelho.net>
  * Copyright (c) 2000-2007 SoX contributors
  *
  * SoX vol effect; change volume with basic linear amplitude formula.
@@ -15,9 +14,6 @@
   "\tis only used on peaks (to prevent clipping); default is no limiter."
 
 #include "sox_i.h"
-#include <math.h>
-
-#define LOG_10_20 ((double)(0.1151292546497022842009e0))
 
 typedef struct {
   double    gain; /* amplitude gain. */
@@ -26,22 +22,22 @@ typedef struct {
   double    limitergain;
   int       limited; /* number of limited values to report. */
   int       totalprocessed;
-} * vol_t;
+} priv_t;
 
 enum {vol_amplitude, vol_dB, vol_power};
 
-static enum_item const vol_types[] = {
-  ENUM_ITEM(vol_,amplitude)
-  ENUM_ITEM(vol_,dB)
-  ENUM_ITEM(vol_,power)
+static lsx_enum_item const vol_types[] = {
+  LSX_ENUM_ITEM(vol_,amplitude)
+  LSX_ENUM_ITEM(vol_,dB)
+  LSX_ENUM_ITEM(vol_,power)
   {0, 0}};
 
 /*
  * Process options: gain (float) type (amplitude, power, dB)
  */
-static int getopts(sox_effect_t * effp, int argc, char **argv) 
+static int getopts(sox_effect_t * effp, int argc, char **argv)
 {
-  vol_t     vol = (vol_t) effp->priv; 
+  priv_t *     vol = (priv_t *) effp->priv;
   char      type_string[11];
   char *    type_ptr = type_string;
   char      dummy;             /* To check for extraneous chars. */
@@ -49,10 +45,10 @@ static int getopts(sox_effect_t * effp, int argc, char **argv)
 
   vol->gain = 1;               /* Default is no change. */
   vol->uselimiter = sox_false; /* Default is no limiter. */
-  
+
   /* Get the vol, and the type if it's in the same arg. */
   if (!argc || (have_type = sscanf(argv[0], "%lf %10s %c", &vol->gain, type_string, &dummy) - 1) > 1)
-    return sox_usage(effp);
+    return lsx_usage(effp);
   ++argv, --argc;
 
   /* No type yet? Get it from the next arg: */
@@ -63,11 +59,11 @@ static int getopts(sox_effect_t * effp, int argc, char **argv)
   }
 
   if (have_type) {
-    enum_item const * p = find_enum_text(type_ptr, vol_types);
+    lsx_enum_item const * p = lsx_find_enum_text(type_ptr, vol_types);
     if (!p)
-      return sox_usage(effp);
+      return lsx_usage(effp);
     switch (p->value) {
-      case vol_dB: vol->gain = exp(vol->gain*LOG_10_20); break;
+      case vol_dB: vol->gain = dB_to_linear(vol->gain); break;
       case vol_power: /* power to amplitude, keep phase change */
         vol->gain = vol->gain > 0 ? sqrt(vol->gain) : -sqrt(-vol->gain);
         break;
@@ -76,18 +72,18 @@ static int getopts(sox_effect_t * effp, int argc, char **argv)
 
   if (argc) {
     if (fabs(vol->gain) < 1 || sscanf(*argv, "%lf %c", &vol->limitergain, &dummy) != 1 || vol->limitergain <= 0 || vol->limitergain >= 1)
-      return sox_usage(effp);
-    
+      return lsx_usage(effp);
+
     vol->uselimiter = sox_true;
-    /* The following equation is derived so that there is no 
+    /* The following equation is derived so that there is no
      * discontinuity in output amplitudes */
-    /* and a SOX_SAMPLE_MAX input always maps to a SOX_SAMPLE_MAX output 
+    /* and a SOX_SAMPLE_MAX input always maps to a SOX_SAMPLE_MAX output
      * when the limiter is activated. */
-    /* (NOTE: There **WILL** be a discontinuity in the slope 
+    /* (NOTE: There **WILL** be a discontinuity in the slope
      * of the output amplitudes when using the limiter.) */
     vol->limiterthreshhold = SOX_SAMPLE_MAX * (1.0 - vol->limitergain) / (fabs(vol->gain) - vol->limitergain);
   }
-  sox_debug("mult=%g limit=%g", vol->gain, vol->limitergain);
+  lsx_debug("mult=%g limit=%g", vol->gain, vol->limitergain);
   return SOX_SUCCESS;
 }
 
@@ -96,8 +92,8 @@ static int getopts(sox_effect_t * effp, int argc, char **argv)
  */
 static int start(sox_effect_t * effp)
 {
-    vol_t vol = (vol_t) effp->priv;
-    
+    priv_t * vol = (priv_t *) effp->priv;
+
     if (vol->gain == 1)
       return SOX_EFF_NULL;
 
@@ -110,28 +106,28 @@ static int start(sox_effect_t * effp)
 /*
  * Process data.
  */
-static int flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf, 
-                sox_size_t *isamp, sox_size_t *osamp)
+static int flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obuf,
+                size_t *isamp, size_t *osamp)
 {
-    vol_t vol = (vol_t) effp->priv;
+    priv_t * vol = (priv_t *) effp->priv;
     register double gain = vol->gain;
     register double limiterthreshhold = vol->limiterthreshhold;
     register double sample;
-    register sox_size_t len;
-    
+    register size_t len;
+
     len = min(*osamp, *isamp);
 
     /* report back dealt with amount. */
     *isamp = len; *osamp = len;
-    
+
     if (vol->uselimiter)
     {
         vol->totalprocessed += len;
-        
+
         for (;len>0; len--)
             {
                 sample = *ibuf++;
-                
+
                 if (sample > limiterthreshhold)
                 {
                         sample =  (SOX_SAMPLE_MAX - vol->limitergain * (SOX_SAMPLE_MAX - sample));
@@ -170,9 +166,9 @@ static int flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obu
 
 static int stop(sox_effect_t * effp)
 {
-  vol_t vol = (vol_t) effp->priv;
+  priv_t * vol = (priv_t *) effp->priv;
   if (vol->limited) {
-    sox_warn("limited %d values (%d percent).", 
+    lsx_warn("limited %d values (%d percent).",
          vol->limited, (int) (vol->limited * 100.0 / vol->totalprocessed));
   }
   return SOX_SUCCESS;
@@ -181,7 +177,27 @@ static int stop(sox_effect_t * effp)
 sox_effect_handler_t const * sox_vol_effect_fn(void)
 {
   static sox_effect_handler_t handler = {
-    "vol", vol_usage, SOX_EFF_MCHAN, getopts, start, flow, 0, stop, 0
+    "vol", vol_usage, SOX_EFF_MCHAN, getopts, start, flow, 0, stop, 0, sizeof(priv_t)
   };
   return &handler;
 }
+
+static int gain_getopts(sox_effect_t * effp, int argc, char * * argv)
+{
+  char * args[] = {0, "dB"};
+
+  if (argc != 1)
+    return lsx_usage(effp);
+  args[0] = argv[0];
+  return sox_vol_effect_fn()->getopts(effp, (int)array_length(args), args);
+}
+
+sox_effect_handler_t const * sox_gain_effect_fn(void)
+{
+  static sox_effect_handler_t handler;
+  handler = *sox_vol_effect_fn();
+  handler.name = "gain";
+  handler.usage = "dB-gain";
+  handler.getopts = gain_getopts;
+  return &handler;
+}
